/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainAtLeastOneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainAtLeastOneOfLogicalAndSpec.scala"
  
  object `a Array` {
    
    val fumArray: Array[String] = Array("fum")
    val toArray: Array[String] = Array("to")
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fum") and contain atLeastOneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fum") and contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and contain atLeastOneOf ("have", "a", "nice", "day"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("fum") and contain atLeastOneOf ("fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and (contain atLeastOneOf ("fum")))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\"") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and contain atLeastOneOf ("have", "a", "nice", "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and contain atLeastOneOf ("fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\"") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (contain atLeastOneOf ("fum") and contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        (fumArray should (contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM ") and contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf () and contain atLeastOneOf("fie", "fee", "fum", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf ("fie", "fee", "fum", "foe") and contain atLeastOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum") and contain atLeastOneOf("fie", "fee", "fum", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fum") and contain atLeastOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (equal (fumArray) and contain atLeastOneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) and contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (equal (fumArray) and contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (equal (fumArray) and contain atLeastOneOf ("have", "a", "nice", "day"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) and contain atLeastOneOf ("fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (equal (fumArray) and (contain atLeastOneOf ("fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (equal (toArray) and contain atLeastOneOf ("have", "a", "nice", "day"))) (decided by invertedArrayOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (toArray) and contain atLeastOneOf ("fum"))) (decided by invertedArrayOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) and contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedArrayOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) and contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (equal (fumArray) and contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (equal (fumArray) and contain atLeastOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (legacyEqual (fumArray) and contain atLeastOneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) and contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (legacyEqual (fumArray) and contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (legacyEqual (fumArray) and contain atLeastOneOf ("have", "a", "nice", "day"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) and contain atLeastOneOf ("fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (legacyEqual (fumArray) and (contain atLeastOneOf ("fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (legacyEqual (fumArray) and contain atLeastOneOf ("have", "a", "nice", "day"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (fumArray) and contain atLeastOneOf ("fum"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) and contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) and contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (legacyEqual (fumArray) and contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (legacyEqual (fumArray) and contain atLeastOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain atLeastOneOf("fie", "fee", "fum", "foe") and legacyEqual (fumArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fum") and legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and legacyEqual (fumArray))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (contain atLeastOneOf ("have", "a", "nice", "day") and legacyEqual (fumArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("fum") and legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("fum") and (legacyEqual (fumArray)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain atLeastOneOf ("have", "a", "nice", "day") and legacyEqual (fumArray))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain atLeastOneOf ("fum") and legacyEqual (fumArray))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you") and legacyEqual (toArray))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\"") + ", but " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM ") and legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf() and legacyEqual (fumArray))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf("fee", "fie", "foe", "fie", "fum") and legacyEqual (fumArray))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fum") and not contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not contain atLeastOneOf ("happy", "birthday", "to", "you") and not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\"") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (not contain atLeastOneOf ("fum") and not contain atLeastOneOf ("fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain atLeastOneOf ("happy", "birthday", "to", "you") and not contain atLeastOneOf ("fum"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not contain atLeastOneOf ("fum") and (not contain atLeastOneOf ("happy", "birthday", "to", "you")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\"") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain atLeastOneOf ("fum") and not contain atLeastOneOf ("fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain atLeastOneOf ("fum") and not contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"fum\"") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not contain atLeastOneOf ("happy", "birthday", "to", "you") and not contain atLeastOneOf ("fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        (fumArray should (contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM ") and contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atLeastOneOf () and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atLeastOneOf ("fie", "fee", "fuu", "foe") and not contain atLeastOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum") and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atLeastOneOf ("fie", "fee", "fuu", "foe") and not contain atLeastOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not equal (toArray) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) and not contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (toArray) and not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (not equal (toArray) and not contain atLeastOneOf ("fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) and not contain atLeastOneOf ("fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (toArray) and (not contain atLeastOneOf ("happy", "birthday", "to", "you")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not equal (fumArray) and not contain atLeastOneOf ("fum"))) (decided by invertedArrayOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (fumArray) and not contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedArrayOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) and not contain atLeastOneOf ("fum"))) (decided by invertedArrayOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (not contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUU ") and not contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not equal (toArray) and not contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not equal (toArray) and not contain atLeastOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not be (toArray) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) and not contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not be (toArray) and not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (not be (toArray) and not contain atLeastOneOf ("fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) and not contain atLeastOneOf ("fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not be (toArray) and (not contain atLeastOneOf ("happy", "birthday", "to", "you")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not be (toArray) and not contain atLeastOneOf ("fum"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (toArray) and not contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) and not contain atLeastOneOf ("fum"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        (fumArray should (not contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUU ") and not contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not be (toArray) and not contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not be (toArray) and not contain atLeastOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
  }
  
  object `collection of Arrays` {
    
    val list1s: Vector[Array[Int]] = Vector(Array(1), Array(1), Array(1))
    val lists: Vector[Array[Int]] = Vector(Array(1), Array(1), Array(2))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1), Array(1), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi"), Array("hi"), Array("hi"))
    val toArrays: Vector[Array[String]] = Vector(Array("to"), Array("to"), Array("to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf (3, 2, 1) and contain atLeastOneOf (1, 3, 4))
        atLeast (2, lists) should (contain atLeastOneOf (3, 1, 5) and contain atLeastOneOf (1, 3, 4))
        atMost (2, lists) should (contain atLeastOneOf (3, 2, 8) and contain atLeastOneOf (2, 3, 4))
        no (lists) should (contain atLeastOneOf (3, 6, 9) and contain atLeastOneOf (3, 4, 5))
        no (nils) should (contain atLeastOneOf (1, 2, 8) and contain atLeastOneOf (1, 3, 4))
        no (listsArray) should (contain atLeastOneOf(3, 8, 5) and contain atLeastOneOf (3, 4, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1, 6, 8) and contain atLeastOneOf (1, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2)) + " did not contain at least one of (1, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1, 2, 8) and contain atLeastOneOf (1, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(2)) + " contained at least one of (1, 2, 8), but " + decorateToStringValue(Array(2)) + " did not contain at least one of (1, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf ("hi", "hello") and contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array()) + " did not contain at least one of (\"hi\", \"hello\")", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (contain atLeastOneOf ("hi", "hello") and contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained at least one of (\"hi\", \"hello\"), but " + decorateToStringValue(Array("hi")) + " did not contain at least one of (\"ho\", \"hey\", \"howdy\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsArray) should (contain atLeastOneOf (1, 3, 4) and contain atLeastOneOf (1, 3, Array()))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(Array()) + " did not contain at least one of (1, 3, 4)", thisLineNumber - 2, listsArray), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1, 2, 8) and contain atLeastOneOf (1, 3, 4))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue(Array(2)) + " contained at least one of (1, 2, 8), but " + decorateToStringValue(Array(2)) + " did not contain at least one of (1, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (contain atLeastOneOf ("ho") and contain atLeastOneOf ("he"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain atLeastOneOf ("hi") and contain atLeastOneOf ("ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (contain atLeastOneOf ("ho") and contain atLeastOneOf ("hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\"), but " + decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain atLeastOneOf ("ho") and contain atLeastOneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain atLeastOneOf ("hi") and contain atLeastOneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (contain atLeastOneOf ("ho") and contain atLeastOneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\"), but " + decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf () and contain atLeastOneOf (1, 3, 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (1, 3, 4) and contain atLeastOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (3, 2, 2, 1) and contain atLeastOneOf (1, 3, 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (1, 3, 4) and contain atLeastOneOf (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Array(1)) and contain atLeastOneOf (1, 3, 4))
        atLeast (2, lists) should (be (Array(1)) and contain atLeastOneOf (1, 3, 4))
        atMost (2, lists) should (be (Array(1)) and contain atLeastOneOf (2, 3, 4))
        no (lists) should (be (Array(8)) and contain atLeastOneOf (3, 4, 5))
        no (nils) should (be (Array(8)) and contain atLeastOneOf (1, 3, 4))
        no (listsArray) should (be (Array(8)) and contain atLeastOneOf (3, 4, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (Array(1)) and contain atLeastOneOf (1, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2)) + " was not equal to " + decorateToStringValue(Array(1)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (Array(1)) and contain atLeastOneOf (2, 3, 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array(1)) + " was equal to " + decorateToStringValue(Array(1)) + ", but " + decorateToStringValue(Array(1)) + " did not contain at least one of (2, 3, 8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (Array("hey")) and contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array()) + " was not equal to " + decorateToStringValue(Array("hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("hi")) and contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")) + ", but " + decorateToStringValue(Array("hi")) + " did not contain at least one of (\"ho\", \"hey\", \"howdy\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsArray) should (be (Array(1)) and contain atLeastOneOf (1, 3, Array()))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(Array()) + " was not equal to " + decorateToStringValue(Array(1)), thisLineNumber - 2, listsArray), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (Array(1)) and contain atLeastOneOf (2, 3, 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(Array(1)) + " was equal to " + decorateToStringValue(Array(1)) + ", but " + decorateToStringValue(Array(1)) + " did not contain at least one of (2, 3, 8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (be (Array("hi")) and contain atLeastOneOf ("he"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("ho")) and contain atLeastOneOf ("ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("hi")) and contain atLeastOneOf ("hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")) + ", but " + decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (be (Array("hi")) and contain atLeastOneOf ("ho"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("ho")) and contain atLeastOneOf ("ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("hi")) and contain atLeastOneOf ("hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")) + ", but " + decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (Array(1)) and contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (Array(1)) and contain atLeastOneOf (3, 2, 2, 1))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atLeastOneOf (3, 2, 8) and not contain atLeastOneOf (8, 3, 4))
        atLeast (2, lists) should (not contain atLeastOneOf (3, 8, 5) and not contain atLeastOneOf (8, 3, 4))
        atMost (2, lists) should (not contain atLeastOneOf (3, 6, 8) and contain atLeastOneOf (5, 3, 4))
        no (lists) should (not contain atLeastOneOf (1, 2, 9) and not contain atLeastOneOf (2, 1, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (2, 6, 8) and not contain atLeastOneOf (2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2)) + " contained at least one of (2, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (3, 6, 8) and not contain atLeastOneOf (2, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(2)) + " did not contain at least one of (3, 6, 8), but " + decorateToStringValue(Array(2)) + " contained at least one of (2, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiArrays) should (not contain atLeastOneOf ("hi", "hello") and not contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained at least one of (\"hi\", \"hello\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (not contain atLeastOneOf ("ho", "hey", "howdy") and not contain atLeastOneOf ("hi", "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain at least one of (\"ho\", \"hey\", \"howdy\"), but " + decorateToStringValue(Array("hi")) + " contained at least one of (\"hi\", \"hello\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (not contain atLeastOneOf ("hi") and not contain atLeastOneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain atLeastOneOf ("ho") and not contain atLeastOneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (not contain atLeastOneOf ("hi") and not contain atLeastOneOf ("ho"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\"), but " + decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not contain atLeastOneOf ("hi") and not contain atLeastOneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain atLeastOneOf ("ho") and not contain atLeastOneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain atLeastOneOf ("hi") and not contain atLeastOneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain at least one of (\"hi\"), but " + decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf () and not contain atLeastOneOf (8, 3, 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf (8, 3, 4) and not contain atLeastOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf (3, 2, 2, 1) and not contain atLeastOneOf (8, 3, 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf (8, 3, 4) and not contain atLeastOneOf (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Array(2)) and not contain atLeastOneOf (8, 3, 4))
        atLeast (2, lists) should (not be (Array(3)) and not contain atLeastOneOf (8, 3, 4))
        atMost (2, lists) should (not be (Array(3)) and contain atLeastOneOf (5, 3, 4))
        no (list1s) should (not be (Array(1)) and not contain atLeastOneOf (2, 1, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (Array(2)) and not contain atLeastOneOf (2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2)) + " was equal to " + decorateToStringValue(Array(2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (Array(3)) and not contain atLeastOneOf (2, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(2)) + " was not equal to " + decorateToStringValue(Array(3)) + ", but " + decorateToStringValue(Array(2)) + " contained at least one of (2, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi")) and not contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("ho")) and not contain atLeastOneOf ("hi", "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("hi")) + " contained at least one of (\"hi\", \"hello\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (not be (Array("ho")) and not contain atLeastOneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi")) and not contain atLeastOneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("ho")) and not contain atLeastOneOf ("ho"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not be (Array("ho")) and not contain atLeastOneOf ("hi"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi")) and not contain atLeastOneOf ("hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("ho")) and not contain atLeastOneOf ("ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("hi")) + " contained at least one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (Array(2)) and not contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (Array(2)) and not contain atLeastOneOf (3, 2, 2, 1))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
  }
}
