/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainAtLeastOneOfSpec extends Spec with Matchers {

  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  //ADDITIONAL//

  object `a Array` {

    val fumArray: Array[String] = Array("fum")
    val toArray: Array[String] = Array("to")

    object `when used with contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should contain atLeastOneOf ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          fumArray should contain atLeastOneOf ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should contain atLeastOneOf ("happy", "birthday", "to", "you")
        intercept[TestFailedException] {
          fumArray should contain atLeastOneOf ("fum")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should contain atLeastOneOf ("happy", "birthday", "to", "you")) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (fumArray should contain atLeastOneOf ("fum")) (decided by invertedStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM ")
        }
        (fumArray should contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM ")) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should contain atLeastOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

    object `when used with (contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you"))
        intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf ("fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should (contain atLeastOneOf ("happy", "birthday", "to", "you"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (fumArray should (contain atLeastOneOf ("fum"))) (decided by invertedStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should (contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM "))
        }
        (fumArray should (contain atLeastOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

/*
 I purposely don't want to support this syntax:

        fumArray should contain (atLeastOneOf ("fee", "fie", "foe", "fum"))
        fumArray should (contain (atLeastOneOf ("fee", "fie", "foe", "fum")))

 Reason is that I don't want people putting parentheses between contain and atLeastOneOf, etc. This will not compile.
*/
    object `when used with not contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should not contain atLeastOneOf ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          toArray should not contain atLeastOneOf ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtLeastOneOf", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        toArray should not contain atLeastOneOf ("to")
        intercept[TestFailedException] {
          toArray should not contain atLeastOneOf ("fee", "fie", "foe", "fum")
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should not contain atLeastOneOf ("to")) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (toArray should not contain atLeastOneOf ("fee", "fie", "foe", "fum")) (decided by invertedStringEquality)
        }
        toArray should not contain atLeastOneOf (" TO ")
        intercept[TestFailedException] {
          (toArray should not contain atLeastOneOf (" TO ")) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should not contain atLeastOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toArray should not contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

/*
Interesting, of these three, the top one does happen to compile and run:

        toArray should not contain (atLeastOneOf ("fee", "fie", "foe", "fum"))
        // toArray should not (contain (atLeastOneOf ("fee", "fie", "foe", "fum")))
        // toArray should (not (contain (atLeastOneOf ("fee", "fie", "foe", "fum"))))

The bottom two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          toArray should (not contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtLeastOneOf", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        toArray should (not contain atLeastOneOf ("to"))
        intercept[TestFailedException] {
          toArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should (not contain atLeastOneOf ("to"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (toArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fum"))) (decided by invertedStringEquality)
        }
        toArray should (not contain atLeastOneOf (" TO "))
        intercept[TestFailedException] {
          (toArray should (not contain atLeastOneOf (" TO "))) (after being lowerCased and trimmed)
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toArray should (not contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
  }

  object `a collection of Arrays` {

    val list1s: Vector[Array[Int]] = Vector(Array(1), Array(1), Array(1))
    val lists: Vector[Array[Int]] = Vector(Array(1), Array(1), Array(2))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1), Array(1), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi"), Array("hi"), Array("hi"))
    val toArrays: Vector[Array[String]] = Vector(Array("to"), Array("to"), Array("to"))

    object `when used with contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atLeastOneOf (1, 3, 4)
        atLeast (2, lists) should contain atLeastOneOf (1, 3, 4)
        atMost (2, lists) should contain atLeastOneOf (2, 3, 4)
        no (lists) should contain atLeastOneOf (3, 4, 5)
        no (nils) should contain atLeastOneOf (1, 3, 4)
        no (listsArray) should contain atLeastOneOf (3, 4, 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atLeastOneOf (1, 3, 4)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array(2)) + " did not contain at least one of (1, 3, 4) (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should contain atLeastOneOf ("ho", "hey", "howdy")
        }
        e2.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Array()) + " did not contain at least one of (\"ho\", \"hey\", \"howdy\") (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsArray) should contain atLeastOneOf (1, 3, 4)
        }
        e4.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array()) + " did not contain at least one of (1, 3, 4) (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsArray)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should contain atLeastOneOf ("hi")
        intercept[TestFailedException] {
          all (hiArrays) should contain atLeastOneOf ("ho")
        }
        implicit val ise = invertedStringEquality
        all (hiArrays) should contain atLeastOneOf ("ho")
        intercept[TestFailedException] {
          all (hiArrays) should contain atLeastOneOf ("hi")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should contain atLeastOneOf ("ho")) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should contain atLeastOneOf ("hi")) (decided by invertedStringEquality)
        }
        implicit val ise = invertedStringEquality
        (all (hiArrays) should contain atLeastOneOf ("hi")) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should contain atLeastOneOf ("ho")) (decided by defaultEquality[String])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (hiArrays) should contain atLeastOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atLeastOneOf (1, 3, 3, 4)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

    object `when used with (contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf (1, 3, 4))
        atLeast (2, lists) should (contain atLeastOneOf (1, 3, 4))
        atMost (2, lists) should (contain atLeastOneOf (2, 3, 4))
        no (lists) should (contain atLeastOneOf (3, 4, 5))
        no (nils) should (contain atLeastOneOf (1, 3, 4))
        no (listsArray) should (contain atLeastOneOf (3, 4, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1, 3, 4))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array(2)) + " did not contain at least one of (1, 3, 4) (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf ("ho", "hey", "howdy"))
        }
        e2.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Array()) + " did not contain at least one of (\"ho\", \"hey\", \"howdy\") (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsArray) should (contain atLeastOneOf (1, 3, 4))
        }
        e4.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array()) + " did not contain at least one of (1, 3, 4) (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsArray)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should (contain atLeastOneOf ("hi"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain atLeastOneOf ("ho"))
        }
        implicit val ise = invertedStringEquality
        all (hiArrays) should (contain atLeastOneOf ("ho"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain atLeastOneOf ("hi"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain atLeastOneOf ("ho"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should (contain atLeastOneOf ("hi"))) (decided by invertedStringEquality)
        }
        implicit val ise = invertedStringEquality
        (all (hiArrays) should (contain atLeastOneOf ("hi"))) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should (contain atLeastOneOf ("ho"))) (decided by defaultEquality[String])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (hiArrays) should (contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (1, 3, 3, 4))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

/*
 I purposely don't want to support this syntax:

scala> all (list1s) should contain (atLeastOneOf (1, 3, 4))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should contain (atLeastOneOf (1, 3, 4))
                                             ^

scala> all (list1s) should (contain (atLeastOneOf (1, 3, 4)))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should (contain (atLeastOneOf (1, 3, 4)))
                                              ^

 Reason is that I don't want people putting parentheses between contain and atLeastOneOf, etc. This will not compile.
*/
    object `when used with not contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should not contain atLeastOneOf ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          all (toArrays) should not contain atLeastOneOf ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Array("to")) + " contained at least one of (\"happy\", \"birthday\", \"to\", \"you\") (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        all (toArrays) should not contain atLeastOneOf ("to")
        intercept[TestFailedException] {
          all (toArrays) should not contain atLeastOneOf ("fee", "fie", "foe", "fum")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should not contain atLeastOneOf ("to")) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should not contain atLeastOneOf ("fee", "fie", "foe", "fum")) (decided by invertedStringEquality)
        }
        all (toArrays) should not contain atLeastOneOf (" TO ")
        intercept[TestFailedException] {
          (all (toArrays) should not contain atLeastOneOf (" TO ")) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (hiArrays) should not contain atLeastOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should not contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

/*
Interesting, of these three, the last one does happen to compile and run:

scala> all (toArrays) should (not (contain (atLeastOneOf ("fee", "fie", "foe", "fum"))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (toArrays) should (not (contain (atLeastOneOf ("fee", "fie", "foe", "fum"))))
                                                    ^

scala> all (toArrays) should not (contain (atLeastOneOf ("fee", "fie", "foe", "fum")))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (toArrays) should not (contain (atLeastOneOf ("fee", "fie", "foe", "fum")))
                                                   ^

scala> all (toArrays) should not contain (atLeastOneOf ("fee", "fie", "foe", "fum"))

The top two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should (not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          all (toArrays) should (not contain atLeastOneOf ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Array("to")) + " contained at least one of (\"happy\", \"birthday\", \"to\", \"you\") (ArrayShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        all (toArrays) should (not contain atLeastOneOf ("to"))
        intercept[TestFailedException] {
          all (toArrays) should (not contain atLeastOneOf ("fee", "fie", "foe", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should (not contain atLeastOneOf ("to"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should (not contain atLeastOneOf ("fee", "fie", "foe", "fum"))) (decided by invertedStringEquality)
        }
        all (toArrays) should (not contain atLeastOneOf (" TO "))
        intercept[TestFailedException] {
          (all (toArrays) should (not contain atLeastOneOf (" TO "))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (hiArrays) should (not contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should (not contain atLeastOneOf ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
  }
}
