/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainAtMostOneOfLogicalOrSpec extends FreeSpec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainAtMostOneOfLogicalOrSpec.scala"
  
  "a Array" - {
    
    val fumArray: Array[String] = Array("fum", "foe")
    val toArray: Array[String] = Array("to", "you")
    
      "when used with (contain atMostOneOf (...) or contain atMostOneOf (...)) syntax" - {
      
        "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fam") or contain atMostOneOf("fie", "fee", "fam", "foe"))
        fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fam") or contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fum") or contain atMostOneOf("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fum") or contain atMostOneOf ("fie", "fee", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FAM") or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))
        fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FAM") or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or (contain atMostOneOf ("FIE", "FEE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FAM") or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FAM") or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumArray should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FAM ") or contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf () or contain atMostOneOf("fie", "fee", "fam", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf ("fie", "fee", "fam", "foe") or contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fie", "fum") or contain atMostOneOf("fie", "fee", "fam", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf ("fie", "fee", "fam", "foe") or contain atMostOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (equal (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (equal (fumArray) or contain atMostOneOf("fie", "fee", "fam", "foe"))
        fumArray should (equal (toArray) or contain atMostOneOf("fie", "fee", "fam", "foe"))
        fumArray should (equal (fumArray) or contain atMostOneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or contain atMostOneOf ("fie", "fee", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))
        fumArray should (equal (toArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))
        fumArray should (equal (fumArray) or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or (contain atMostOneOf ("FIE", "FEE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (equal (toArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (fumArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (toArray) or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) or contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (equal (fumArray) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (equal (fumArray) or contain atMostOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (legacyEqual (...) and contain theMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (legacyEqual (fumArray) or contain atMostOneOf("fie", "fee", "fam", "foe"))
        fumArray should (legacyEqual (toArray) or contain atMostOneOf("fie", "fee", "fam", "foe"))
        fumArray should (legacyEqual (fumArray) or contain atMostOneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or contain atMostOneOf ("fie", "fee", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))
        fumArray should (legacyEqual (toArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))
        fumArray should (legacyEqual (fumArray) or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or (contain atMostOneOf ("FIE", "FEE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (legacyEqual (fumArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (toArray) or contain atMostOneOf ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (fumArray) or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) or contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) or contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (legacyEqual (fumArray) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (legacyEqual (fumArray) or contain atMostOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    "when used with (contain oneOf (...) and legacyEqual (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (contain atMostOneOf("fie", "fee", "fam", "foe") or legacyEqual (fumArray))
        fumArray should (contain atMostOneOf("fie", "fee", "fum", "foe") or legacyEqual (fumArray))
        fumArray should (contain atMostOneOf("fie", "fee", "fam", "foe") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fum") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain atMostOneOf ("FIE", "FEE", "FAM", "FOE") or legacyEqual (fumArray))
        fumArray should (contain atMostOneOf ("FIE", "FEE", "FUM", "FOE") or legacyEqual (fumArray))
        fumArray should (contain atMostOneOf ("FIE", "FEE", "FAM", "FOE") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (contain atMostOneOf ("FIE", "FEE", "FAM", "FOE") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain atMostOneOf ("FIE", "FEE", "FUM", "FOE") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain atMostOneOf ("FIE", "FEE", "FAM", "FOE") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FaM ") or legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf() or legacyEqual (fumArray))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf("fee", "fie", "foe", "fie", "fum") or legacyEqual (fumArray))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain atMostOneOf (...) and not contain atMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fum") or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fum") or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fuu\"") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU") or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or not contain atMostOneOf ("FIE", "FEE", "FUU", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU") or not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\"") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\""), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU") or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM") or not contain atMostOneOf ("FIE", "FEE", "FUU", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU") or not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\"") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\""), fileName, thisLineNumber - 2)
        (fumArray should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FAM ") or contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atMostOneOf () or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atMostOneOf ("fie", "fee", "fum", "foe") or not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fie", "fum") or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atMostOneOf ("fie", "fee", "fum", "foe") or not contain atMostOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not equal (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (not equal (toArray) or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (not equal (fumArray) or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (not equal (toArray) or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (not equal (fumArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (not equal (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUU", "FOE"))
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\""), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (not equal (fumArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (fumArray) or not contain atMostOneOf ("FIE", "FEE", "FUU", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) or not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\""), fileName, thisLineNumber - 2)
        (fumArray should (not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM ") or not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not equal (toArray) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not equal (toArray) or not contain atMostOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (not be (toArray) or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (not be (fumArray) or not contain atMostOneOf("fie", "fee", "fum", "foe"))
        fumArray should (not be (toArray) or not contain atMostOneOf("fee", "fie", "foe", "fuu"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (not be (fumArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))
        fumArray should (not be (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUU", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\""), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (not be (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        (fumArray should (not be (fumArray) or not contain atMostOneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        (fumArray should (not be (toArray) or not contain atMostOneOf ("FIE", "FEE", "FUU", "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) or not contain atMostOneOf ("FEE", "FIE", "FOE", "FUU"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUU\""), fileName, thisLineNumber - 2)
        (fumArray should (not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM ") or not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not be (toArray) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not be (toArray) or not contain atMostOneOf("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  "collection of Arrays" - {
    
    val list1s: Vector[Array[Int]] = Vector(Array(1, 2), Array(1, 2), Array(1, 2))
    val lists: Vector[Array[Int]] = Vector(Array(1, 2), Array(1, 2), Array(2, 3))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1), Array(1), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "he"), Array("hi", "he"), Array("hi", "he"))
    val toArrays: Vector[Array[String]] = Vector(Array("to", "you"), Array("to", "you"), Array("to", "you"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain oneOf (..) and contain oneOf (..)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain atMostOneOf (1, 6, 8) or contain atMostOneOf (1, 3, 4))
        all (list1s) should (contain atMostOneOf (1, 2, 3) or contain atMostOneOf (1, 3, 4))
        all (list1s) should (contain atMostOneOf (1, 6, 8) or contain atMostOneOf (1, 2, 3))
        
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) or contain atMostOneOf (2, 3, 4))
        atLeast (2, lists) should (contain atMostOneOf (1, 2, 3) or contain atMostOneOf (2, 3, 4))
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) or contain atMostOneOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (2, 3, 4) or contain atMostOneOf (4, 3, 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of (2, 3, 4), and " + decorateToStringValue(lists(2)) + " did not contain at most one of (4, 3, 2)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (contain atMostOneOf ("HI", "HO") or contain atMostOneOf ("HO", "HI"))
        all (hiArrays) should (contain atMostOneOf ("HI", "HE") or contain atMostOneOf ("HO", "HI"))
        all (hiArrays) should (contain atMostOneOf ("HI", "HO") or contain atMostOneOf ("HI", "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain atMostOneOf ("HI", "HE") or contain atMostOneOf ("HE", "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " did not contain at most one of (\"HI\", \"HE\"), and " + decorateToStringValue(hiArrays(0)) + " did not contain at most one of (\"HE\", \"HI\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (contain atMostOneOf ("HI", "HO") or contain atMostOneOf ("HO", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain atMostOneOf ("HI", "HE") or contain atMostOneOf ("HO", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain atMostOneOf ("HI", "HO") or contain atMostOneOf ("HI", "HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain atMostOneOf ("HI", "HE") or contain atMostOneOf ("HE", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " did not contain at most one of (\"HI\", \"HE\"), and " + decorateToStringValue(hiArrays(0)) + " did not contain at most one of (\"HE\", \"HI\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () or contain atMostOneOf (1, 3, 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1, 3, 4) or contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1, 2, 2, 3) or contain atMostOneOf (1, 3, 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1, 3, 4) or contain atMostOneOf (1, 2, 2, 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (be (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (Array(1, 2)) or contain atMostOneOf (1, 6, 8))
        all (list1s) should (be (Array(2, 3)) or contain atMostOneOf (1, 6, 8))
        all (list1s) should (be (Array(1, 2)) or contain atMostOneOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (Array(2, 3)) or contain atMostOneOf (1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was not equal to " + decorateToStringValue(Array(2, 3)) + ", and " + decorateToStringValue(list1s(0)) + " did not contain at most one of (1, 2, 3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (be (Array("hi", "he")) or contain atMostOneOf ("HI", "HO"))
        all (hiArrays) should (be (Array("HO")) or contain atMostOneOf ("HI", "HO"))
        all (hiArrays) should (be (Array("hi", "he")) or contain atMostOneOf ("HI", "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("HO")) or contain atMostOneOf ("HI", "HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " was not equal to " + decorateToStringValue(Array("HO")) + ", and " + decorateToStringValue(hiArrays(0)) + " did not contain at most one of (\"HI\", \"HE\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (be (Array("hi", "he")) or contain atMostOneOf ("HI", "HO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("HO")) or contain atMostOneOf ("HI", "HO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("hi", "he")) or contain atMostOneOf ("HI", "HE"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("HO")) or contain atMostOneOf ("HI", "HE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " was not equal to " + decorateToStringValue(Array("HO")) + ", and " + decorateToStringValue(hiArrays(0)) + " did not contain at most one of (\"HI\", \"HE\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (Array(1, 2)) or contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (Array(1, 2)) or contain atMostOneOf (1, 2, 2, 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain oneOf (..) and not contain oneOf (..))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain atMostOneOf (3, 2, 1) or not contain atMostOneOf (1, 2, 3))
        all (list1s) should (not contain atMostOneOf (1, 2, 8) or not contain atMostOneOf (1, 2, 3))
        all (list1s) should (not contain atMostOneOf (3, 2, 1) or not contain atMostOneOf (1, 2, 8))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (1, 2, 8) or not contain atMostOneOf (8, 2, 1))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of (1, 2, 8), and " + decorateToStringValue(lists(2)) + " contained at most one of (8, 2, 1)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not contain atMostOneOf ("HI", "HE") or not contain atMostOneOf ("HE", "HI"))
        all (hiArrays) should (not contain atMostOneOf ("hi") or not contain atMostOneOf ("HE", "HI"))
        all (hiArrays) should (not contain atMostOneOf ("HI", "HE") or not contain atMostOneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain atMostOneOf ("hi") or not contain atMostOneOf ("he"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " contained at most one of (\"hi\"), and " + decorateToStringValue(hiArrays(0)) + " contained at most one of (\"he\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (not contain atMostOneOf ("HI", "HE") or not contain atMostOneOf ("HE", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain atMostOneOf ("hi") or not contain atMostOneOf ("HE", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain atMostOneOf ("HI", "HE") or not contain atMostOneOf ("hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain atMostOneOf ("hi") or not contain atMostOneOf ("he"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " contained at most one of (\"hi\"), and " + decorateToStringValue(hiArrays(0)) + " contained at most one of (\"he\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () or not contain atMostOneOf (1, 2, 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1, 2, 3) or not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1, 2, 2, 3) or not contain atMostOneOf (1, 2, 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1, 2, 3) or not contain atMostOneOf (1, 2, 2, 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain atMostOneOf (...))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (Array(2, 3)) or not contain atMostOneOf (1, 2, 3))
        all (list1s) should (not be (Array(1, 2)) or not contain atMostOneOf (1, 2, 3))
        all (list1s) should (not be (Array(2, 3)) or not contain atMostOneOf (2, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (Array(1, 2)) or not contain atMostOneOf (2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(Array(1, 2)) + ", and " + decorateToStringValue(list1s(0)) + " contained at most one of (2, 3, 4)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not be (Array("ho")) or not contain atMostOneOf ("HI", "HE"))
        all (hiArrays) should (not be (Array("hi", "he")) or not contain atMostOneOf ("HE", "HI"))
        all (hiArrays) should (not be (Array("ho")) or not contain atMostOneOf ("HI", "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi", "he")) or not contain atMostOneOf ("HI", "HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " was equal to " + decorateToStringValue(Array("hi", "he")) + ", and " + decorateToStringValue(hiArrays(0)) + " contained at most one of (\"HI\", \"HO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (not be (Array("ho")) or not contain atMostOneOf ("HI", "HE"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hi", "he")) or not contain atMostOneOf ("HE", "HI"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("ho")) or not contain atMostOneOf ("HI", "HE"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi", "he")) or not contain atMostOneOf ("HI", "HO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiArrays(0)) + " was equal to " + decorateToStringValue(Array("hi", "he")) + ", and " + decorateToStringValue(hiArrays(0)) + " contained at most one of (\"HI\", \"HO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (Array(2, 3)) or not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (Array(2, 3)) or not contain atMostOneOf (1, 2, 2, 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
