/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainAtMostOneOfSpec extends Spec with Matchers {

  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  //ADDITIONAL//

  object `a Array` {

    val fumArray: Array[String] = Array("fum", "foe")
    val toArray: Array[String] = Array("to", "you")

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should contain atMostOneOf ("fee", "fie", "foe", "fam")
        val e1 = intercept[TestFailedException] {
          fumArray should contain atMostOneOf ("fee", "fie", "foe", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumArray should contain atMostOneOf ("FEE", "FIE", "FOE", "FAM")
        intercept[TestFailedException] {
          fumArray should contain atMostOneOf ("FEE", "FIE", "FOE", "FUM")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should contain atMostOneOf ("FEE", "FIE", "FOE", "FAM")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should contain atMostOneOf ("FEE", "FIE", "FOE", "FUM")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumArray should contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM ")) (after being lowerCased and trimmed)
          
        }
        fumArray should contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM ")
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should contain atMostOneOf ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fam"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FAM"))
        intercept[TestFailedException] {
          fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FAM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should (contain atMostOneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumArray should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
        }
        fumArray should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM "))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain atMostOneOf ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should not contain atMostOneOf ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          fumArray should not contain atMostOneOf ("fee", "fie", "foe", "fam")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fam\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumArray should not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM")
        intercept[TestFailedException] {
          fumArray should not contain atMostOneOf ("FEE", "FIE", "FOE", "FAM")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should not contain atMostOneOf ("FEE", "FIE", "FOE", "FAM")) (decided by upperCaseStringEquality)
        }
        (fumArray should not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumArray should not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM ")
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should not contain atMostOneOf ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fam"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fam\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM"))
        intercept[TestFailedException] {
          fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FAM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FAM"))) (decided by upperCaseStringEquality)
        }
        (fumArray should (not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumArray should (not contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FUM "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (not contain atMostOneOf ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }

  object `a collection of Arrays` {

    val list1s: Vector[Array[Int]] = Vector(Array(1, 2), Array(1, 2), Array(1, 2))
    val lists: Vector[Array[Int]] = Vector(Array(1, 2), Array(1, 2), Array(2, 3))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1, 2), Array(1, 2), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "he"), Array("hi", "he"), Array("hi", "he"))
    val toArrays: Vector[Array[String]] = Vector(Array("to", "you"), Array("to", "you"), Array("to", "you"))

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atMostOneOf (1, 3, 4)
        atLeast (2, lists) should contain atMostOneOf (2, 3, 4)
        atMost (2, lists) should contain atMostOneOf (2, 3, 4)
        no (lists) should contain atMostOneOf (1, 2, 3)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atMostOneOf (2, 3, 4)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of (2, 3, 4) (ArrayShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should contain atMostOneOf ("hi", "ho")
        intercept[TestFailedException] {
          all (hiArrays) should contain atMostOneOf ("hi", "he")
        }
        implicit val use = upperCaseStringEquality
        all (hiArrays) should contain atMostOneOf ("HI", "HO")
        intercept[TestFailedException] {
          all (hiArrays) should contain atMostOneOf ("HI", "HE")
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should contain atMostOneOf ("hi", "ho")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should contain atMostOneOf ("hi", "he")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should contain atMostOneOf ("hi", "HE")) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should contain atMostOneOf ("hi", "he")) (decided by defaultEquality[String])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf (1, 2, 2, 3)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf (1, 3, 4))
        atLeast (2, lists) should (contain atMostOneOf (2, 3, 4))
        atMost (2, lists) should (contain atMostOneOf (2, 3, 4))
        no (lists) should (contain atMostOneOf (1, 2, 3))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (2, 3, 4))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of (2, 3, 4) (ArrayShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should (contain atMostOneOf ("hi", "ho"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain atMostOneOf ("hi", "he"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should (contain atMostOneOf ("HI", "HO"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain atMostOneOf ("HI", "HE"))
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain atMostOneOf ("hi", "ho"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should (contain atMostOneOf ("hi", "he"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should (contain atMostOneOf ("hi", "HE"))) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should (contain atMostOneOf ("hi", "he"))) (decided by defaultEquality[String])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1, 2, 2, 3))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should not contain atMostOneOf ("happy", "birthday", "to", "you")
        val e1 = intercept[TestFailedException] {
          all (toArrays) should not contain atMostOneOf ("fee", "fie", "foe", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toArrays(0)) + " contained at most one of (\"fee\", \"fie\", \"foe\", \"fum\") (ArrayShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should not contain atMostOneOf ("HAPPY", "BIRTHDAY", "TO", "YOU")
        intercept[TestFailedException] {
          all (toArrays) should not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM")
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (toArrays) should not contain atMostOneOf ("HAPPY", "BIRTHDAY", "TO", "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM")) (decided by upperCaseStringEquality)
        }
        (all (toArrays) should not contain atMostOneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toArrays) should not contain atMostOneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should not contain atMostOneOf ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should (not contain atMostOneOf ("happy", "birthday", "to", "you"))
        val e1 = intercept[TestFailedException] {
          all (toArrays) should (not contain atMostOneOf ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toArrays(0)) + " contained at most one of (\"fee\", \"fie\", \"foe\", \"fum\") (ArrayShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should (not contain atMostOneOf ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        intercept[TestFailedException] {
          all (toArrays) should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should (not contain atMostOneOf ("HAPPY", "BIRTHDAY", "TO", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should (not contain atMostOneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        (all (toArrays) should (not contain atMostOneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toArrays) should (not contain atMostOneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should (not contain atMostOneOf ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
