/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class ArrayShouldContainInOrderSpec extends Spec with Matchers {

  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }

  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  //ADDITIONAL//

  object `a Array` {

    val fumArray: Array[String] = Array("fex", "fum", "fum", "foe", "fie", "fie", "fie", "fee", "fee")
    val toArray: Array[String] = Array("happy", "happy", "happy", "birthday", "to", "you", "too")

    object `when used with contain inOrder (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should contain inOrder ("fum", "foe", "fie", "fee")
        val e1 = intercept[TestFailedException] {
          fumArray should contain inOrder ("fee", "fie", "foe", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should contain inOrder ("FUM", "FOE", "FIE", "FEE")
        intercept[TestFailedException] {
          fumArray should contain inOrder ("fee", "fie", "foe", "fum")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should contain inOrder ("FUM", "FOE", "FIE", "FEE")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should contain inOrder ("fee", "fie", "foe", "fum")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should contain inOrder (" FUM ", " FOE ", " FIE ", " FEE ")
        }
        (fumArray should contain inOrder (" FUM ", " FOE ", " FIE ", " FEE ")) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should contain inOrder ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should contain inOrder ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (contain inOrder (..))` {
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain inOrder ("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain inOrder ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumArray),  "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain inOrder ("FUM", "FOE", "FIE", "FEE"))
        intercept[TestFailedException] {
          fumArray should (contain inOrder ("fee", "fie", "foe", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should (contain inOrder ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should (contain inOrder ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should (contain inOrder (" FUM ", " FOE ", " FIE ", " FEE "))
        }
        (fumArray should (contain inOrder (" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain inOrder ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumArray should (contain inOrder ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with not contain inOrder (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should not contain inOrder ("you", "to", "birthday", "happy")
        val e1 = intercept[TestFailedException] {
          toArray should not contain inOrder ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAllOfElementsInOrder", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toArray should not contain inOrder ("YOU", "TO", "BIRTHDAY", "HAPPY")
        intercept[TestFailedException] {
          toArray should not contain inOrder ("HAPPY", "BIRTHDAY", "TO", "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should not contain inOrder ("YOU", "TO", "BIRTHDAY", "HAPPY")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toArray should not contain inOrder ("HAPPY", "BIRTHDAY", "TO", "YOU")) (decided by upperCaseStringEquality)
        }
        toArray should not contain inOrder (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")
        intercept[TestFailedException] {
          (toArray should not contain inOrder (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toArray should not contain inOrder ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toArray should not contain inOrder ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (not contain inOrder (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should (not contain inOrder ("you", "to", "birthday", "happy"))
        val e1 = intercept[TestFailedException] {
          toArray should (not contain inOrder ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAllOfElementsInOrder", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toArray should (not contain inOrder ("YOU", "TO", "BIRTHDAY", "HAPPY"))
        intercept[TestFailedException] {
          toArray should (not contain inOrder ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should (not contain inOrder ("YOU", "TO", "BIRTHDAY", "HAPPY"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toArray should (not contain inOrder ("HAPPY", "BIRTHDAY", "TO", "YOU"))) (decided by upperCaseStringEquality)
        }
        toArray should (not contain inOrder (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))
        intercept[TestFailedException] {
          (toArray should (not contain inOrder (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toArray should (not contain inOrder ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toArray should (not contain inOrder ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
  }

  object `a col of Arrays` {

    val list1s: Vector[Array[Int]] = Vector( Array(0, 1, 2, 2, 3), Array(0, 1, 1, 2, 3, 3, 3), Array(0, 1, 2, 3))
    val lists: Vector[Array[Int]] = Vector( Array(0, 1, 2, 2, 3, 3, 3), Array(0, 1, 1, 1, 2, 3), Array(8, 2, 2, 3, 4))
    val listsArray: Vector[Array[Int]] = Vector( Array(0, 1, 1, 1, 2, 2, 2, 3, 3, 3), Array(0, 1, 2, 2, 3), Array())
    val hiArrays: Vector[Array[String]] = Vector( Array("hello", "hi", "hi", "he"), Array("hello", "hi", "he", "he", "he"), Array("hello", "hi", "he"))
    val toArrays: Vector[Array[String]] = Vector( Array("nice", "to", "you"), Array("nice", "to", "you"), Array("nice", "to", "you"))

    object `when used with contain inOrder (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain inOrder (1, 2, 3)
        atLeast (2, lists) should contain inOrder (1, 2, 3)
        atMost (2, lists) should contain inOrder (1, 2, 3)
        no (lists) should contain inOrder (3, 4, 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain inOrder (1, 2, 3)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain all of " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (listsArray) should contain inOrder (1, 2, 3)
        }
        e3.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(listsArray(2)) + " did not contain all of " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsArray)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should contain inOrder ("hi", "he")
        intercept[TestFailedException] {
          all (hiArrays) should contain inOrder ("hi", "ho")
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should contain inOrder ("HI", "HE")
        intercept[TestFailedException] {
          all (hiArrays) should contain inOrder ("HI", "HO")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should contain inOrder ("HI", "HE")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should contain inOrder ("HI", "HO")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should contain inOrder ("hi", "he")) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should contain inOrder ("hi", "ho")) (decided by defaultEquality[String])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain inOrder ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain inOrder (1, 2, 2, 3)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (contain inOrder (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrder (1, 2, 3))
        atLeast (2, lists) should (contain inOrder (1, 2, 3))
        atMost (2, lists) should (contain inOrder (1, 2, 3))
        no (lists) should (contain inOrder (3, 4, 5))
        no (listsArray) should (contain inOrder (3, 4, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrder (1, 2, 3))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain all of " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (listsArray) should (contain inOrder (1, 2, 3))
        }
        e4.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(listsArray(2)) + " did not contain all of " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsArray)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should (contain inOrder ("hi", "he"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain inOrder ("he", "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should (contain inOrder ("HI", "HE"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain inOrder ("HI", "HO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain inOrder ("HI", "HE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should (contain inOrder ("HI", "HO"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should (contain inOrder ("hi", "he"))) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should (contain inOrder ("he", "hi"))) (decided by defaultEquality[String])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (1, 2, 2, 3))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with not contain inOrder (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should not contain inOrder ("you", "to")
        val e1 = intercept[TestFailedException] {
          all (toArrays) should not contain inOrder ("to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toArrays(0)) + " contained all of " + "(\"to\", \"you\")" +  " in order (ArrayShouldContainInOrderSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should not contain inOrder ("YOU", "TO")
        intercept[TestFailedException] {
          all (toArrays) should not contain inOrder ("TO", "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should not contain inOrder ("YOU", "TO")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should not contain inOrder ("TO", "YOU")) (decided by upperCaseStringEquality)
        }
        all (toArrays) should not contain inOrder (" TO ", " YOU ")
        intercept[TestFailedException] {
          (all (toArrays) should not contain inOrder (" TO ", " YOU ")) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should not contain inOrder ()
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should not contain inOrder ("fee", "fie", "foe", "fie", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (not contain inOrder (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should (not contain inOrder ("you", "to"))
        val e1 = intercept[TestFailedException] {
          all (toArrays) should (not contain inOrder ("to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toArrays(0)) + " contained all of " + "(\"to\", \"you\")" + " in order (ArrayShouldContainInOrderSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should (not contain inOrder ("YOU", "TO"))
        intercept[TestFailedException] {
          all (toArrays) should (not contain inOrder ("TO", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should (not contain inOrder ("YOU", "TO"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should (not contain inOrder ("TO", "YOU"))) (decided by upperCaseStringEquality)
        }
        all (toArrays) should (not contain inOrder (" TO ", " YOU "))
        intercept[TestFailedException] {
          (all (toArrays) should (not contain inOrder (" TO ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should (not contain inOrder ())
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toArrays) should (not contain inOrder ("fee", "fie", "foe", "fie", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
  }
}
