/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class ArrayShouldContainTheSameElementsInOrderAsLogicalOrSpec extends FreeSpec with Matchers {
  
  //ADDITIONAL//
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val fileName: String = "ArrayShouldContainTheSameElementsInOrderAsLogicalOrSpec.scala"
  
  "a Array" - {
    
    val fumArray: Array[String] = Array("fum", "foe", "fie", "fee")
    val toArray: Array[String] = Array("you", "to", "birthday", "happy")
    
    "when used with (contain theSameElementsInOrderAs xx or contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") or contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        fumArray should (contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum") or contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") or contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum") or contain theSameElementsInOrderAs LinkedList("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("happy", "birthday", "to", "you"))), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or (contain theSameElementsInOrderAs LinkedList("FIE", "FEE", "FAM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or contain theSameElementsInOrderAs LinkedList("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsInOrderAs LinkedList(" FUM ", " FOE ", " FIE ", " FEE ") or contain theSameElementsInOrderAs LinkedList(" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (equal xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (equal (fumArray) or contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        fumArray should (equal (toArray) or contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        fumArray should (equal (fumArray) or contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        fumArray should (equal (toArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        fumArray should (equal (fumArray) or contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (equal (toArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (fumArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (toArray) or contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) or contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) or contain theSameElementsInOrderAs LinkedList(" FEE ", " FIE ", " FOE ", " FUM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    "when used with (legacyEqual xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        fumArray should (legacyEqual (toArray) or contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        fumArray should (legacyEqual (toArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (toArray) or contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) or contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) or contain theSameElementsInOrderAs LinkedList(" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed)
      }
    }

    "when used with (contain theSameElementsInOrderAs xx and legacyEqual xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsInOrderAs LinkedList(" FUM ", " FOE ", " FIE ", " FEE ") or legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")) or not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")) or not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")) or not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")) or not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not equal xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (not equal (toArray) or not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        fumArray should (not equal (fumArray) or not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        fumArray should (not equal (toArray) or not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        fumArray should (not equal (fumArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        fumArray should (not equal (toArray) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (not equal (fumArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (toArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (fumArray) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumArray should (not be (toArray) or not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        fumArray should (not be (fumArray) or not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        fumArray should (not be (toArray) or not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        fumArray should (not be (fumArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        fumArray should (not be (toArray) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumArray should (not be (toArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality)
        (fumArray should (not be (fumArray) or not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality)
        (fumArray should (not be (toArray) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) or not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
        (fumArray should (not contain theSameElementsInOrderAs (Set(" FEE ", " FIE ", " FOE ", " FUU ")) or not contain theSameElementsInOrderAs (Set(" FEE ", " FIE ", " FOE ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  "collection of Arrays" - {
    
    val list1s: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array(1, 2, 3))
    val lists: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array(2, 3, 4))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "hello"), Array("hi", "hello"), Array("hi", "hello"))
    val toArrays: Vector[Array[String]] = Vector(Array("you", "to"), Array("you", "to"), Array("you", "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(3, 2, 5) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) or contain theSameElementsInOrderAs LinkedList(2, 3, 4))
        
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(3, 6, 5) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) or contain theSameElementsInOrderAs LinkedList(8, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(1, 2, 3)) + ", and " + decorateToStringValue(Array(2, 3, 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(1, 2, 3)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") or contain theSameElementsInOrderAs LinkedList("hi", "hello"))
        all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HELLO", "HO") or contain theSameElementsInOrderAs LinkedList("hi", "hello"))
        all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") or contain theSameElementsInOrderAs LinkedList("hello", "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HELLO", "HO") or contain theSameElementsInOrderAs LinkedList("hello", "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HO")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hello", "ho")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") or contain theSameElementsInOrderAs LinkedList("hi", "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HELLO", "HO") or contain theSameElementsInOrderAs LinkedList("hi", "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") or contain theSameElementsInOrderAs LinkedList("hello", "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HELLO", "HO") or contain theSameElementsInOrderAs LinkedList("hello", "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HO")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hello", "ho")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (be xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (Array(1, 2, 3)) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        all (list1s) should (be (Array(2, 3, 4)) or contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        all (list1s) should (be (Array(1, 2, 3)) or contain theSameElementsInOrderAs LinkedList(2, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (Array(2, 3, 4)) or contain theSameElementsInOrderAs LinkedList(2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(1, 2, 3)) + " was not equal to " + decorateToStringValue(Array(2, 3, 4)) + ", and " + decorateToStringValue(Array(1, 2, 3)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 4)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))
        all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))
        all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(3, 2, 8)) or not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)) or not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(3, 2, 8)) or not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList(2, 3, 4)) or not contain theSameElementsInOrderAs (LinkedList(2, 3, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 4)) + ", and " + decorateToStringValue(Array(2, 3, 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) or not contain theSameElementsInOrderAs (LinkedList("hello", "hi")))
        all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")) or not contain theSameElementsInOrderAs (LinkedList("hello", "hi")))
        all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) or not contain theSameElementsInOrderAs (LinkedList("hi", "hello")))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")) or not contain theSameElementsInOrderAs (LinkedList("hi", "hello")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) or not contain theSameElementsInOrderAs (LinkedList("hello", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")) or not contain theSameElementsInOrderAs (LinkedList("hello", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) or not contain theSameElementsInOrderAs (LinkedList("hi", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")) or not contain theSameElementsInOrderAs (LinkedList("hi", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (Array(2)) or not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        all (list1s) should (not be (Array(1, 2, 3)) or not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        all (list1s) should (not be (Array(2)) or not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (Array(1, 2, 3)) or not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(1, 2, 3)) + " was equal to " + decorateToStringValue(Array(1, 2, 3)) + ", and " + decorateToStringValue(Array(1, 2, 3)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(1, 2, 3)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsInOrderAs (LinkedList("HELLO", "HO")))
        all (hiArrays) should (not be (Array("hi", "hello")) or not contain theSameElementsInOrderAs (LinkedList("HELLO", "HO")))
        all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi", "hello")) or not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsInOrderAs (LinkedList("HELLO", "HO")))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hi", "hello")) or not contain theSameElementsInOrderAs (LinkedList("HELLO", "HO")))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi", "hello")) or not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
  }
}
