/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAtMostOneOfLogicalAndSpec extends Spec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainAtMostOneOfLogicalAndSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"), Entry("foe", "foe"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"), Entry("you", "you"))
    
    object `when used with (contain atMostOneOf (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) and contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) and contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")) and contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf () and contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) and contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) and contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) and contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (tojavaMap) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) and contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) and contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) and contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) and contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) and contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and legacyEqual (fumjavaMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (fumjavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and (legacyEqual (tojavaMap)))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")) and legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf() and legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain atMostOneOf (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") and not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fam=fam, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and (not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")) and contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf () and not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) and not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) and not contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) and not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fam=fam, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and (not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (fumjavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) and not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) and not contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fam=fam, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and (not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) and not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) and not contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  object `collection of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(2, 2), Entry(3, 3)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain atMostOneOf (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf (3, 4, 1) and contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) and contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (contain atMostOneOf (Entry(3, 3), Entry(2, 2), Entry(8, 8)) and contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (lists) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain atMostOneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)) and contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of (2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)) and contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of (1=1, 3=3, 4=4), but " + decorateToStringValue(lists(2)) + " did not contain at most one of (2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atMostOneOf (Entry("hi", "hi"), Entry("hello", "hello")) and contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (hi=hi, hello=hello), but " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (hi=hi, he=he)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) and contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of (1=1, 6=6, 8=8), but " + decorateToStringValue(lists(2)) + " did not contain at most one of (2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) and contain atMostOneOf ("HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) and contain atMostOneOf ("HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) and contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO), but " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) and contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO), but " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () and contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)) and contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)) and contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (lists) should (be (javaMap(Entry(8, 8))) and contain atMostOneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(javaMap(Entry(1, 1), Entry(2, 2))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1), Entry(2, 2))) + ", but " + decorateToStringValue(list1s(0)) + " did not contain at most one of (1=1, 2=2, 3=3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))) + ", but " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (hi=hi, he=he)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1), Entry(2, 2))) + ", but " + decorateToStringValue(list1s(0)) + " did not contain at most one of (1=1, 2=2, 3=3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and contain atMostOneOf ("HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) and contain atMostOneOf ("HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))) + ", but " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))) + ", but " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) and contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain atMostOneOf (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atMostOneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (not contain atMostOneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) and not contain atMostOneOf (Entry(3, 3), Entry(6, 6), Entry(8, 8)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of (1=1, 2=2, 8=8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of (1=1, 2=2, 3=3), but " + decorateToStringValue(lists(2)) + " contained at most one of (1=1, 2=2, 8=8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atMostOneOf (Entry("hi", "hi"), Entry("hello", "hello")) and not contain atMostOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (hi=hi, hello=hello)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he")) and not contain atMostOneOf (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (hi=hi, he=he), but " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (hi=hi, hello=hello)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) and not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) and not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) and not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE), but " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) and not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) and not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) and not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE), but " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) and contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (list1s) should (not be (javaMap(Entry(1, 1), Entry(2, 2))) and not contain atMostOneOf (Entry(3, 3), Entry(6, 6), Entry(8, 8)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was equal to " + decorateToStringValue(javaMap(Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(javaMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + ", but " + decorateToStringValue(lists(2)) + " contained at most one of (1=1, 2=2, 8=8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and not contain atMostOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("ho", "ho"))) and not contain atMostOneOf (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("ho", "ho"))) + ", but " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (hi=hi, hello=hello)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("HI", "HI"), Entry("HO", "HO"))) and not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and not contain atMostOneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("ho", "ho"))) and not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("ho", "ho"))) + ", but " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap(Entry("HI", "HI"), Entry("HO", "HO"))) and not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) and not contain atMostOneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("ho", "ho"))) and not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("ho", "ho"))) + ", but " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) and not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) and not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
