/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAtMostOneOfLogicalOrSpec extends FreeSpec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainAtMostOneOfLogicalOrSpec.scala"
  
  "a javaMap" - {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"), Entry("foe", "foe"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"), Entry("you", "you"))
    
      "when used with (contain atMostOneOf (...) or contain atMostOneOf (...)) syntax" - {
      
        "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")) or contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf () or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (equal (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (equal (tojavaMap) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        fumjavaMap should (equal (tojavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (equal (tojavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (tojavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) or contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) or contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (legacyEqual (...) and contain theMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (tojavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) or contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) or contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    "when used with (contain oneOf (...) and legacyEqual (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FaM ") or legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf() or legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain atMostOneOf (...) and not contain atMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "\"fee\", \"fie\", \"foe\", \"fuu\"") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")) or contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf () or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or not contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not equal (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not equal (fumjavaMap) or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not equal (fumjavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (not equal (fumjavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (fumjavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) or not contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not be (fumjavaMap) or not contain atMostOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf("fee", "fie", "foe", "fuu"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not be (fumjavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (fumjavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) or not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUU=FUU"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) or not contain atMostOneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  "collection of javaMaps" - {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(2, 2), Entry(3, 3)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain oneOf (..) and contain oneOf (..)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) or contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) or contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) or contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) or contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)) or contain atMostOneOf (Entry(4, 4), Entry(3, 3), Entry(2, 2)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of (2=2, 3=3, 4=4), and " + decorateToStringValue(lists(2)) + " did not contain at most one of (4=4, 3=3, 2=2)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) or contain atMostOneOf (Entry("HO", "HO"), Entry("HI", "HI")))
        all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or contain atMostOneOf (Entry("HO", "HO"), Entry("HI", "HI")))
        all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) or contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE), and " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HE=HE, HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) or contain atMostOneOf (Entry("HO", "HO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or contain atMostOneOf (Entry("HO", "HO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")) or contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE), and " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HE=HE, HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () or contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)) or contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)) or contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (be (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) or contain atMostOneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (be (javaMap(Entry(2, 2), Entry(3, 3))) or contain atMostOneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) or contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(2, 2), Entry(3, 3))) or contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry(2, 2), Entry(3, 3))) + ", and " + decorateToStringValue(list1s(0)) + " did not contain at most one of (1=1, 2=2, 3=3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))
        all (hijavaMaps) should (be (javaMap(Entry("HO", "HO"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("HO", "HO"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("HO", "HO"))) + ", and " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("HO", "HO"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("HO", "HO"))) or contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was not equal to " + decorateToStringValue(javaMap(Entry("HO", "HO"))) + ", and " + decorateToStringValue(hijavaMaps(0)) + " did not contain at most one of (HI=HI, HE=HE)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) or contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(1, 1), Entry(2, 2))) or contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain oneOf (..) and not contain oneOf (..))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain atMostOneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (not contain atMostOneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) or not contain atMostOneOf (8, 2, 1))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of (1=1, 2=2, 8=8), and " + decorateToStringValue(lists(2)) + " contained at most one of (8, 2, 1)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))
        all (hijavaMaps) should (not contain atMostOneOf (Entry("hi", "hi")) or not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))
        all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or not contain atMostOneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atMostOneOf (Entry("hi", "hi")) or not contain atMostOneOf (Entry("he", "he")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (hi=hi), and " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (he=he)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not contain atMostOneOf (Entry("hi", "hi")) or not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")) or not contain atMostOneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain atMostOneOf (Entry("hi", "hi")) or not contain atMostOneOf (Entry("he", "he")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " contained at most one of (hi=hi), and " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (he=he)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain atMostOneOf (...))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (not be (javaMap(Entry(1, 1), Entry(2, 2))) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) or not contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaMap(Entry(1, 1), Entry(2, 2))) or not contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1), Entry(2, 2))) + ", and " + decorateToStringValue(list1s(0)) + " contained at most one of (2=2, 3=3, 4=4)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))) + ", and " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or not contain atMostOneOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("he", "he"))) or not contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaMaps(0)) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("he", "he"))) + ", and " + decorateToStringValue(hijavaMaps(0)) + " contained at most one of (HI=HI, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) or not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2), Entry(3, 3))) or not contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
