/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAtMostOneOfSpec extends Spec with Matchers {

  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry

  object `a javaMap` {

    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"), Entry("foe", "foe"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"), Entry("you", "you"))

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumjavaMap should contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))
        intercept[TestFailedException] {
          fumjavaMap should contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumjavaMap should contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))) (after being lowerCased and trimmed)
          
        }
        fumjavaMap should contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))
        intercept[TestFailedException] {
          fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
        }
        fumjavaMap should (contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumjavaMap should not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))
        intercept[TestFailedException] {
          fumjavaMap should not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))) (decided by upperCaseStringEquality)
        }
        (fumjavaMap should not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumjavaMap should not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))) (decided by upperCaseStringEquality)
        }
        (fumjavaMap should (not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumjavaMap should (not contain atMostOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }

  object `a collection of javaMaps` {

    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(2, 2), Entry(3, 3)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1), Entry(2, 2)), javaMap(Entry(1, 1), Entry(2, 2)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")))

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))
        atLeast (2, lists) should contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4))
        atMost (2, lists) should contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4))
        no (lists) should contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3))

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of (2=2, 3=3, 4=4) (JavaMapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should contain atMostOneOf ("hi", "ho")
        intercept[TestFailedException] {
          all (hijavaMaps) should contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he"))
        }
        implicit val use = upperCaseStringEquality
        all (hijavaMaps) should contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE"))
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should contain atMostOneOf ("hi", "ho")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaMaps) should contain atMostOneOf ("hi", "HE")) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (lists) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of (2=2, 3=3, 4=4) (JavaMapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should (contain atMostOneOf ("hi", "ho"))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he")))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HO", "HO")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain atMostOneOf (Entry("HI", "HI"), Entry("HE", "HE")))
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain atMostOneOf ("hi", "ho"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he")))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaMaps) should (contain atMostOneOf ("hi", "HE"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atMostOneOf (Entry("hi", "hi"), Entry("he", "he")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should not contain atMostOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(tojavaMaps(0)) + " contained at most one of (fee=fee, fie=fie, foe=foe, fum=fum) (JavaMapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaMaps) should not contain atMostOneOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should not contain atMostOneOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) (decided by upperCaseStringEquality)
        }
        (all (tojavaMaps) should not contain atMostOneOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain atMostOneOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should (not contain atMostOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(tojavaMaps(0)) + " contained at most one of (fee=fee, fie=fie, foe=foe, fum=fum) (JavaMapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaMaps) should (not contain atMostOneOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should (not contain atMostOneOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain atMostOneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        (all (tojavaMaps) should (not contain atMostOneOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain atMostOneOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should (not contain atMostOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
