/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainInOrderLogicalAndSpec extends Spec with Matchers {

  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val invertedjavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = a != b
    }

  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCasejavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  val fileName: String = "JavaMapShouldContainInOrderLogicalAndSpec.scala"

  object `a javaSortedMap` {

    val fumjavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("fex", "fex"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("too", "too"), Entry("you", "you"), Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"), Entry("happy", "happy"))

    object `when used with (contain inOrder xx and contain inOrder xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee") + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }

      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) and contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder () and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (equal xx and contain inOrder xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (decided by invertedjavaSortedMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (legacyEqual xx and contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (contain inOrder xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and legacyEqual (fumjavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and legacyEqual (fumjavaSortedMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (fumjavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")) and (legacyEqual (fumjavaSortedMap)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "HAPPY=HAPPY, BIRTHDAY=BIRTHDAY, TO=TO, YOU=YOU"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) and legacyEqual (fumjavaSortedMap))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder () and legacyEqual (fumjavaSortedMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and legacyEqual (fumjavaSortedMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not contain inOrder xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and not contain inOrder (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder () and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not equal xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (not contain inOrder (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) and not contain inOrder (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
  }
  
  object `col of javaSortedMaps` {
    
    val list1s: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
    val lists: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3)), javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4)))
    val nils: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(), javaSortedMap(), javaSortedMap())
    val listsMap: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap())
    val hijavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain inOrder xx and contain inOrder xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (contain inOrder (Entry(3, 3), Entry(6, 6), Entry(9, 9)) and contain inOrder (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain inOrder (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (contain inOrder (Entry(3, 3), Entry(8, 8), Entry(5, 5)) and contain inOrder (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain all of " + "(1=1, 2=2, 3=3)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(list1s(0)) + " contained all of " + "(1=1, 2=2, 3=3)" + " in order" + ", but " + decorateToStringValue(list1s(0)) + " did not contain all of " + "(1=1, 3=3, 4=4)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")) and contain inOrder (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " did not contain all of " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")) and contain inOrder (Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(hi=hi, hello=hello)" + " in order" + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(hello=hello, hi=hi)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrder (Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HO=HO, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order" + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }

      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain inOrder (Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HO=HO, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order" + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder () and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (be xx and contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3))) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (be (javaSortedMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (be (javaSortedMap(Entry(3, 3), Entry(6, 6), Entry(9, 9))) and contain inOrder (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (be (javaSortedMap(Entry(1, 1), Entry(6, 6), Entry(8, 8))) and contain inOrder (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (be (javaSortedMap(Entry(2, 2), Entry(6, 6), Entry(8, 8))) and contain inOrder (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3))) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrder (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(list1s(0)) + " did not contain all of " + "(2=2, 3=3, 8=8)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (javaSortedMap(Entry("hey", "hey"))) and contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("hey", "hey"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrder (Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(hello=hello, hi=hi)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(listsMap(2)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrder (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(list1s(0)) + " did not contain all of " + "(2=2, 3=3, 8=8)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (not contain inOrder xx and not contain inOrder xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain inOrder (Entry(3, 3), Entry(2, 2), Entry(8, 8)) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (not contain inOrder (Entry(3, 3), Entry(8, 8), Entry(5, 5)) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (not contain inOrder (2, 4, 3) and contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (list1s) should (not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained all of " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain inOrder (Entry(3, 3), Entry(6, 6), Entry(8, 8)) and not contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain all of " + "(3=3, 6=6, 8=8)" + " in order" + ", but " + decorateToStringValue(lists(2)) + " contained all of " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")) and not contain inOrder (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrder (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")) and not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(ho=ho, hey=hey, howdy=howdy)" + " in order" + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and not contain inOrder (Entry("HO", "HO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order" + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and not contain inOrder (Entry("HO", "HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order" + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder () and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)) and not contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (not be (javaSortedMap(Entry(3, 3))) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (not be (javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (list1s) should (not be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaSortedMap(Entry(3, 3))) and not contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(3, 3))) + ", but " + decorateToStringValue(lists(2)) + " contained all of " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and not contain inOrder (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrder (Entry("HO", "HO"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrder (Entry("HO", "HO"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) and not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
  }
}
