/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainInOrderLogicalOrSpec extends Spec with Matchers {
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val invertedjavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val fileName: String = "JavaMapShouldContainInOrderLogicalOrSpec.scala"
  
  object `a javaSortedMap` {
    
    val fumjavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("fex", "fex"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"), Entry("fee", "fee"), Entry("fee", "fee"))
    val tojavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("too", "too"), Entry("you", "you"), Entry("to", "to"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("birthday", "birthday"), Entry("happy", "happy"))
    
    object `when used with (contain inOrder xx or contain inOrder xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain inOrder (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or (contain inOrder (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain inOrder (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) or contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder () or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (equal xx and contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (equal (tojavaSortedMap) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (equal (tojavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) or (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (equal (tojavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (equal (tojavaSortedMap) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (equal (tojavaSortedMap) or contain inOrder (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (decided by invertedjavaSortedMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (legacyEqual xx and contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (contain inOrder xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or legacyEqual (tojavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (tojavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain inOrder (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) or legacyEqual (fumjavaSortedMap))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder () or legacyEqual (fumjavaSortedMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or legacyEqual (fumjavaSortedMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not contain inOrder xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaSortedMap should (not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee") + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaSortedMap should (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder () or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not equal xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) or (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain inOrder (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) or (not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain inOrder (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain inOrder (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedAllOfElementsInOrder", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (not contain inOrder (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))) or not contain inOrder (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) or not contain inOrder (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
  }
  
  object `collection of javaSortedMaps` {
    
    val list1s: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
    val lists: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3)), javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4)))
    val nils: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(), javaSortedMap(), javaSortedMap())
    val listsMap: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap())
    val hijavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello")))
    val tojavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("nice", "nice"), Entry("you", "you"), Entry("to", "to")), javaSortedMap(Entry("nice", "nice"), Entry("you", "you"), Entry("to", "to")), javaSortedMap(Entry("nice", "nice"), Entry("you", "you"), Entry("to", "to")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain inOrder xx and contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (contain inOrder (3, 2, 5) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        atLeast (2, lists) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain inOrder (Entry(3, 3), Entry(6, 6), Entry(5, 5)) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain all of " + "(1=1, 2=2, 3=3)" + " in order" + ", and " + decorateToStringValue(lists(2)) + " did not contain all of " + "(1=1, 2=2, 3=3)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        all (hijavaSortedMaps) should (contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain inOrder (Entry("hello", "hello"), Entry("ho", "ho")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain inOrder (Entry("hello", "hello"), Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HO=HO)" + " in order" + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(hello=hello, ho=ho)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain inOrder (Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)

        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain inOrder (Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HO=HO)" + " in order" + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(hello=hello, ho=ho)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder () or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }

    object `when used with (be xx and contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (be (javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4))) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) or contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4))) or contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(8, 8), Entry(2, 2), Entry(3, 3), Entry(4, 4))) + ", and " + decorateToStringValue(list1s(0)) + " did not contain all of " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " did not contain all of " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) or contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) or contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not contain inOrder xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain inOrder (Entry(3, 3), Entry(2, 2), Entry(8, 8)) or not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not contain inOrder (Entry(3, 3), Entry(2, 2), Entry(8, 8)) or not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)) or not contain inOrder (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained all of " + "(2=2, 3=3, 4=4)" + " in order" + ", and " + decorateToStringValue(lists(2)) + " contained all of " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain inOrder (Entry("hello", "hello"), Entry("hi", "hi")))
        all (hijavaSortedMaps) should (not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or not contain inOrder (Entry("hello", "hello"), Entry("hi", "hi")))
        all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order" + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain inOrder (Entry("hello", "hello"), Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or not contain inOrder (Entry("hello", "hello"), Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not contain inOrder (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")) or not contain inOrder (Entry("hi", "hi"), Entry("hello", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order" + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder () or not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)) or not contain inOrder ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)) or not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrder xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) or not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) or not contain inOrder (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) or not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) or not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(0, 0), Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + ", and " + decorateToStringValue(list1s(0)) + " contained all of " + "(1=1, 2=2, 3=3)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or not contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or not contain inOrder (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)

        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) or not contain inOrder (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("he", "he"), Entry("hi", "hi"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(hijavaSortedMaps(0)) + " contained all of " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaSortedMap(Entry(2, 2))) or not contain inOrder ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaSortedMap(Entry(2, 2))) or not contain inOrder (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderDuplicate")))
      }
    }
  }
}
