/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainInOrderOnlyLogicalAndSpec extends Spec with Matchers {

  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val invertedjavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = a != b
    }

  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCasejavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  val fileName: String = "JavaMapShouldContainInOrderOnlyLogicalAndSpec.scala"

  object `a javaSortedMap` {

    val fumjavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("fum", "fum"), Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("you", "you"), Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"), Entry("happy", "happy"))

    object `when used with (contain inOrderOnly xx and contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee") + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }

      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain inOrderOnly (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) and contain inOrderOnly (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrderOnly () and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (equal xx and contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (equal (tojavaSortedMap) and contain inOrderOnly (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (decided by invertedjavaSortedMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (legacyEqual xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (contain inOrderOnly xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and legacyEqual (fumjavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and legacyEqual (fumjavaSortedMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (fumjavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")) and (legacyEqual (fumjavaSortedMap)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "HAPPY=HAPPY, BIRTHDAY=BIRTHDAY, TO=TO, YOU=YOU"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain inOrderOnly (Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) and legacyEqual (fumjavaSortedMap))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrderOnly () and legacyEqual (fumjavaSortedMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and legacyEqual (fumjavaSortedMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not contain inOrderOnly xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and not contain inOrderOnly (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and (not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrderOnly () and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not equal xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and (not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly (Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "fum=fum, foe=foe, fie=fie, fee=fee"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and (not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly (Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumjavaSortedMap), "FUM=FUM, FOE=FOE, FIE=FIE, FEE=FEE"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain inOrderOnly (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (not contain inOrderOnly (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) and not contain inOrderOnly (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain inOrderOnly (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
  }
  
  object `col of javaSortedMaps` {
    
    val list1s: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
    val lists: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3)), javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
    val nils: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(), javaSortedMap(), javaSortedMap())
    val listsMap: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap())
    val hijavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain inOrderOnly xx and contain inOrderOnly xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (contain inOrderOnly (Entry(3, 3), Entry(6, 6), Entry(9, 9)) and contain inOrderOnly (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain inOrderOnly (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (contain inOrderOnly (Entry(3, 3), Entry(8, 8), Entry(5, 5)) and contain inOrderOnly (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain only " + "(1=1, 2=2, 3=3)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + " contained only " + "(1=1, 2=2, 3=3)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + " did not contain only " + "(1=1, 3=3, 4=4)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain inOrderOnly (Entry("hi", "hi"), Entry("hello", "hello")) and contain inOrderOnly (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " did not contain only " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrderOnly (Entry("hi", "hi"), Entry("hello", "hello")) and contain inOrderOnly (Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(hi=hi, hello=hello)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(hello=hello, hi=hi)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrderOnly (Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HO=HO, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }

      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain inOrderOnly (Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HO=HO, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly () and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (be xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3))) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (be (javaSortedMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (be (javaSortedMap(Entry(3, 3), Entry(6, 6), Entry(9, 9))) and contain inOrderOnly (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (be (javaSortedMap(Entry(1, 1), Entry(6, 6), Entry(8, 8))) and contain inOrderOnly (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (be (javaSortedMap(Entry(2, 2), Entry(6, 6), Entry(8, 8))) and contain inOrderOnly (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3))) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + " did not contain only " + "(2=2, 3=3, 8=8)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (javaSortedMap(Entry("hey", "hey"))) and contain inOrderOnly (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("hey", "hey"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrderOnly (Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(hello=hello, hi=hi)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(listsMap(2)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) + " did not contain only " + "(2=2, 3=3, 8=8)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HI=HI)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (not contain inOrderOnly xx and not contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain inOrderOnly (Entry(3, 3), Entry(2, 2), Entry(8, 8)) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (not contain inOrderOnly (Entry(3, 3), Entry(8, 8), Entry(5, 5)) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (not contain inOrderOnly (2, 4, 3) and contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (list1s) should (not contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)) and not contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(4, 4)) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained only " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain inOrderOnly (Entry(3, 3), Entry(6, 6), Entry(8, 8)) and not contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain only " + "(3=3, 6=6, 8=8)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained only " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("hi", "hi"), Entry("hello", "hello")) and not contain inOrderOnly (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")) and not contain inOrderOnly (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(ho=ho, hey=hey, howdy=howdy)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and not contain inOrderOnly (Entry("HO", "HO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HI=HI)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")) and not contain inOrderOnly (Entry("HO", "HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")) and not contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HI=HI)" + " in order" + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly () and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)) and not contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)) and not contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (not be (javaSortedMap(Entry(3, 3))) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (not be (javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (list1s) should (not be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))) and not contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain inOrderOnly (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaSortedMap(Entry(3, 3))) and not contain inOrderOnly (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained only " + "(2=2, 3=3, 4=4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain inOrderOnly (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrderOnly (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(hi=hi, hello=hello)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrderOnly (Entry("HO", "HO"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrderOnly (Entry("HO", "HO"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain inOrderOnly (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain inOrderOnly (Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HI=HI, HELLO=HELLO)" + " in order", thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain inOrderOnly (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
  }
}
