/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class JavaMapShouldContainNoneOfLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = b match {
    case entry: java.util.Map.Entry[_, _] => 
      (entry.getKey, entry.getValue) match {
        case (k: String, v: String) => a.getKey.toUpperCase == k && a.getValue.toUpperCase == v
        case _ => (a.getKey.toUpperCase, a.getValue.toUpperCase) == b
      }
    case _ => (a.getKey.toUpperCase, a.getValue.toUpperCase) == b
  }
    }
  
  val upperCasejavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a.asScala.map(e => upperCase(e)) == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainNoneOfLogicalOrSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"))
    
    object `when used with (contain noneOf (..) or contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")) or contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain noneOf () or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or contain noneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (equal (tojavaMap) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (equal (fumjavaMap) or contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) or contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (equal (tojavaMap) or contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (equal (fumjavaMap) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) or contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (fumjavaMap) or contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (tojavaMap) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) or contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) or contain noneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) or contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (tojavaMap) or contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) or contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FAM ", " FAM ")))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) or contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with (contain noneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain noneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain noneOf() or legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain noneOf("fee", "fie", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or not contain noneOf ("fee", "fie", "fam", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "\"fee\", \"fie\", \"fam\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain noneOf () or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")) or not contain noneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("fum", "fum"), Entry("foe", "foe")) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not equal (fumjavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not equal (tojavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) or not contain noneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not equal (fumjavaMap) or not contain noneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (not equal (tojavaMap) or not contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or (not contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) or not contain noneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (tojavaMap) or not contain noneOf (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (fumjavaMap) or not contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) or not contain noneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or not contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) or not contain noneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not be (fumjavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not be (tojavaMap) or not contain noneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not be (fumjavaMap) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not be (tojavaMap) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or (not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (fumjavaMap) or not contain noneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (tojavaMap) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) or not contain noneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or not contain noneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) or not contain noneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) or not contain noneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
  }
  
  object `collection of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(2, 2)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to")), javaMap(Entry("to", "to")), javaMap(Entry("to", "to")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (Entry(3, 3), Entry(6, 6), Entry(9, 9)) or contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (contain noneOf (Entry(3, 3), Entry(6, 6), Entry(9, 9)) or contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        atLeast (2, lists) should (contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or contain noneOf (Entry(3, 3), Entry(6, 6), Entry(9, 9)))
        atLeast (2, lists) should (contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain noneOf (Entry(3, 3), Entry(6, 6), Entry(9, 9)))
        atLeast (2, lists) should (contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or contain noneOf (Entry(2, 2), Entry(3, 3), Entry(5, 5)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " contained one of (2=2, 6=6, 8=8), and " + decorateToStringValue(javaMap(Entry(2, 2))) + " contained one of (2=2, 3=3, 5=5)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (contain noneOf (Entry("hi", "hi")) or contain noneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (contain noneOf (Entry("hi", "hi")) or contain noneOf (Entry("HI", "HI")))
        all (hijavaMaps) should (contain noneOf (Entry("HI", "HI")) or contain noneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain noneOf (Entry("HI", "HI")) or contain noneOf (Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (HI=HI), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain noneOf (Entry("hi", "hi")) or contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (contain noneOf (Entry("hi", "hi")) or contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (contain noneOf (Entry("HI", "HI")) or contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain noneOf (Entry("HI", "HI")) or contain noneOf (Entry("HI", "HI")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (HI=HI), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain noneOf () or contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or contain noneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain noneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or contain noneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(Entry(1, 1))) or contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (be (javaMap(Entry(2, 2))) or contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (be (javaMap(Entry(1, 1))) or contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(2, 2))) or contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry(1, 1))) + " was not equal to " + decorateToStringValue(javaMap(Entry(2, 2))) + ", and " + decorateToStringValue(javaMap(Entry(1, 1))) + " contained one of (1=1, 2=2, 3=3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain noneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain noneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain noneOf (Entry("HI", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain noneOf (Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(1, 1))) or contain noneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(1, 1))) or contain noneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (not contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (not contain noneOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or not contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) or not contain noneOf (1, 3, 5))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain one of (1=1, 6=6, 8=8), and " + decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain one of (1, 3, 5)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not contain noneOf (Entry("HI", "HI")) or not contain noneOf (Entry("HI", "HI")))
        all (hijavaMaps) should (not contain noneOf (Entry("hi", "hi")) or not contain noneOf (Entry("HI", "HI")))
        all (hijavaMaps) should (not contain noneOf (Entry("HI", "HI")) or not contain noneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain noneOf (Entry("hi", "hi")) or not contain noneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain noneOf (Entry("HI", "HI")) or not contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not contain noneOf (Entry("hi", "hi")) or not contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not contain noneOf (Entry("HI", "HI")) or not contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain noneOf (Entry("hi", "hi")) or not contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain noneOf () or not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) or not contain noneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain noneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) or not contain noneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (not be (javaMap(Entry(1, 1))) or not contain noneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)))
        all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaMap(Entry(1, 1))) or not contain noneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1))) + ", and " + decorateToStringValue(javaMap(Entry(1, 1))) + " did not contain one of (2=2, 6=6, 8=8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain noneOf (Entry("HI", "HI")))
        all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain noneOf (Entry("HI", "HI")))
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain noneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain noneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain noneOf (Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain noneOf (Entry("hi", "hi")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain noneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain noneOf (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
  }
}
