/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainOnlyLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainOnlyLogicalOrSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))
    
    object `when used with (contain only (..) or contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain only (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam") + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only () or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (equal (tojavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (equal (fumjavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or contain only (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (equal (tojavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (equal (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (tojavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) or contain only Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) or contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (equal (fumjavaMap) or contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fam=fam, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (tojavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) or contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FAM=FAM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) or contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) or contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (contain only (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FAM=FAM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only () or legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or legacyEqual (fumjavaMap))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not contain only (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain only ("fee", "fie", "foe", "fuu") or not contain only ("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain only ("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not contain only ("fee", "fie", "foe", "fuu") or not contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUU", "FUU")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain only () or not contain only ("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain only ("fie", "fee", "fuu", "foe") or not contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")) or not contain only ("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not contain only ("fie", "fee", "fuu", "foe") or not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) or not contain only ("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not equal (fumjavaMap) or not contain only ("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not equal (tojavaMap) or not contain only (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        fumjavaMap should (not equal (fumjavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        fumjavaMap should (not equal (tojavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (tojavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (fumjavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FIE=FIE, FEE=FEE, FUM=FUM, FOE=FOE"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) or not contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) or not contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not equal (tojavaMap) or not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) or not contain only ("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not be (fumjavaMap) or not contain only ("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not be (tojavaMap) or not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        fumjavaMap should (not be (fumjavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))
        fumjavaMap should (not be (tojavaMap) or not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or (not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (fumjavaMap) or not contain only (Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUU", "FUU"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (tojavaMap) or not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) or not contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) or not contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) or not contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (not be (tojavaMap) or not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
  }
  
  object `collection of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi"), Entry("hello", "hello")), javaMap(Entry("hi", "hi"), Entry("hello", "hello")), javaMap(Entry("hi", "hi"), Entry("hello", "hello")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("you", "you"), Entry("to", "to")), javaMap(Entry("you", "you"), Entry("to", "to")), javaMap(Entry("you", "you"), Entry("to", "to")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain only (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only (Entry(3, 3), Entry(2, 2), Entry(1, 1)) or contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        all (list1s) should (contain only (3, 2, 5) or contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        all (list1s) should (contain only (Entry(3, 3), Entry(2, 2), Entry(1, 1)) or contain only (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        atLeast (2, lists) should (contain only (Entry(3, 3), Entry(1, 1), Entry(2, 2)) or contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain only (Entry(3, 3), Entry(6, 6), Entry(5, 5)) or contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        atLeast (2, lists) should (contain only (Entry(3, 3), Entry(1, 1), Entry(2, 2)) or contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only (Entry(3, 3), Entry(1, 1), Entry(2, 2)) or contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain only " + "(3=3, 1=1, 2=2)" + ", and " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain only " + "(1=1, 3=3, 2=2)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or contain only (Entry("hello", "hello"), Entry("hi", "hi")))
        all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain only (Entry("hello", "hello"), Entry("hi", "hi")))
        all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or contain only (Entry("hello", "hello"), Entry("ho", "ho")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain only (Entry("hello", "hello"), Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HO=HO)" + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(hello=hello, ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or contain only (Entry("hello", "hello"), Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain only (Entry("hello", "hello"), Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or contain only (Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain only (Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HO=HO)" + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(hello=hello, ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only () or contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)) or contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only (3, 2, 2, 1) or contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only (Entry(1, 1), Entry(3, 3), Entry(2, 2)) or contain only (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) or contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (be (javaMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) or contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) or contain only (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) or contain only (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " was not equal to " + decorateToStringValue(javaMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + ", and " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " did not contain only " + "(2=2, 3=3, 4=4)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain only " + "(HELLO=HELLO, HO=HO)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) or contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) or contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not contain only xx and not contain only xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain only (Entry(3, 3), Entry(2, 2), Entry(8, 8)) or not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)) or not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not contain only (Entry(3, 3), Entry(2, 2), Entry(8, 8)) or not contain only (Entry(2, 2), Entry(3, 3), Entry(1, 1)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain only (Entry(4, 4), Entry(2, 2), Entry(3, 3)) or not contain only (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " contained only " + "(4=4, 2=2, 3=3)" + ", and " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " contained only " + "(2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or not contain only (Entry("hello", "hello"), Entry("ho", "ho")))
        all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain only (Entry("hello", "hello"), Entry("ho", "ho")))
        all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or not contain only (Entry("hello", "hello"), Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain only (Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HELLO=HELLO, HI=HI)" + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(hello=hello, hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or not contain only (Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain only (Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")) or not contain only (Entry("hello", "hello"), Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")) or not contain only (Entry("hello", "hello"), Entry("hi", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HELLO=HELLO, HI=HI)" + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(hello=hello, hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only () or not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)) or not contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)) or not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)) or not contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not be (...) and not contain only (...))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) or not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) or not contain only (Entry(2, 2), Entry(3, 3), Entry(1, 1)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + ", and " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " contained only " + "(2=2, 3=3, 1=1)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        all (hijavaMaps) should (not be (javaMap(Entry("hello", "hello"), Entry("hi", "hi"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))
        all (hijavaMaps) should (not be (javaMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HELLO=HELLO, HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("hello", "hello"), Entry("hi", "hi"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) or not contain only (Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained only " + "(HELLO=HELLO, HI=HI)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only (Entry(8, 8), Entry(3, 3), Entry(4, 4)) or not contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
  }
}
