/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainOnlySpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry

  object `a javaMap` {

    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))

    object `when used with contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should contain only (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))
        intercept[TestFailedException] {
          fumjavaMap should contain only ("fee", "fie", "foe")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should contain only ("fee", "fie", "foe")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))
        }
        (fumjavaMap should contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should contain only ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        intercept[TestFailedException] {
          fumjavaMap should (contain only ("fee", "fie", "foe"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain only (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (contain only ("fee", "fie", "foe"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should (contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))
        }
        (fumjavaMap should (contain only (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumjavaMap should (contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with not contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          tojavaMap should not contain only (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOnlyElements", decorateToStringValue(tojavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        tojavaMap should not contain only ("happy", "birthday", "to")
        intercept[TestFailedException] {
          tojavaMap should not contain only (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should not contain only ("happy", "birthday", "to")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (tojavaMap should not contain only (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))) (decided by upperCaseStringEquality)
        }
        tojavaMap should not contain only (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))
        intercept[TestFailedException] {
          (tojavaMap should not contain only (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          tojavaMap should not contain only ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          tojavaMap should not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (not contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should (not contain only (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))
        val e1 = intercept[TestFailedException] {
          tojavaMap should (not contain only (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOnlyElements", decorateToStringValue(tojavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        tojavaMap should (not contain only ("NICE", "TO", "MEET", "YOU"))
        intercept[TestFailedException] {
          tojavaMap should (not contain only (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should (not contain only ("NICE", "TO", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (tojavaMap should (not contain only (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))) (decided by upperCaseStringEquality)
        }
        tojavaMap should (not contain only (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))
        intercept[TestFailedException] {
          (tojavaMap should (not contain only (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          tojavaMap should (not contain only ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          tojavaMap should (not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
  }

  object `a col of javaMaps` {

    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2)))
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("hi", "hi"), Entry("he", "he")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")), javaMap(Entry("to", "to"), Entry("you", "you")))

    object `when used with contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        atLeast (2, lists) should contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        atMost (2, lists) should contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        no (lists) should contain only (Entry(3, 3), Entry(4, 4), Entry(5, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain only " + "(1=1, 2=2, 3=3)" + " (JavaMapShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (lists) should contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        }
        e3.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain only " + "(1=1, 2=2, 3=3)" + " (JavaMapShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should contain only (Entry("he", "he"), Entry("hi", "hi"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain only (Entry("ho", "ho"), Entry("hi", "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaMaps) should contain only (Entry("HE", "HE"), Entry("HI", "HI"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain only (Entry("HO", "HO"), Entry("HI", "HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should contain only (Entry("HE", "HE"), Entry("HI", "HI"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain only (Entry("HO", "HO"), Entry("HI", "HI"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaMaps) should contain only (Entry("he", "he"), Entry("hi", "hi"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain only (Entry("ho", "ho"), Entry("hi", "hi"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain only ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (contain only (Entry(3, 3), Entry(4, 4), Entry(5, 5)))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain only " + "(1=1, 2=2, 3=3)" + " (JavaMapShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (lists) should (contain only (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e4.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain only " + "(1=1, 2=2, 3=3)" + " (JavaMapShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should (contain only (Entry("he", "he"), Entry("hi", "hi")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain only (Entry("ho", "ho"), Entry("hi", "hi")))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaMaps) should (contain only (Entry("HE", "HE"), Entry("HI", "HI")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain only (Entry("HO", "HO"), Entry("HI", "HI")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain only (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain only (Entry("HO", "HO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaMaps) should (contain only (Entry("he", "he"), Entry("hi", "hi")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain only (Entry("ho", "ho"), Entry("hi", "hi")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only (Entry(1, 1), Entry(2, 2), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with not contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should not contain only (Entry("you", "you"), Entry("to", "to"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap(Entry("to", "to"), Entry("you", "you"))) + " contained only " + "(you=you, to=to)" +  " (JavaMapShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaMaps) should not contain only ("NICE", "MEET", "YOU")
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain only (Entry("YOU", "YOU"), Entry("TO", "TO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should not contain only ("NICE", "MEET", "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain only (Entry("YOU", "YOU"), Entry("TO", "TO"))) (decided by upperCaseStringEquality)
        }
        all (tojavaMaps) should not contain only (Entry(" YOU ", " YOU "), Entry(" TO ", " TO "))
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain only (Entry(" YOU ", " YOU "), Entry(" TO ", " TO "))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should not contain only ()
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (not contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should (not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should (not contain only (Entry("you", "you"), Entry("to", "to")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap(Entry("to", "to"), Entry("you", "you"))) + " contained only " + "(you=you, to=to)" + " (JavaMapShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaMaps) should (not contain only ("NICE", "MEET", "YOU"))
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain only (Entry("YOU", "YOU"), Entry("TO", "TO")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should (not contain only ("NICE", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain only (Entry("YOU", "YOU"), Entry("TO", "TO")))) (decided by upperCaseStringEquality)
        }
        all (tojavaMaps) should (not contain only (Entry(" YOU ", " YOU "), Entry(" TO ", " TO ")))
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain only (Entry(" YOU ", " YOU "), Entry(" TO ", " TO ")))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should (not contain only ())
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (tojavaMaps) should (not contain only (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
  }
}
