/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainTheSameElementsAsLogicalAndSpec extends Spec with Matchers {
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCasejavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainTheSameElementsAsLogicalAndSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))
    
    object `when used with (contain theSameElementsAs (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain theSameElementsAs Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain theSameElementsAs Set(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and contain theSameElementsAs Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e2, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e2, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) and contain theSameElementsAs Set(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain theSameElementsAs Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and (contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (tojavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) and contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) and contain theSameElementsAs Set(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain theSameElementsAs Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and (contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (fumjavaMap) and contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) and contain theSameElementsAs Set(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain theSameElementsAs (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain theSameElementsAs Set(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and legacyEqual (fumjavaMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain theSameElementsAs Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain theSameElementsAs Set(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")) and (legacyEqual (fumjavaMap)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain theSameElementsAs Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) and legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain theSameElementsAs Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")) and legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain theSameElementsAs Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")) and legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))) + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain theSameElementsAs xx and not contain theSameElementsAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain theSameElementsAs (Set("fee", "fie", "foe", "fuu")) and not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain theSameElementsAs (Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) and not contain theSameElementsAs (Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e1, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain theSameElementsAs (Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))) and not contain theSameElementsAs (Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))
        }
        checkMessageStackDepth(e1, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))) and (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and contain theSameElementsAs Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) and not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain theSameElementsAs (Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and not contain theSameElementsAs (Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (fumjavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain theSameElementsAs (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))) and not contain theSameElementsAs (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) and not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain theSameElementsAs (Set(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and not contain theSameElementsAs (Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and (not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (tojavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedSameElements", decorateToStringValue(fumjavaMap), decorateToStringValue(Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FUM", "FUM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) and not contain theSameElementsAs (Set(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain theSameElementsAs (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))) and not contain theSameElementsAs (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `col of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi"), Entry("hello", "hello")), javaMap(Entry("hi", "hi"), Entry("hello", "hello")), javaMap(Entry("hi", "hi"), Entry("hello", "hello")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain theSameElementsAs xx and contain theSameElementsAs xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsAs Set(Entry(3, 3), Entry(2, 2), Entry(1, 1)) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        atLeast (2, lists) should (contain theSameElementsAs Set(Entry(3, 3), Entry(1, 1), Entry(2, 2)) and contain theSameElementsAs Set(Entry(2, 2), Entry(3, 3), Entry(1, 1)))
        atMost (2, lists) should (contain theSameElementsAs Set(Entry(3, 3), Entry(1, 1), Entry(2, 2)) and contain theSameElementsAs Set(Entry(2, 2), Entry(3, 3), Entry(1, 1)))
        no (lists) should (contain theSameElementsAs Set(Entry(3, 3), Entry(6, 6), Entry(9, 9)) and contain theSameElementsAs Set(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (contain theSameElementsAs Set(Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (contain theSameElementsAs Set(Entry(3, 3), Entry(8, 8), Entry(5, 5)) and contain theSameElementsAs Set(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsAs Set(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain theSameElementsAs Set(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " contained the same elements as " + decorateToStringValue(Set(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry(1, 1), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain theSameElementsAs Set(Entry("hi", "hi"), Entry("hello", "hello")) and contain theSameElementsAs Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap()) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain theSameElementsAs Set(Entry("hi", "hi"), Entry("hello", "hello")) and contain theSameElementsAs Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")) and contain theSameElementsAs Set(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain theSameElementsAs Set(Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain theSameElementsAs Set(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")) and contain theSameElementsAs Set(Entry("HO", "HO"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")) and contain theSameElementsAs Set(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain theSameElementsAs Set(Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain theSameElementsAs Set(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")) and contain theSameElementsAs Set(Entry("HO", "HO"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        atLeast (2, lists) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        atMost (2, lists) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(2, 2), Entry(3, 3), Entry(1, 1)))
        no (lists) should (be (javaMap(Entry(3, 3), Entry(6, 6), Entry(9, 9))) and contain theSameElementsAs Set(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (be (javaMap(Entry(1, 1), Entry(6, 6), Entry(8, 8))) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (be (javaMap(Entry(2, 2), Entry(6, 6), Entry(8, 8))) and contain theSameElementsAs Set(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " was not equal to " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + ", but " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry(2, 2), Entry(3, 3), Entry(8, 8))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (javaMap(Entry("hey", "hey"))) and contain theSameElementsAs Set(Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap()) + " was not equal to " + decorateToStringValue(javaMap(Entry("hey", "hey"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsAs Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(1, 1), Entry(3, 3), Entry(2, 2)))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(javaMap()) + " was not equal to " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsAs Set(Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + ", but " + decorateToStringValue(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry(2, 2), Entry(3, 3), Entry(8, 8))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsAs Set(Entry("HO", "HO"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain theSameElementsAs Set(Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsAs Set(Entry("HO", "HO"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain theSameElementsAs xx and not contain theSameElementsAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain theSameElementsAs (Set(Entry(3, 3), Entry(2, 2), Entry(8, 8))) and not contain theSameElementsAs (Set(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atLeast (2, lists) should (not contain theSameElementsAs (Set(Entry(3, 3), Entry(8, 8), Entry(5, 5))) and not contain theSameElementsAs (Set(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atMost (2, lists) should (not contain theSameElementsAs (Set(2, 4, 3)) and contain theSameElementsAs (Set(Entry(4, 4), Entry(3, 3), Entry(2, 2))))
        no (list1s) should (not contain theSameElementsAs (Set(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and not contain theSameElementsAs (Set(Entry(1, 1), Entry(3, 3), Entry(2, 2))))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsAs (Set(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain theSameElementsAs (Set(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " contained the same elements as " + decorateToStringValue(Set(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsAs (Set(Entry(3, 3), Entry(6, 6), Entry(8, 8))) and not contain theSameElementsAs (Set(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry(3, 3), Entry(6, 6), Entry(8, 8))) + ", but " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " contained the same elements as " + decorateToStringValue(Set(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("hello", "hello"), Entry("hi", "hi"))) and not contain theSameElementsAs (Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("hello", "hello"), Entry("hi", "hi"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))) and not contain theSameElementsAs (Set(Entry("hello", "hello"), Entry("hi", "hi"))))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("hello", "hello"), Entry("hi", "hi"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("HI", "HI"))) and not contain theSameElementsAs (Set(Entry("HO", "HO"))))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) and not contain theSameElementsAs (Set(Entry("HO", "HO"))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("HI", "HI"))) and not contain theSameElementsAs (Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HI", "HI"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("HI", "HI"))) and not contain theSameElementsAs (Set(Entry("HO", "HO"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) and not contain theSameElementsAs (Set(Entry("HO", "HO"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain theSameElementsAs (Set(Entry("HI", "HI"))) and not contain theSameElementsAs (Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements as " + decorateToStringValue(Set(Entry("HI", "HI"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(Entry(2, 2))) and not contain theSameElementsAs (Set(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atLeast (2, lists) should (not be (javaMap(Entry(3, 3))) and not contain theSameElementsAs (Set(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atMost (2, lists) should (not be (javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) and not contain theSameElementsAs (Set(Entry(3, 3), Entry(4, 4), Entry(2, 2))))
        no (list1s) should (not be (javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and not contain theSameElementsAs (Set(Entry(1, 1), Entry(2, 2), Entry(3, 3))))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) and not contain theSameElementsAs (Set(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " was equal to " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(Entry(3, 3))) and not contain theSameElementsAs (Set(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " was not equal to " + decorateToStringValue(javaMap(Entry(3, 3))) + ", but " + decorateToStringValue(javaMap(Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " contained the same elements as " + decorateToStringValue(Set(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsAs (Set(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain theSameElementsAs (Set(Entry("hello", "hello"), Entry("hi", "hi"))))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("hello", "hello"), Entry("hi", "hi"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain theSameElementsAs (Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsAs (Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain theSameElementsAs (Set(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain theSameElementsAs (Set(Entry("HO", "HO"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsAs (Set(Entry("HELLO", "HELLO"), Entry("HI", "HI"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain theSameElementsAs (Set(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements as " + decorateToStringValue(Set(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
