/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class JavaMapShouldContainTheSameElementsInOrderAsLogicalAndSpec extends Spec with Matchers {
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val invertedjavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCasejavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val fileName: String = "JavaMapShouldContainTheSameElementsInOrderAsLogicalAndSpec.scala"
  
  object `a javaSortedMap` {
    
    val fumjavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))
    
    object `when used with (contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs Set(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) and contain theSameElementsInOrderAs Set(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (equal (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (equal (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (fumjavaSortedMap) and (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FAM", "FAM")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (equal (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (equal (tojavaSortedMap) and contain theSameElementsInOrderAs Set(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (decided by invertedjavaSortedMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) and contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain theSameElementsInOrderAs xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and legacyEqual (fumjavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) and legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))) + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and legacyEqual (fumjavaSortedMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (fumjavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")) and (legacyEqual (fumjavaSortedMap)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) and legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) and legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) and legacyEqual (fumjavaSortedMap))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) and (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not equal xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (tojavaSortedMap) and (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should (not be (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not be (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (tojavaSortedMap) and (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (not be (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (tojavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (fumjavaSortedMap) and not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))) and not contain theSameElementsInOrderAs (LinkedList(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `col of javaSortedMaps` {
    
    val list1s: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
    val lists: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
    val nils: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(), javaSortedMap(), javaSortedMap())
    val listsMap: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap())
    val hijavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(6, 6), Entry(9, 9)) and contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(8, 8), Entry(5, 5)) and contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")) and contain theSameElementsInOrderAs LinkedList(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")) and contain theSameElementsInOrderAs LinkedList(Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hello", "hello"), Entry("hi", "hi"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HO", "HO"), Entry("HELLO", "HELLO")) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HO", "HO"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) and contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (be (javaSortedMap(Entry(3, 3), Entry(2, 2), Entry(1, 1))) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (be (javaSortedMap(Entry(3, 3), Entry(6, 6), Entry(9, 9))) and contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (be (javaSortedMap(Entry(1, 1), Entry(6, 6), Entry(8, 8))) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (be (javaSortedMap(Entry(2, 2), Entry(6, 6), Entry(8, 8))) and contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain theSameElementsInOrderAs LinkedList(Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(8, 8))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (javaSortedMap(Entry("hey", "hey"))) and contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("hey", "hey"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsInOrderAs LinkedList(Entry("hello", "hello"), Entry("hi", "hi")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hello", "hello"), Entry("hi", "hi"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(listsMap(2)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and contain theSameElementsInOrderAs LinkedList(Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(8, 8))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(Entry(3, 3), Entry(2, 2), Entry(8, 8))) and not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atLeast (2, lists) should (not contain theSameElementsInOrderAs (LinkedList(Entry(3, 3), Entry(8, 8), Entry(5, 5))) and not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atMost (2, lists) should (not contain theSameElementsInOrderAs (LinkedList(2, 4, 3)) and contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        no (list1s) should (not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList(Entry(3, 3), Entry(6, 6), Entry(8, 8))) and not contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(3, 3), Entry(6, 6), Entry(8, 8))) + ", but " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HO", "HO"))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HO", "HO"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) and not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atLeast (2, lists) should (not be (javaSortedMap(Entry(3, 3))) and not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        atMost (2, lists) should (not be (javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        no (list1s) should (not be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) and not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) and not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaSortedMap(Entry(3, 3))) and not contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(3, 3))) + ", but " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hijavaSortedMaps(0)) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HO", "HO"), Entry("HELLO", "HELLO"))))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HO", "HO"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("ho", "ho"))) and not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
