/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class JavaMapShouldContainTheSameElementsInOrderAsLogicalOrSpec extends FreeSpec with Matchers {
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry
  
  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val invertedjavaSortedMapOfStringEquality = 
    new Equality[java.util.SortedMap[String, String]] {
      def areEqual(a: java.util.SortedMap[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val fileName: String = "JavaMapShouldContainTheSameElementsInOrderAsLogicalOrSpec.scala"
  
  "a javaSortedMap" - {
    
    val fumjavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))
    
    "when used with (contain theSameElementsInOrderAs xx or contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain theSameElementsInOrderAs LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or (contain theSameElementsInOrderAs LinkedList(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE"))))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain theSameElementsInOrderAs LinkedList(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FIE", "FIE"), Entry("FEE", "FEE"), Entry("FAM", "FAM"), Entry("FOE", "FOE")))), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) or contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (equal xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (equal (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (equal (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (equal (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (equal (tojavaSortedMap) or (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (equal (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (equal (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (equal (tojavaSortedMap) or contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (equal (fumjavaSortedMap) or contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (equal (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (decided by invertedjavaSortedMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    "when used with (legacyEqual xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (legacyEqual (tojavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (legacyEqual (fumjavaSortedMap) or contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed)
      }
    }

    "when used with (contain theSameElementsInOrderAs xx and legacyEqual xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")) or legacyEqual (tojavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaSortedMap))
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (tojavaSortedMap))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaSortedMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaSortedMap))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")) or legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaSortedMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")) or legacyEqual (fumjavaSortedMap))) (after being lowerCased and trimmed)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not equal xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        val e2 = intercept[TestFailedException] {
          fumjavaSortedMap should (not equal (fumjavaSortedMap) or (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not equal (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not equal (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by invertedjavaSortedMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(tojavaSortedMap)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        fumjavaSortedMap should (not be (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (not be (fumjavaSortedMap) or (not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaSortedMap should (not be (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))) (decided by upperCaseStringEquality)
        (fumjavaSortedMap should (not be (tojavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaSortedMap should (not be (fumjavaSortedMap) or not contain theSameElementsInOrderAs (LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(fumjavaSortedMap)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), decorateToStringValue(LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))), fileName, thisLineNumber - 2)
        (fumjavaSortedMap should (not contain theSameElementsInOrderAs (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))) or not contain theSameElementsInOrderAs (Set(Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU "))))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  "collection of javaSortedMaps" - {
    
    val list1s: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
    val lists: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
    val nils: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(), javaSortedMap(), javaSortedMap())
    val listsMap: Vector[java.util.SortedMap[Int, Int]] = Vector(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap())
    val hijavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")), javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello")))
    val tojavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector(javaSortedMap(Entry("you", "you"), Entry("to", "to")), javaSortedMap(Entry("you", "you"), Entry("to", "to")), javaSortedMap(Entry("you", "you"), Entry("to", "to")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(3, 2, 5) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain theSameElementsInOrderAs LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(6, 6), Entry(5, 5)) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain theSameElementsInOrderAs LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + ", and " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")))
        all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")))
        all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain theSameElementsInOrderAs LinkedList(Entry("hello", "hello"), Entry("ho", "ho")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain theSameElementsInOrderAs LinkedList(Entry("hello", "hello"), Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hello", "hello"), Entry("ho", "ho"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("hello", "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")) or contain theSameElementsInOrderAs LinkedList(Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO")) or contain theSameElementsInOrderAs LinkedList(Entry("hello", "hello"), Entry("ho", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hello", "hello"), Entry("ho", "ho"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (be xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (be (javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) or contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        all (list1s) should (be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) or contain theSameElementsInOrderAs LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) or contain theSameElementsInOrderAs LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + ", and " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))
        all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO")))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (be (javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) or contain theSameElementsInOrderAs LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was not equal to " + decorateToStringValue(javaSortedMap(Entry("ho", "ho"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(Entry(3, 3), Entry(2, 2), Entry(8, 8))) or not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) or not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(Entry(3, 3), Entry(2, 2), Entry(8, 8))) or not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))) or not contain theSameElementsInOrderAs (LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + ", and " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(2, 2), Entry(3, 3), Entry(4, 4))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hello", "hello"), Entry("hi", "hi"))))
        all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hello", "hello"), Entry("hi", "hi"))))
        all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hello", "hello"), Entry("hi", "hi"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hello", "hello"), Entry("hi", "hi"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HI", "HI"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("hi", "hi"), Entry("hello", "hello"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) or not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        all (list1s) should (not be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) or not contain theSameElementsInOrderAs (LinkedList(Entry(8, 8), Entry(3, 3), Entry(4, 4))))
        all (list1s) should (not be (javaSortedMap(Entry(2, 2))) or not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) or not contain theSameElementsInOrderAs (LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + ", and " + decorateToStringValue(javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))))
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))))
        all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HELLO", "HELLO"), Entry("HO", "HO"))))) (decided by upperCaseStringEquality)
        (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hello", "hello"), Entry("ho", "ho"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (not be (javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) or not contain theSameElementsInOrderAs (LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " was equal to " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + ", and " + decorateToStringValue(javaSortedMap(Entry("hi", "hi"), Entry("hello", "hello"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("HI", "HI"), Entry("HELLO", "HELLO"))), thisLineNumber - 2, hijavaSortedMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
