/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class JavaMapShouldContainTheSameElementsInOrderAsSpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => true
            case _ => false
        }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Uniformity[java.util.Map.Entry[String, String]] = new Uniformity[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (_: String, _: String) => true
            case _ => false
          }
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
import org.scalatest.Entry

  object `a javaSortedMap` {

    val fumjavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaSortedMap: java.util.SortedMap[String, String] = javaSortedMap(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))

    object `when used with contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))
        intercept[TestFailedException] {
          fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE "))
        }
        (fumjavaSortedMap should contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee")))
        val e1 = intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumjavaSortedMap), LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))
        intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("FUM", "FUM"), Entry("FOE", "FOE"), Entry("FIE", "FIE"), Entry("FEE", "FEE")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))
        }
        (fumjavaSortedMap should (contain theSameElementsInOrderAs LinkedList(Entry(" FUM ", " FUM "), Entry(" FOE ", " FOE "), Entry(" FIE ", " FIE "), Entry(" FEE ", " FEE ")))) (after being lowerCased and trimmed)
      }
    }

    object `when used with not contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy")))
        val e1 = intercept[TestFailedException] {
          tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedSameElementsInOrder", decorateToStringValue(tojavaSortedMap), LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList("YOU", "TO", "BIRTHDAY", "HAPPY"))
        intercept[TestFailedException] {
          tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList("YOU", "TO", "BIRTHDAY", "HAPPY"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))) (decided by upperCaseStringEquality)
        }
        tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList(Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))
        intercept[TestFailedException] {
          (tojavaSortedMap should not contain theSameElementsInOrderAs (LinkedList(Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))))
        val e1 = intercept[TestFailedException] {
          tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedSameElementsInOrder", decorateToStringValue(tojavaSortedMap), LinkedList(Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList("YOU", "TO", "BIRTHDAY", "HAPPY")))
        intercept[TestFailedException] {
          tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList("YOU", "TO", "BIRTHDAY", "HAPPY")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))))) (decided by upperCaseStringEquality)
        }
        tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))))
        intercept[TestFailedException] {
          (tojavaSortedMap should (not contain theSameElementsInOrderAs (LinkedList(Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a col of javaSortedMaps` {

    val list1s: Vector[java.util.SortedMap[Int, Int]] = Vector( javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
    val lists: Vector[java.util.SortedMap[Int, Int]] = Vector( javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4)))
    val listsMap: Vector[java.util.SortedMap[Int, Int]] = Vector( javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap(Entry(1, 1), Entry(2, 2), Entry(3, 3)), javaSortedMap())
    val hijavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector( javaSortedMap(Entry("hi", "hi"), Entry("he", "he")), javaSortedMap(Entry("hi", "hi"), Entry("he", "he")), javaSortedMap(Entry("hi", "hi"), Entry("he", "he")))
    val tojavaSortedMaps: Vector[java.util.SortedMap[String, String]] = Vector( javaSortedMap(Entry("to", "to"), Entry("you", "you")), javaSortedMap(Entry("to", "to"), Entry("you", "you")), javaSortedMap(Entry("to", "to"), Entry("you", "you")))

    object `when used with contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))
        atLeast (2, lists) should contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))
        atMost (2, lists) should contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))
        no (lists) should contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " (JavaMapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (listsMap) should contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))
        }
        e3.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(listsMap(2)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " (JavaMapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("he", "he"))
        intercept[TestFailedException] {
          all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("ho", "ho"))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HE", "HE"))
        intercept[TestFailedException] {
          all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HO", "HO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HE", "HE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HO", "HO"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("he", "he"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaSortedMaps) should contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("ho", "ho"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
    }

    object `when used with (contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (listsMap) should (contain theSameElementsInOrderAs LinkedList(Entry(3, 3), Entry(4, 4), Entry(5, 5)))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaSortedMap(Entry(2, 2), Entry(3, 3), Entry(4, 4))) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " (JavaMapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should (contain theSameElementsInOrderAs LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e4.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(listsMap(2)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry(1, 1), Entry(2, 2), Entry(3, 3))) + " (JavaMapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("he", "he")))
        intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("he", "he"), Entry("hi", "hi")))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HE", "HE")))
        intercept[TestFailedException] {
          all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HO", "HO")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HE", "HE")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("HI", "HI"), Entry("HO", "HO")))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("hi", "hi"), Entry("he", "he")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaSortedMaps) should (contain theSameElementsInOrderAs LinkedList(Entry("he", "he"), Entry("hi", "hi")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
    }

    object `when used with not contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry("you", "you"), Entry("to", "to")))
        val e1 = intercept[TestFailedException] {
          all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaSortedMap(Entry("to", "to"), Entry("you", "you"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("to", "to"), Entry("you", "you"))) +  " (JavaMapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaSortedMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry("YOU", "YOU"), Entry("TO", "TO")))
        intercept[TestFailedException] {
          all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry("TO", "TO"), Entry("YOU", "YOU")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry("YOU", "YOU"), Entry("TO", "TO")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry("TO", "TO"), Entry("YOU", "YOU")))) (decided by upperCaseStringEquality)
        }
        all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))
        intercept[TestFailedException] {
          (all (tojavaSortedMaps) should not contain theSameElementsInOrderAs (LinkedList(Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("you", "you"), Entry("to", "to"))))
        val e1 = intercept[TestFailedException] {
          all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("to", "to"), Entry("you", "you"))))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaSortedMap(Entry("to", "to"), Entry("you", "you"))) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(Entry("to", "to"), Entry("you", "you"))) + " (JavaMapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaSortedMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("YOU", "YOU"), Entry("TO", "TO"))))
        intercept[TestFailedException] {
          all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("TO", "TO"), Entry("YOU", "YOU"))))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("YOU", "YOU"), Entry("TO", "TO"))))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry("TO", "TO"), Entry("YOU", "YOU"))))) (decided by upperCaseStringEquality)
        }
        all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))))
        intercept[TestFailedException] {
          (all (tojavaSortedMaps) should (not contain theSameElementsInOrderAs (LinkedList(Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
