/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAllOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainAllOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fex" -> "fex", "fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("too" -> "too", "you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy")
    
    object `when used with (contain allOf (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FAM" -> "FAM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumList should (contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain allOf () and contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain allOf () and contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain allOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain allOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }

    object `when used with (contain allOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain allOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain allOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU") and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(HAPPY,HAPPY), (BIRTHDAY,BIRTHDAY), (TO,TO), (YOU,YOU)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain allOf () and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (not contain allOf xx and not contain allOf xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain allOf ("fee", "fie", "foe", "fuu") and not contain allOf ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)") + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)") + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumList should (contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain allOf () and not contain allOf ("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain allOf ("fie", "fee", "fuu", "foe") and not contain allOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and not contain allOf ("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain allOf ("fie", "fee", "fuu", "foe") and not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain allOf ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain allOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain allOf ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain allOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain allOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
  }
  
  object `col of Lists` {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello"), scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello"), scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain allOf xx and contain allOf xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain allOf (3 -> 3, 2 -> 2, 1 -> 1) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        atLeast (2, lists) should (contain allOf (3 -> 3, 1 -> 1, 2 -> 2) and contain allOf (2 -> 2, 3 -> 3, 1 -> 1))
        atMost (2, lists) should (contain allOf (3 -> 3, 1 -> 1, 2 -> 2) and contain allOf (2 -> 2, 3 -> 3, 1 -> 1))
        no (lists) should (contain allOf (3 -> 3, 6 -> 6, 9 -> 9) and contain allOf (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain allOf (1 -> 1, 2 -> 2, 3 -> 3) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " did not contain all of " + "((1,1), (2,2), (3,3))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain allOf (1 -> 1, 2 -> 2, 3 -> 3) and contain allOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + " contained all of " + "((1,1), (2,2), (3,3))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + " did not contain all of " + "((1,1), (3,3), (4,4))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain allOf ("hi" -> "hi", "hello" -> "hello") and contain allOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((hi,hi), (hello,hello))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((ho,ho), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain allOf ("HELLO" -> "HELLO", "HI" -> "HI") and contain allOf ("HI" -> "HI", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain allOf ("HO" -> "HO", "HELLO" -> "HELLO") and contain allOf ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain allOf ("HELLO" -> "HELLO", "HI" -> "HI") and contain allOf ("HO" -> "HO", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HELLO,HELLO), (HI,HI))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain allOf ("HELLO" -> "HELLO", "HI" -> "HI") and contain allOf ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain allOf ("HO" -> "HO", "HELLO" -> "HELLO") and contain allOf ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain allOf ("HELLO" -> "HELLO", "HI" -> "HI") and contain allOf ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HELLO,HELLO), (HI,HI))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain allOf () and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain allOf (1 -> 1, 3 -> 3, 2 -> 2) and contain allOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain allOf (3, 2, 2, 1) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain allOf (1 -> 1, 3 -> 3, 2 -> 2) and contain allOf (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        atLeast (2, lists) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        atMost (2, lists) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (2 -> 2, 3 -> 3, 1 -> 1))
        no (lists) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 6 -> 6, 9 -> 9)) and contain allOf (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 6 -> 6, 8 -> 8)) and contain allOf (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2, 6 -> 6, 8 -> 8)) and contain allOf (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + " did not contain all of " + "((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (scala.collection.mutable.LinkedHashMap("hey" -> "hey")) and contain allOf ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hey" -> "hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and contain allOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((ho,ho), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) + " did not contain all of " + "((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) and contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and contain allOf ("HO" -> "HO", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) and contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and contain allOf ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0)) and contain allOf (3, 2, 2, 1))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (not contain allOf xx and not contain allOf xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain allOf (3 -> 3, 2 -> 2, 8 -> 8) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not contain allOf (3 -> 3, 8 -> 8, 5 -> 5) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not contain allOf (2 -> 2, 4 -> 4, 3 -> 3) and contain allOf (4 -> 4, 3 -> 3, 2 -> 2))
        no (list1s) should (not contain allOf (1 -> 1, 2 -> 2, 3 -> 3) and not contain allOf (1 -> 1, 3 -> 3, 2 -> 2))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain allOf (2 -> 2, 3 -> 3, 4 -> 4) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " contained all of " + "((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain allOf (3 -> 3, 6 -> 6, 8 -> 8) and not contain allOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " did not contain all of " + "((3,3), (6,6), (8,8))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " contained all of " + "((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain allOf ("hello" -> "hello", "hi" -> "hi") and not contain allOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((hello,hello), (hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain allOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy") and not contain allOf ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((ho,ho), (hey,hey), (howdy,howdy))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((hello,hello), (hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain allOf ("TO") and not contain allOf ("HO" -> "HO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain allOf ("HELLO" -> "HELLO", "HI" -> "HI") and not contain allOf ("HO" -> "HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain allOf ("HO" -> "HO") and not contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain allOf ("TO") and not contain allOf ("HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain allOf ("HELLO" -> "HELLO", "HI" -> "HI") and not contain allOf ("HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain allOf ("HO" -> "HO") and not contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " did not contain all of " + "((HO,HO))" + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain allOf () and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain allOf (8 -> 8, 3 -> 3, 4 -> 4) and not contain allOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain allOf (3, 2, 2, 1) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain allOf (8 -> 8, 3 -> 3, 4 -> 4) and not contain allOf (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("allOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(4 -> 4, 3 -> 3, 2 -> 2)) and not contain allOf (3 -> 3, 4 -> 4, 2 -> 2))
        no (list1s) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1)) and not contain allOf (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) and not contain allOf (8 -> 8, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and not contain allOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " contained all of " + "((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and not contain allOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain allOf ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((hello,hello), (hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain allOf ("HO" -> "HO", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and not contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain allOf ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HI,HI), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain allOf ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) and not contain allOf ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain allOf ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "hello" -> "hello")) + " contained all of " + "((HI,HI), (HELLO,HELLO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain allOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain allOf (3, 2, 2, 1))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("allOfDuplicate")))
      }
    }
  }
}
