/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAtLeastOneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainAtLeastOneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to")
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fum" -> "fum") and contain atLeastOneOf ("fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and (contain atLeastOneOf ("fum" -> "fum")))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ("fum" -> "fum") and contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atLeastOneOf () and contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and contain atLeastOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (equal (fumList) and contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atLeastOneOf ("fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain atLeastOneOf ("fum" -> "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (legacyEqual (fumList) and contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atLeastOneOf ("fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain atLeastOneOf ("fum" -> "fum")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fum" -> "fum") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fum" -> "fum") and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") and legacyEqual (fumList))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ("fum" -> "fum") and legacyEqual (fumList))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and legacyEqual (toList))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atLeastOneOf() and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (not contain atLeastOneOf ("fum" -> "fum") and not contain atLeastOneOf ("fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and not contain atLeastOneOf ("fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ("fum" -> "fum") and (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain atLeastOneOf ("fum" -> "fum") and not contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atLeastOneOf ("fum" -> "fum") and not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum)") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and not contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atLeastOneOf () and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atLeastOneOf ("fie", "fee", "fuu", "foe") and not contain atLeastOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atLeastOneOf ("fie", "fee", "fuu", "foe") and not contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (not equal (toList) and not contain atLeastOneOf ("fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atLeastOneOf ("fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (not be (toList) and not contain atLeastOneOf ("fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atLeastOneOf ("fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain atLeastOneOf ("fum" -> "fum"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain atLeastOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf (3 -> 3, 2 -> 2, 1 -> 1) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atLeastOneOf (3 -> 3, 1 -> 1, 5 -> 5) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain atLeastOneOf (3 -> 3, 2 -> 2, 8 -> 8) and contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain atLeastOneOf (3 -> 3, 6 -> 6, 9 -> 9) and contain atLeastOneOf (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (contain atLeastOneOf (1 -> 1, 2 -> 2, 8 -> 8) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (contain atLeastOneOf(3 -> 3, 8 -> 8, 5 -> 5) and contain atLeastOneOf (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1 -> 1, 6 -> 6, 8 -> 8) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain at least one of ((1,1), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1 -> 1, 2 -> 2, 8 -> 8) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((1,1), (2,2), (8,8)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain at least one of ((1,1), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf ("hi" -> "hi", "hello" -> "hello") and contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " did not contain at least one of ((hi,hi), (hello,hello))", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ("hi" -> "hi", "hello" -> "hello") and contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((hi,hi), (hello,hello)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((ho,ho), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4) and contain atLeastOneOf (1 -> 1, 3 -> 3, Map()))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " did not contain at least one of ((1,1), (3,3), (4,4))", thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (1 -> 1, 2 -> 2, 8 -> 8) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((1,1), (2,2), (8,8)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain at least one of ((1,1), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") and contain atLeastOneOf ("he" -> "he"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ("hi" -> "hi") and contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") and contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") and contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ("hi" -> "hi") and contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") and contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf () and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4) and contain atLeastOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (3, 2, 2, 1) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4) and contain atLeastOneOf (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (be (scala.collection.mutable.LinkedHashMap(8 -> 8)) and contain atLeastOneOf (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (be (scala.collection.mutable.LinkedHashMap(8 -> 8)) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (be (scala.collection.mutable.LinkedHashMap(8 -> 8)) and contain atLeastOneOf (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " did not contain at least one of ((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (scala.collection.mutable.LinkedHashMap("hey" -> "hey")) and contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hey" -> "hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((ho,ho), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (1 -> 1, 3 -> 3, Map()))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " did not contain at least one of ((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain atLeastOneOf ("he" -> "he"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain atLeastOneOf (3, 2, 2, 1))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atLeastOneOf (3 -> 3, 2 -> 2, 8 -> 8) and not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not contain atLeastOneOf (3 -> 3, 8 -> 8, 5 -> 5) and not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not contain atLeastOneOf (3 -> 3, 6 -> 6, 8 -> 8) and contain atLeastOneOf (5 -> 5, 3 -> 3, 4 -> 4))
        no (lists) should (not contain atLeastOneOf (1 -> 1, 2 -> 2, 9 -> 9) and not contain atLeastOneOf (2 -> 2, 1 -> 1, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (2 -> 2, 6 -> 6, 8 -> 8) and not contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((2,2), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (3 -> 3, 6 -> 6, 8 -> 8) and not contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain at least one of ((3,3), (6,6), (8,8)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi", "hello" -> "hello") and not contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((hi,hi), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy") and not contain atLeastOneOf ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((ho,ho), (hey,hey), (howdy,howdy)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((hi,hi), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") and not contain atLeastOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho") and not contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") and not contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") and not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho") and not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") and not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf () and not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4) and not contain atLeastOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf (3, 2, 2, 1) and not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4) and not contain atLeastOneOf (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and contain atLeastOneOf (5 -> 5, 3 -> 3, 4 -> 4))
        no (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and not contain atLeastOneOf (2 -> 2, 1 -> 1, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and not contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and not contain atLeastOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain atLeastOneOf ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((hi,hi), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain atLeastOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and not contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain atLeastOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain atLeastOneOf (3, 2, 2, 1))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atLeastOneOfDuplicate")))
      }
    }
  }
}
