/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAtMostOneOfLogicalAndSpec extends Spec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainAtMostOneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you")
    
    object `when used with (contain atMostOneOf (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") and contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") and contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") and contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf () and contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") and contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") and contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and legacyEqual (fumList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and (legacyEqual (toList)))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") and legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf() and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain atMostOneOf (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") and not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fam,fam), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU") and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and (not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") and contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf () and not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and not contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fam,fam), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fam,fam), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain atMostOneOf (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf (3, 4, 1) and contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atMostOneOf (3 -> 3, 2 -> 2, 5 -> 5) and contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain atMostOneOf (3 -> 3, 2 -> 2, 8 -> 8) and contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) and contain atMostOneOf (3 -> 3, 2 -> 2, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4) and contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4) and contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ((1,1), (3,3), (4,4)), but " + decorateToStringValue(lists(2)) + " did not contain at most one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ("hi" -> "hi", "hello" -> "hello") and contain atMostOneOf ("hi" -> "hi", "he" -> "he"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((hi,hi), (hello,hello)), but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((hi,hi), (he,he))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (1 -> 1, 6 -> 6, 8 -> 8) and contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ((1,1), (6,6), (8,8)), but " + decorateToStringValue(lists(2)) + " did not contain at most one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") and contain atMostOneOf ("HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") and contain atMostOneOf ("HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") and contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO)), but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") and contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO)), but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () and contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4) and contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3) and contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4) and contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (be (scala.collection.mutable.LinkedHashMap(8 -> 8)) and contain atMostOneOf (3 -> 3, 2 -> 2, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) + ", but " + decorateToStringValue(list1s(0)) + " did not contain at most one of ((1,1), (2,2), (3,3))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and contain atMostOneOf ("hi" -> "hi", "he" -> "he"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) + ", but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((hi,hi), (he,he))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) + ", but " + decorateToStringValue(list1s(0)) + " did not contain at most one of ((1,1), (2,2), (3,3))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and contain atMostOneOf ("HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and contain atMostOneOf ("HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) + ", but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) + ", but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain atMostOneOf (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atMostOneOf (3 -> 3, 2 -> 2, 1 -> 1) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        atMost (2, lists) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8) and contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        no (lists) should (not contain atMostOneOf (2 -> 2, 6 -> 6, 8 -> 8) and not contain atMostOneOf (3 -> 3, 6 -> 6, 8 -> 8))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ((1,1), (2,2), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) and not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of ((1,1), (2,2), (3,3)), but " + decorateToStringValue(lists(2)) + " contained at most one of ((1,1), (2,2), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ("hi" -> "hi", "hello" -> "hello") and not contain atMostOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((hi,hi), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ("hi" -> "hi", "he" -> "he") and not contain atMostOneOf ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((hi,hi), (he,he)), but " + decorateToStringValue(hiLists(0)) + " contained at most one of ((hi,hi), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") and not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") and not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") and not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE)), but " + decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") and not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") and not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") and not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE)), but " + decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) and not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) and not contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        atMost (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) and contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        no (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) and not contain atMostOneOf (3 -> 3, 6 -> 6, 8 -> 8))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) and not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)) and not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)) + ", but " + decorateToStringValue(lists(2)) + " contained at most one of ((1,1), (2,2), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and not contain atMostOneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) and not contain atMostOneOf ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) + ", but " + decorateToStringValue(hiLists(0)) + " contained at most one of ((hi,hi), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HO" -> "HO")) and not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and not contain atMostOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) and not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) + ", but " + decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HO" -> "HO")) and not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) and not contain atMostOneOf ("hi" -> "hi"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) and not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) + ", but " + decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) and not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) and not contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
