/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAtMostOneOfLogicalOrSpec extends FreeSpec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainAtMostOneOfLogicalOrSpec.scala"
  
  "a List" - {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you")
    
      "when used with (contain atMostOneOf (...) or contain atMostOneOf (...)) syntax" - {
      
        "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") or contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf () or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") or contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") or contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (equal (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (equal (fumList) or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        fumList should (equal (toList) or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        fumList should (equal (fumList) or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        fumList should (equal (toList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        fumList should (equal (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (equal (toList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) or contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (legacyEqual (...) and contain theMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (legacyEqual (fumList) or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        fumList should (legacyEqual (toList) or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        fumList should (legacyEqual (fumList) or contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        fumList should (legacyEqual (toList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        fumList should (legacyEqual (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (legacyEqual (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) or contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    "when used with (contain oneOf (...) and legacyEqual (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") or legacyEqual (fumList))
        fumList should (contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (fumList))
        fumList should (contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") or legacyEqual (fumList))
        fumList should (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") or legacyEqual (fumList))
        fumList should (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FaM ") or legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf() or legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") or legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain atMostOneOf (...) and not contain atMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\"") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU") or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUU" -> "FUU", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU") or not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU") or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUU" -> "FUU", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU") or not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") or contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf () or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or not contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not equal (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not equal (toList) or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not equal (fumList) or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not equal (toList) or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (not equal (fumList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (not equal (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUU" -> "FUU", "FOE" -> "FOE"))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not equal (fumList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUU" -> "FUU", "FOE" -> "FOE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) or not contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not be (toList) or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not be (fumList) or not contain atMostOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not be (toList) or not contain atMostOneOf("fee", "fie", "foe", "fuu"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (not be (fumList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))
        fumList should (not be (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUU" -> "FUU", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not be (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain atMostOneOf ("FIE" -> "FIE", "FEE" -> "FEE", "FUU" -> "FUU", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUU" -> "FUU"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUU,FUU)"), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) or not contain atMostOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  "collection of Lists" - {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain oneOf (..) and contain oneOf (..)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain atMostOneOf (1 -> 1, 6 -> 6, 8 -> 8) or contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) or contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain atMostOneOf (1 -> 1, 6 -> 6, 8 -> 8) or contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        
        atLeast (2, lists) should (contain atMostOneOf (3 -> 3, 2 -> 2, 5 -> 5) or contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) or contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atMostOneOf (3 -> 3, 2 -> 2, 5 -> 5) or contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4) or contain atMostOneOf (4 -> 4, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of ((2,2), (3,3), (4,4)), and " + decorateToStringValue(lists(2)) + " did not contain at most one of ((4,4), (3,3), (2,2))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") or contain atMostOneOf ("HO" -> "HO", "HI" -> "HI"))
        all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or contain atMostOneOf ("HO" -> "HO", "HI" -> "HI"))
        all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") or contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE)), and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HE,HE), (HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") or contain atMostOneOf ("HO" -> "HO", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or contain atMostOneOf ("HO" -> "HO", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO") or contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE)), and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HE,HE), (HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () or contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4) or contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3) or contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4) or contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (be (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) or contain atMostOneOf (1 -> 1, 6 -> 6, 8 -> 8))
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) or contain atMostOneOf (1 -> 1, 6 -> 6, 8 -> 8))
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) or contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) or contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) + ", and " + decorateToStringValue(list1s(0)) + " did not contain at most one of ((1,1), (2,2), (3,3))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("HO" -> "HO")) or contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("HO" -> "HO")) or contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HO" -> "HO")) + ", and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("HO" -> "HO")) or contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("HO" -> "HO")) or contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HO" -> "HO")) + ", and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ((HI,HI), (HE,HE))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) or contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) or contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain oneOf (..) and not contain oneOf (..))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain atMostOneOf (3 -> 3, 2 -> 2, 1 -> 1) or not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8) or not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (not contain atMostOneOf (3 -> 3, 2 -> 2, 1 -> 1) or not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 8 -> 8) or not contain atMostOneOf (8, 2, 1))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ((1,1), (2,2), (8,8)), and " + decorateToStringValue(lists(2)) + " contained at most one of (8, 2, 1)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))
        all (hiLists) should (not contain atMostOneOf ("hi" -> "hi") or not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))
        all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or not contain atMostOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ("hi" -> "hi") or not contain atMostOneOf ("he" -> "he"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((hi,hi)), and " + decorateToStringValue(hiLists(0)) + " contained at most one of ((he,he))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain atMostOneOf ("hi" -> "hi") or not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE") or not contain atMostOneOf ("hi" -> "hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atMostOneOf ("hi" -> "hi") or not contain atMostOneOf ("he" -> "he"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ((hi,hi)), and " + decorateToStringValue(hiLists(0)) + " contained at most one of ((he,he))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () or not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) or not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3) or not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3) or not contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain atMostOneOf (...))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) or not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) or not contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) or not contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) or not contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2)) + ", and " + decorateToStringValue(list1s(0)) + " contained at most one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) + ", and " + decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or not contain atMostOneOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) or not contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) + ", and " + decorateToStringValue(hiLists(0)) + " contained at most one of ((HI,HI), (HO,HO))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) or not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3)) or not contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
