/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAtMostOneOfSpec extends Spec with Matchers {

  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }

  object `a List` {

    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you")

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam")
        val e1 = intercept[TestFailedException] {
          fumList should contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumList should contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")
        intercept[TestFailedException] {
          fumList should contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumList should contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
          
        }
        fumList should contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))
        intercept[TestFailedException] {
          fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
        }
        fumList should (contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          fumList should not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumList should not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")
        intercept[TestFailedException] {
          fumList should not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")) (decided by upperCaseStringEquality)
        }
        (fumList should not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumList should not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality)
        }
        (fumList should (not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumList should (not contain atMostOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }

  object `a collection of Lists` {

    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"))

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4)
        atLeast (2, lists) should contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4)
        atMost (2, lists) should contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4)
        no (lists) should contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of ((2,2), (3,3), (4,4)) (MapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain atMostOneOf ("hi", "ho")
        intercept[TestFailedException] {
          all (hiLists) should contain atMostOneOf ("hi" -> "hi", "he" -> "he")
        }
        implicit val use = upperCaseStringEquality
        all (hiLists) should contain atMostOneOf ("HI" -> "HI", "HO" -> "HO")
        intercept[TestFailedException] {
          all (hiLists) should contain atMostOneOf ("HI" -> "HI", "HE" -> "HE")
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain atMostOneOf ("hi", "ho")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain atMostOneOf ("hi" -> "hi", "he" -> "he")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should contain atMostOneOf ("hi", "HE")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should contain atMostOneOf ("hi" -> "hi", "he" -> "he")) (decided by defaultEquality[(String, String)])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain atMostOneOf (1 -> 1, 2 -> 2, 3 -> 3))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of ((2,2), (3,3), (4,4)) (MapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain atMostOneOf ("hi", "ho"))
        intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ("hi" -> "hi", "he" -> "he"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HO" -> "HO"))
        intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ("HI" -> "HI", "HE" -> "HE"))
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atMostOneOf ("hi", "ho"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ("hi" -> "hi", "he" -> "he"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should (contain atMostOneOf ("hi", "HE"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ("hi" -> "hi", "he" -> "he"))) (decided by defaultEquality[(String, String)])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain atMostOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toLists(0)) + " contained at most one of ((fee,fee), (fie,fie), (foe,foe), (fum,fum)) (MapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should not contain atMostOneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        intercept[TestFailedException] {
          all (toLists) should not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain atMostOneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) (decided by upperCaseStringEquality)
        }
        (all (toLists) should not contain atMostOneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toLists) should not contain atMostOneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain atMostOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toLists(0)) + " contained at most one of ((fee,fee), (fie,fie), (foe,foe), (fum,fum)) (MapShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should (not contain atMostOneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        intercept[TestFailedException] {
          all (toLists) should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain atMostOneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain atMostOneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        (all (toLists) should (not contain atMostOneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toLists) should (not contain atMostOneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain atMostOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
