/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainInOrderOnlyLogicalAndSpec extends Spec with Matchers {

  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.SortedMap[String, String]] {
      def areEqual(a: scala.collection.SortedMap[String, String], b: Any): Boolean = a != b
    }

  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[scala.collection.SortedMap[String, String]] {
      def areEqual(a: scala.collection.SortedMap[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  val fileName: String = "MapShouldContainInOrderOnlyLogicalAndSpec.scala"

  object `a List` {

    val fumList: scala.collection.SortedMap[String, String] = sortedMap("fum" -> "fum", "fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fie" -> "fie", "fee" -> "fee")
    val toList: scala.collection.SortedMap[String, String] = sortedMap("you" -> "you", "you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy", "happy" -> "happy")

    object `when used with (contain inOrderOnly xx and contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(fum,fum), (foe,foe), (fie,fie), (fee,fee)") + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)") + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }

      def `should use an explicitly provided Equality` {
        (fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)") + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        (fumList should (contain inOrderOnly (" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE ") and contain inOrderOnly (" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain inOrderOnly () and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (equal xx and contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain inOrderOnly (" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (legacyEqual xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain inOrderOnly (" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (contain inOrderOnly xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(fum,fum), (foe,foe), (fie,fie), (fee,fee)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain inOrderOnly ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU") and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(HAPPY,HAPPY), (BIRTHDAY,BIRTHDAY), (TO,TO), (YOU,YOU)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain inOrderOnly (" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE ") and legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain inOrderOnly () and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not contain inOrderOnly xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") and not contain inOrderOnly ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(fum,fum), (foe,foe), (fie,fie), (fee,fee)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(fum,fum), (foe,foe), (fie,fie), (fee,fee)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and (not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain inOrderOnly () and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum") and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not equal xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(fum,fum), (foe,foe), (fie,fie), (fee,fee)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain inOrderOnly ("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(fum,fum), (foe,foe), (fie,fie), (fee,fee)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain inOrderOnly ("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumList), "(FUM,FUM), (FOE,FOE), (FIE,FIE), (FEE,FEE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain inOrderOnly ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain inOrderOnly (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain inOrderOnly (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain inOrderOnly ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
  }
  
  object `col of Lists` {
    
    val list1s: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
    val lists: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(1 -> 1, 2 -> 2, 3 -> 3, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3, 3 -> 3), sortedMap(2 -> 2, 3 -> 3, 4 -> 4))
    val nils: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(), sortedMap(), sortedMap())
    val listsMap: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap())
    val hiLists: Vector[scala.collection.SortedMap[String, String]] = Vector(sortedMap("hi" -> "hi", "hello" -> "hello"), sortedMap("hi" -> "hi", "hello" -> "hello"), sortedMap("hi" -> "hi", "hello" -> "hello"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain inOrderOnly xx and contain inOrderOnly xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        atMost (2, lists) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        no (lists) should (contain inOrderOnly (3 -> 3, 6 -> 6, 9 -> 9) and contain inOrderOnly (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (contain inOrderOnly (1 -> 1, 2 -> 2, 8 -> 8) and contain inOrderOnly (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (contain inOrderOnly (3 -> 3, 8 -> 8, 5 -> 5) and contain inOrderOnly (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " did not contain only " + "((1,1), (2,2), (3,3))" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + " contained only " + "((1,1), (2,2), (3,3))" + " in order" + ", but " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + " did not contain only " + "((1,1), (3,3), (4,4))" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain inOrderOnly ("hi" -> "hi", "hello" -> "hello") and contain inOrderOnly ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " did not contain only " + "((hi,hi), (hello,hello))" + " in order", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain inOrderOnly ("hi" -> "hi", "hello" -> "hello") and contain inOrderOnly ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hi,hi), (hello,hello))" + " in order" + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((hello,hello), (hi,hi))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO") and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain inOrderOnly ("HO" -> "HO", "HELLO" -> "HELLO") and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO") and contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order" + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HELLO,HELLO), (HI,HI))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }

      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO") and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain inOrderOnly ("HO" -> "HO", "HELLO" -> "HELLO") and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO") and contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order" + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HELLO,HELLO), (HI,HI))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly () and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and contain inOrderOnly (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (be xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3, 3 -> 3)) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        atMost (2, lists) should (be (sortedMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        no (lists) should (be (sortedMap(3 -> 3, 6 -> 6, 9 -> 9)) and contain inOrderOnly (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (be (sortedMap(1 -> 1, 6 -> 6, 8 -> 8)) and contain inOrderOnly (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (be (sortedMap(2 -> 2, 6 -> 6, 8 -> 8)) and contain inOrderOnly (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3, 3 -> 3)) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " was not equal to " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3, 3 -> 3)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) and contain inOrderOnly (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + " was equal to " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + ", but " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + " did not contain only " + "((2,2), (3,3), (8,8))" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (sortedMap("hey" -> "hey")) and contain inOrderOnly ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(nils(0)) + " was not equal to " + decorateToStringValue(sortedMap("hey" -> "hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) and contain inOrderOnly ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((hello,hello), (hi,hi))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) and contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(listsMap(2)) + " was not equal to " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3)), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) and contain inOrderOnly (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + " was equal to " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + ", but " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) + " did not contain only " + "((2,2), (3,3), (8,8))" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (sortedMap("HI" -> "HI", "HELLO" -> "HELLO")) and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) and contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HELLO,HELLO), (HI,HI))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (sortedMap("HI" -> "HI", "HELLO" -> "HELLO")) and contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) and contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HELLO,HELLO), (HI,HI))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) and contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) and contain inOrderOnly (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }

    object `when used with (not contain inOrderOnly xx and not contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain inOrderOnly (3 -> 3, 2 -> 2, 8 -> 8) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not contain inOrderOnly (3 -> 3, 8 -> 8, 5 -> 5) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not contain inOrderOnly (2 -> 2, 4 -> 4, 3 -> 3) and contain inOrderOnly (2 -> 2, 3 -> 3, 4 -> 4))
        no (list1s) should (not contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3) and not contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain inOrderOnly (2 -> 2, 3 -> 3, 4 -> 4) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " contained only " + "((2,2), (3,3), (4,4))" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain inOrderOnly (3 -> 3, 6 -> 6, 8 -> 8) and not contain inOrderOnly (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " did not contain only " + "((3,3), (6,6), (8,8))" + " in order" + ", but " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " contained only " + "((2,2), (3,3), (4,4))" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain inOrderOnly ("hi" -> "hi", "hello" -> "hello") and not contain inOrderOnly ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hi,hi), (hello,hello))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain inOrderOnly ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy") and not contain inOrderOnly ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((ho,ho), (hey,hey), (howdy,howdy))" + " in order" + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hi,hi), (hello,hello))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI") and not contain inOrderOnly ("HELLO" -> "HELLO", "HO" -> "HO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO") and not contain inOrderOnly ("HO" -> "HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI") and not contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HELLO,HELLO), (HI,HI))" + " in order" + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI") and not contain inOrderOnly ("HELLO" -> "HELLO", "HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO") and not contain inOrderOnly ("HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI") and not contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HELLO,HELLO), (HI,HI))" + " in order" + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly () and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4) and not contain inOrderOnly ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4) and not contain inOrderOnly (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
    
    object `when used with (not be xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (sortedMap(2 -> 2)) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not be (sortedMap(3 -> 3)) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not be (sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) and not contain inOrderOnly (2 -> 2, 3 -> 3, 4 -> 4))
        no (list1s) should (not be (sortedMap(1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)) and not contain inOrderOnly (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) and not contain inOrderOnly (8 -> 8, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " was equal to " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (sortedMap(3 -> 3)) and not contain inOrderOnly (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " was not equal to " + decorateToStringValue(sortedMap(3 -> 3)) + ", but " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " contained only " + "((2,2), (3,3), (4,4))" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) and not contain inOrderOnly ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be (sortedMap("ho" -> "ho")) and not contain inOrderOnly ("hi" -> "hi", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("ho" -> "ho")) + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hi,hi), (hello,hello))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (sortedMap("ho" -> "ho")) and not contain inOrderOnly ("HO" -> "HO", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) and not contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be (sortedMap("ho" -> "ho")) and not contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("ho" -> "ho")) + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (sortedMap("ho" -> "ho")) and not contain inOrderOnly ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) and not contain inOrderOnly ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be (sortedMap("ho" -> "ho")) and not contain inOrderOnly ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("ho" -> "ho")) + ", but " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))" + " in order", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (sortedMap(2 -> 2)) and not contain inOrderOnly ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be (sortedMap(2 -> 2)) and not contain inOrderOnly (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("inOrderOnlyDuplicate")))
      }
    }
  }
}
