/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainNoneOfSpec extends Spec with Matchers {

  val upperCaseEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = b match {
    case (b1: String, b2: String) => a._1.toUpperCase == b1 && a._2.toUpperCase == b2
    case _ => (a._1.toUpperCase, a._2.toUpperCase) == b
  }
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }

  object `a List` {

    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to")

    object `when used with contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam")
        val e1 = intercept[TestFailedException] {
          fumList should contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
        // Contains duplicate elements in the right list
        val e2 = intercept[NotAllowedException] {
          fumList should contain noneOf ("fee", "fam", "foe", "fam")
        }
        e2.getMessage should be (Resources("noneOfDuplicate"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumList should contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU")
        intercept[TestFailedException] {
          fumList should contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumList should contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          (fumList should contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
        }
        (fumList should contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ")) (after being lowerCased and trimmed)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain noneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with (contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumList should (contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU"))
        intercept[TestFailedException] {
          fumList should (contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumList should (contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))) (decided by upperCaseEquality)
        }
        fumList should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
        intercept[TestFailedException] {
          (fumList should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain noneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with not contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          fumList should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumList should not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")
        intercept[TestFailedException] {
          fumList should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumList should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        }
        (fumList should not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumList should not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should not contain noneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with (not contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          toList should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(toList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        toList should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        intercept[TestFailedException] {
          toList should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (toList should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        }
        (toList should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          toList should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toList should (not contain noneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toList should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
  }

  object `a collection of Lists` {

    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"))

    object `when used with contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain noneOf (2 -> 2, 3 -> 3, 4 -> 4)
        atLeast (2, lists) should contain noneOf (8 -> 8, 3 -> 3, 4 -> 4)
        atMost (2, lists) should contain noneOf (2 -> 2, 3 -> 3, 4 -> 4)
        no (lists) should contain noneOf (1 -> 1, 2 -> 2, 5 -> 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain noneOf (2 -> 2, 3 -> 3, 4 -> 4)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (3,3), (4,4)) (MapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain noneOf ("ho" -> "ho")
        intercept[TestFailedException] {
          all (hiLists) should contain noneOf ("hi" -> "hi")
        }
        implicit val ise = upperCaseEquality
        all (hiLists) should contain noneOf ("hi" -> "hi")
        intercept[TestFailedException] {
          all (hiLists) should contain noneOf ("HI" -> "HI")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain noneOf ("hi" -> "hi")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain noneOf ("HI" -> "HI")) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hiLists) should contain noneOf ("ho" -> "ho")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should contain noneOf ("hi" -> "hi")) (decided by defaultEquality[(String, String)])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain noneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain noneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with (contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain noneOf (1 -> 1, 2 -> 2, 5 -> 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (3,3), (4,4)) (MapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain noneOf ("ho" -> "ho"))
        intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ("hi" -> "hi"))
        }
        implicit val ise = upperCaseEquality
        all (hiLists) should (contain noneOf ("hi" -> "hi"))
        intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ("HI" -> "HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain noneOf ("hi" -> "hi"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain noneOf ("HI" -> "HI"))) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hiLists) should (contain noneOf ("ho" -> "ho"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should (contain noneOf ("hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain noneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain noneOf (1 -> 1, 2 -> 2, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with not contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("to" -> "to")) + " did not contain one of ((fee,fee), (fie,fie), (foe,foe), (fum,fum)) (MapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (toLists) should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        intercept[TestFailedException] {
          all (toLists) should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")) (decided by upperCaseEquality)
        }
        (all (toLists) should not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toLists) should not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain noneOf ()
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }

    object `when used with (not contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("to" -> "to")) + " did not contain one of ((fee,fee), (fie,fie), (foe,foe), (fum,fum)) (MapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (toLists) should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        intercept[TestFailedException] {
          all (toLists) should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        }
        (all (toLists) should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toLists) should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain noneOf ())
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fie" -> "fie", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("noneOfDuplicate")))
      }
    }
  }
}
