/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class MapShouldContainTheSameElementsInOrderAsLogicalOrSpec extends FreeSpec with Matchers {
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val invertedStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.SortedMap[String, String]] {
      def areEqual(a: scala.collection.SortedMap[String, String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val fileName: String = "MapShouldContainTheSameElementsInOrderAsLogicalOrSpec.scala"
  
  "a List" - {
    
    val fumList: scala.collection.SortedMap[String, String] = sortedMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")
    val toList: scala.collection.SortedMap[String, String] = sortedMap("you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy")
    
    "when used with (contain theSameElementsInOrderAs xx or contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))), fileName, thisLineNumber - 2)
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE ") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (equal xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (equal (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        fumList should (equal (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        fumList should (equal (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        fumList should (equal (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        fumList should (equal (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (equal (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    "when used with (legacyEqual xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))) (after being lowerCased and trimmed)
      }
    }

    "when used with (contain theSameElementsInOrderAs xx and legacyEqual xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE") or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE ") or legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not equal xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        fumList should (not be (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        fumList should (not be (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not be (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))), fileName, thisLineNumber - 2)
        (fumList should (not contain theSameElementsInOrderAs (Set(" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ")) or not contain theSameElementsInOrderAs (Set(" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  "collection of Lists" - {
    
    val list1s: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3))
    val lists: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(2 -> 2, 3 -> 3, 4 -> 4))
    val nils: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(), sortedMap(), sortedMap())
    val listsMap: Vector[scala.collection.SortedMap[Int, Int]] = Vector(sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap())
    val hiLists: Vector[scala.collection.SortedMap[String, String]] = Vector(sortedMap("hi" -> "hi", "hello" -> "hello"), sortedMap("hi" -> "hi", "hello" -> "hello"), sortedMap("hi" -> "hi", "hello" -> "hello"))
    val toLists: Vector[scala.collection.SortedMap[String, String]] = Vector(sortedMap("you" -> "you", "to" -> "to"), sortedMap("you" -> "you", "to" -> "to"), sortedMap("you" -> "you", "to" -> "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 5 -> 5) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4))
        
        atLeast (2, lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(3 -> 3, 6 -> 6, 5 -> 5) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(8 -> 8, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)) + ", and " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello"))
        all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello"))
        all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hello" -> "hello", "ho" -> "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hello" -> "hello", "ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hello" -> "hello", "ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hello" -> "hello", "ho" -> "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO") or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hello" -> "hello", "ho" -> "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hello" -> "hello", "ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (be xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (be (sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        all (list1s) should (be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) + " was not equal to " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + ", and " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO"))
        all (hiLists) should (be (sortedMap("ho" -> "ho", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO"))
        all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (sortedMap("ho" -> "ho", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("ho" -> "ho", "hello" -> "hello")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (sortedMap("ho" -> "ho", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (sortedMap("hi" -> "hi", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (sortedMap("ho" -> "ho", "hello" -> "hello")) or contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(sortedMap("ho" -> "ho", "hello" -> "hello")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 8 -> 8)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(8 -> 8, 3 -> 3, 4 -> 4)))
        all (list1s) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(8 -> 8, 3 -> 3, 4 -> 4)))
        all (list1s) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 8 -> 8)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)) + ", and " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2, 3 -> 3, 4 -> 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hello" -> "hello", "hi" -> "hi")))
        all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hello" -> "hello", "hi" -> "hi")))
        all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello")))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hello" -> "hello", "hi" -> "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hello" -> "hello", "hi" -> "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HI" -> "HI")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (sortedMap(2 -> 2)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(8 -> 8, 3 -> 3, 4 -> 4)))
        all (list1s) should (not be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(8 -> 8, 3 -> 3, 4 -> 4)))
        all (list1s) should (not be (sortedMap(2 -> 2)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) + " was equal to " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) + ", and " + decorateToStringValue(sortedMap(1 -> 1, 2 -> 2, 3 -> 3)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (sortedMap("hello" -> "hello", "ho" -> "ho")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO")))
        all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO")))
        all (hiLists) should (not be (sortedMap("hello" -> "hello", "ho" -> "ho")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not be (sortedMap("hello" -> "hello", "ho" -> "ho")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO")))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HELLO" -> "HELLO", "HO" -> "HO")))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (sortedMap("hello" -> "hello", "ho" -> "ho")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (sortedMap("hi" -> "hi", "hello" -> "hello")) or not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + ", and " + decorateToStringValue(sortedMap("hi" -> "hi", "hello" -> "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
