/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class MapShouldContainTheSameElementsInOrderAsSpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Uniformity[(String, String)] = new Uniformity[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedCanHandle(b: Any): Boolean =
      b match {
        case (_: String, _: String) => true
        case _ => false
      }
    def normalizedOrSame(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }

  object `a List` {

    val fumList: scala.collection.SortedMap[String, String] = sortedMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")
    val toList: scala.collection.SortedMap[String, String] = sortedMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")

    object `when used with contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")
        val e1 = intercept[TestFailedException] {
          fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")
        intercept[TestFailedException] {
          fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE ")
        }
        (fumList should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))
        intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("FUM" -> "FUM", "FOE" -> "FOE", "FIE" -> "FIE", "FEE" -> "FEE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))
        }
        (fumList should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(" FUM " -> " FUM ", " FOE " -> " FOE ", " FIE " -> " FIE ", " FEE " -> " FEE "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with not contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy"))
        val e1 = intercept[TestFailedException] {
          toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedSameElementsInOrder", decorateToStringValue(toList), scala.collection.mutable.LinkedHashMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO", "BIRTHDAY" -> "BIRTHDAY", "HAPPY" -> "HAPPY"))
        intercept[TestFailedException] {
          toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO", "BIRTHDAY" -> "BIRTHDAY", "HAPPY" -> "HAPPY"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseStringEquality)
        }
        toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (toList should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy")))
        val e1 = intercept[TestFailedException] {
          toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedSameElementsInOrder", decorateToStringValue(toList), scala.collection.mutable.LinkedHashMap("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO", "BIRTHDAY" -> "BIRTHDAY", "HAPPY" -> "HAPPY")))
        intercept[TestFailedException] {
          toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO", "BIRTHDAY" -> "BIRTHDAY", "HAPPY" -> "HAPPY")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")))) (decided by upperCaseStringEquality)
        }
        toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")))
        intercept[TestFailedException] {
          (toList should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a col of Lists` {

    val list1s: Vector[scala.collection.SortedMap[Int, Int]] = Vector( sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3))
    val lists: Vector[scala.collection.SortedMap[Int, Int]] = Vector( sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(2 -> 2, 3 -> 3, 4 -> 4))
    val listsMap: Vector[scala.collection.SortedMap[Int, Int]] = Vector( sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap(1 -> 1, 2 -> 2, 3 -> 3), sortedMap())
    val hiLists: Vector[scala.collection.SortedMap[String, String]] = Vector( sortedMap("hi" -> "hi", "he" -> "he"), sortedMap("hi" -> "hi", "he" -> "he"), sortedMap("hi" -> "hi", "he" -> "he"))
    val toLists: Vector[scala.collection.SortedMap[String, String]] = Vector( sortedMap("to" -> "to", "you" -> "you"), sortedMap("to" -> "to", "you" -> "you"), sortedMap("to" -> "to", "you" -> "you"))

    object `when used with contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)
        atLeast (2, lists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)
        atMost (2, lists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)
        no (lists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(3 -> 3, 4 -> 4, 5 -> 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)) + " (MapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (listsMap) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)
        }
        e3.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(listsMap(2)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)) + " (MapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")
        intercept[TestFailedException] {
          all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HE" -> "HE")
        intercept[TestFailedException] {
          all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HO" -> "HO")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HE" -> "HE")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HO" -> "HO")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "ho" -> "ho")) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with (contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        atMost (2, lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        no (lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(3 -> 3, 4 -> 4, 5 -> 5))
        no (listsMap) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(3 -> 3, 4 -> 4, 5 -> 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(sortedMap(2 -> 2, 3 -> 3, 4 -> 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)) + " (MapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3))
        }
        e4.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(listsMap(2)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1, 2 -> 2, 3 -> 3)) + " (MapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"))
        intercept[TestFailedException] {
          all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("he" -> "he", "hi" -> "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HE" -> "HE"))
        intercept[TestFailedException] {
          all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HO" -> "HO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HE" -> "HE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("HI" -> "HI", "HO" -> "HO"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("hi" -> "hi", "he" -> "he"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should (contain theSameElementsInOrderAs scala.collection.mutable.LinkedHashMap("he" -> "he", "hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with not contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("you" -> "you", "to" -> "to"))
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(sortedMap("to" -> "to", "you" -> "you")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you")) +  " (MapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO"))
        intercept[TestFailedException] {
          all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseStringEquality)
        }
        all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (all (toLists) should not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("you" -> "you", "to" -> "to")))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you")))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(sortedMap("to" -> "to", "you" -> "you")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("to" -> "to", "you" -> "you")) + " (MapShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO")))
        intercept[TestFailedException] {
          all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("TO" -> "TO", "YOU" -> "YOU")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("YOU" -> "YOU", "TO" -> "TO")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap("TO" -> "TO", "YOU" -> "YOU")))) (decided by upperCaseStringEquality)
        }
        all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" TO " -> " TO ", " YOU " -> " YOU ")))
        intercept[TestFailedException] {
          (all (toLists) should (not contain theSameElementsInOrderAs (scala.collection.mutable.LinkedHashMap(" TO " -> " TO ", " YOU " -> " YOU ")))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
