/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainAtMostOneOfLogicalAndSpec extends Spec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[Char] = new Uniformity[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedCanHandle(b: Any): Boolean = b.isInstanceOf[Char]
    def normalizedOrSame(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainAtMostOneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: String = "up"
    val toList: String = "oy"
    
    object `when used with (contain atMostOneOf (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atMostOneOf ('e', 'i', 'p', 'a') and contain atMostOneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'u') and contain atMostOneOf ('i', 'e', 'a', 'p'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'a') and contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and contain atMostOneOf ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') and contain atMostOneOf ('I', 'E', 'A', 'P'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and (contain atMostOneOf ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') and contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and contain atMostOneOf ('E', 'I', 'P', 'A'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf () and contain atMostOneOf('i', 'e', 'a', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('i', 'e', 'a', 'p') and contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'i', 'u') and contain atMostOneOf('i', 'e', 'a', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('i', 'e', 'a', 'p') and contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain atMostOneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atMostOneOf ('e', 'i', 'p', 'a'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain atMostOneOf ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atMostOneOf ('I', 'E', 'A', 'P'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain atMostOneOf ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain atMostOneOf ('E', 'I', 'P', 'A'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) and contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain atMostOneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atMostOneOf ('e', 'i', 'p', 'a'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain atMostOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain atMostOneOf ('E', 'I', 'P', 'A'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atMostOneOf ('E', 'I', 'P', 'A'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain atMostOneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain atMostOneOf ('E', 'I', 'P', 'A'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain atMostOneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain atMostOneOf ('E', 'I', 'P', 'A'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain atMostOneOf ('E', 'I', 'P', 'A'))) (after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) and contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atMostOneOf('i', 'e', 'a', 'p') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'a') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'u') and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and (legacyEqual (toList)))
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and legacyEqual (fumList))) (after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf() and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf('e', 'i', 'p', 'i', 'u') and legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain atMostOneOf (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain atMostOneOf ('e', 'i', 'p', 'u') and not contain atMostOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") and not contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('e', 'i', 'p', 'u') and not contain atMostOneOf ('i', 'e', 'a', 'p'))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'a', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') and not contain atMostOneOf ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('E', 'I', 'P', 'D') and not contain atMostOneOf ('I', 'E', 'U', 'P'))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') and (not contain atMostOneOf ('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') and not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') and not contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'A') and not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') and contain atMostOneOf ('E', 'I', 'P', 'A'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf () and not contain atMostOneOf('i', 'e', 'u', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('i', 'e', 'u', 'p') and not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('e', 'i', 'p', 'i', 'u') and not contain atMostOneOf('i', 'e', 'u', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('i', 'e', 'u', 'p') and not contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain atMostOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain atMostOneOf ('i', 'e', 'a', 'p'))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'a', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain atMostOneOf ('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') and not contain atMostOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) and not contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain atMostOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain atMostOneOf ('i', 'e', 'a', 'p'))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'a', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain atMostOneOf ('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') and not contain atMostOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) and not contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[String] = Vector("12", "12", "12")
    val lists: Vector[String] = Vector("12", "12", "23")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("ie", "ie", "ie")
    val toLists: Vector[String] = Vector("oy", "oy", "oy")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain atMostOneOf (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf (3, 4, 1) and contain atMostOneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) and contain atMostOneOf ('2', '3', '4'))
        atMost (2, lists) should (contain atMostOneOf ('3', '2', '8') and contain atMostOneOf ('2', '3', '4'))
        no (lists) should (contain atMostOneOf ('1', '2', '3') and contain atMostOneOf ('3', '2', '1'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf ('2', '3', '4') and contain atMostOneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf ('1', '3', '4') and contain atMostOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ('1', '3', '4'), but " + decorateToStringValue(lists(2)) + " did not contain at most one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ('i', 'l') and contain atMostOneOf ('i', 'e'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('i', 'l'), but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('i', 'e')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf ('1', '6', '8') and contain atMostOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ('1', '6', '8'), but " + decorateToStringValue(lists(2)) + " did not contain at most one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain atMostOneOf ('I', 'O') and contain atMostOneOf ("HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ('I', 'E') and contain atMostOneOf ("HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ('I', 'O') and contain atMostOneOf ('I', 'E'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O'), but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atMostOneOf ('I', 'O') and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ('I', 'E') and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ('I', 'O') and contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O'), but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () and contain atMostOneOf ('1', '3', '4'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '3', '4') and contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '2', '2', '3') and contain atMostOneOf ('1', '3', '4'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '3', '4') and contain atMostOneOf ('1', '2', '2', '3'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("12") and contain atMostOneOf ('1', '3', '4'))
        atLeast (2, lists) should (be ("12") and contain atMostOneOf ('2', '3', '4'))
        atMost (2, lists) should (be ("12") and contain atMostOneOf ('2', '3', '4'))
        no (lists) should (be ("8") and contain atMostOneOf ('3', '2', '1'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be ("12") and contain atMostOneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("[" + lists(2) + "]") + " was not equal to " + decorateToStringValue("[12]"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be ("12") and contain atMostOneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue("12") + ", but " + decorateToStringValue(list1s(0)) + " did not contain at most one of ('1', '2', '3')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be ("ie") and contain atMostOneOf ('i', 'e'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie") + ", but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('i', 'e')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be ("12") and contain atMostOneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue("12") + ", but " + decorateToStringValue(list1s(0)) + " did not contain at most one of ('1', '2', '3')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be ("ie") and contain atMostOneOf ("HE"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be ("o") and contain atMostOneOf ("HE"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[" + hiLists(0) + "]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be ("ie") and contain atMostOneOf ('I', 'E'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie") + ", but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("ie") and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be ("o") and contain atMostOneOf ("HE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[" + hiLists(0) + "]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be ("ie") and contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie") + ", but " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be ("12") and contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be ("12") and contain atMostOneOf ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not contain atMostOneOf (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atMostOneOf ('3', '2', '1') and not contain atMostOneOf ('1', '2', '3'))
        atLeast (2, lists) should (not contain atMostOneOf ('1', '2', '8') and not contain atMostOneOf ('1', '2', '3'))
        atMost (2, lists) should (not contain atMostOneOf ('1', '2', '8') and contain atMostOneOf ('1', '2', '3'))
        no (lists) should (not contain atMostOneOf ('2', '6', '8') and not contain atMostOneOf ('3', '6', '8'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf ('1', '2', '8') and not contain atMostOneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ('1', '2', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf ('1', '2', '3') and not contain atMostOneOf ('1', '2', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of ('1', '2', '3'), but " + decorateToStringValue(lists(2)) + " contained at most one of ('1', '2', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ('i', 'l') and not contain atMostOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ('i', 'e') and not contain atMostOneOf ('i', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('i', 'e'), but " + decorateToStringValue(hiLists(0)) + " contained at most one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain atMostOneOf ('I', 'E') and not contain atMostOneOf ('E', 'I'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ('I', 'O') and not contain atMostOneOf ('E', 'I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ('I', 'E') and not contain atMostOneOf ('I', 'O'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E'), but " + decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain atMostOneOf ('I', 'E') and not contain atMostOneOf ('E', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atMostOneOf ('I', 'O') and not contain atMostOneOf ('E', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atMostOneOf ('I', 'E') and not contain atMostOneOf ('I', 'O'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E'), but " + decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () and not contain atMostOneOf ('1', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf ('1', '2', '3') and not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf ('1', '2', '2', '3') and not contain atMostOneOf ('1', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf ('1', '2', '3') and not contain atMostOneOf ('1', '2', '2', '3'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain atMostOneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("23") and not contain atMostOneOf ('1', '2', '3'))
        atLeast (2, lists) should (not be ("23") and not contain atMostOneOf ('1', '2', '3'))
        atMost (2, lists) should (not be ("23") and contain atMostOneOf ('1', '2', '3'))
        no (list1s) should (not be ("12") and not contain atMostOneOf ('3', '6', '8'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be ("23") and not contain atMostOneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " was equal to " + decorateToStringValue("23"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be ("234") and not contain atMostOneOf ('1', '2', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(lists(2) + "[]") + " was not equal to " + decorateToStringValue("23[4]") + ", but " + decorateToStringValue(lists(2)) + " contained at most one of ('1', '2', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be ("ie") and not contain atMostOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be ("io") and not contain atMostOneOf ('i', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i[e]") + " was not equal to " + decorateToStringValue("i[o]") + ", but " + decorateToStringValue(hiLists(0)) + " contained at most one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (List('I', 'O')) and not contain atMostOneOf ('I', 'E'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("ie") and not contain atMostOneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be ("io") and not contain atMostOneOf ('I', 'O'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i[e]") + " was not equal to " + decorateToStringValue("i[o]") + ", but " + decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (List('I', 'O')) and not contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("ie") and not contain atMostOneOf ('i'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("io") and not contain atMostOneOf ('I', 'O'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i[e]") + " was not equal to " + decorateToStringValue("i[o]") + ", but " + decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be ("23") and not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be ("23") and not contain atMostOneOf ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
