/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainAtMostOneOfLogicalOrSpec extends FreeSpec with Matchers {
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[Char] = new Uniformity[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedCanHandle(b: Any): Boolean = b.isInstanceOf[Char]
    def normalizedOrSame(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainAtMostOneOfLogicalOrSpec.scala"
  
  "a List" - {
    
    val fumList: String = "up"
    val toList: String = "oy"
    
      "when used with (contain atMostOneOf (...) or contain atMostOneOf (...)) syntax" - {
      
        "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain atMostOneOf ('e', 'i', 'p', 'a') or contain atMostOneOf('i', 'e', 'a', 'p'))
        fumList should (contain atMostOneOf ('e', 'i', 'p', 'a') or contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (contain atMostOneOf ('e', 'i', 'p', 'u') or contain atMostOneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'u') or contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') or contain atMostOneOf ('I', 'E', 'A', 'P'))
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') or contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') or contain atMostOneOf ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') or (contain atMostOneOf ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') or contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') or contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') or contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') or contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') or contain atMostOneOf ('E', 'I', 'P', 'A'))) (after being lowerCased, after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf () or contain atMostOneOf('i', 'e', 'a', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('i', 'e', 'a', 'p') or contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'i', 'u') or contain atMostOneOf('i', 'e', 'a', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('i', 'e', 'a', 'p') or contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (equal (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (equal (fumList) or contain atMostOneOf('i', 'e', 'a', 'p'))
        fumList should (equal (toList) or contain atMostOneOf('i', 'e', 'a', 'p'))
        fumList should (equal (fumList) or contain atMostOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain atMostOneOf ('I', 'E', 'A', 'P'))
        fumList should (equal (toList) or contain atMostOneOf ('I', 'E', 'A', 'P'))
        fumList should (equal (fumList) or contain atMostOneOf ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain atMostOneOf ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (equal (toList) or contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain atMostOneOf ('E', 'I', 'P', 'A'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) or contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (legacyEqual (...) and contain theMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (legacyEqual (fumList) or contain atMostOneOf('i', 'e', 'a', 'p'))
        fumList should (legacyEqual (toList) or contain atMostOneOf('i', 'e', 'a', 'p'))
        fumList should (legacyEqual (fumList) or contain atMostOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain atMostOneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain atMostOneOf ('I', 'E', 'A', 'P'))
        fumList should (legacyEqual (toList) or contain atMostOneOf ('I', 'E', 'A', 'P'))
        fumList should (legacyEqual (fumList) or contain atMostOneOf ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain atMostOneOf ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (legacyEqual (fumList) or contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain atMostOneOf ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain atMostOneOf ('E', 'I', 'P', 'A'))) (after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) or contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) or contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    "when used with (contain oneOf (...) and legacyEqual (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain atMostOneOf('i', 'e', 'a', 'p') or legacyEqual (fumList))
        fumList should (contain atMostOneOf('i', 'e', 'u', 'p') or legacyEqual (fumList))
        fumList should (contain atMostOneOf('i', 'e', 'a', 'p') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'u') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain atMostOneOf ('I', 'E', 'A', 'P') or legacyEqual (fumList))
        fumList should (contain atMostOneOf ('I', 'E', 'U', 'P') or legacyEqual (fumList))
        fumList should (contain atMostOneOf ('I', 'E', 'A', 'P') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain atMostOneOf ('I', 'E', 'A', 'P') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ('I', 'E', 'U', 'P') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain atMostOneOf ('I', 'E', 'A', 'P') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf (" FEE ", " FIE ", " FOE ", " FaM ") or legacyEqual (fumList))) (after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf() or legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf('e', 'i', 'p', 'i', 'u') or legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain atMostOneOf (...) and not contain atMostOneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not contain atMostOneOf ('e', 'i', 'p', 'u') or not contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (not contain atMostOneOf ('e', 'i', 'p', 'u') or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ("fee", "fie", "foe", "fuu") or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\"") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') or not contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (not contain atMostOneOf ('E', 'I', 'P', 'D') or not contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') or not contain atMostOneOf ('I', 'E', 'D', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('E', 'I', 'P', 'D') or not contain atMostOneOf ('E', 'I', 'P', 'D'))
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') or not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'D') or not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') or not contain atMostOneOf ('I', 'E', 'D', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'D') or not contain atMostOneOf ('E', 'I', 'P', 'D'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'") + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A') or contain atMostOneOf ('E', 'I', 'P', 'A'))) (after being lowerCased, after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf () or not contain atMostOneOf('i', 'e', 'u', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('i', 'e', 'u', 'p') or not contain atMostOneOf())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('e', 'i', 'p', 'i', 'u') or not contain atMostOneOf('i', 'e', 'u', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('i', 'e', 'u', 'p') or not contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not equal (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not equal (toList) or not contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (not equal (fumList) or not contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (not equal (toList) or not contain atMostOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (not equal (fumList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (not equal (toList) or not contain atMostOneOf ('I', 'E', 'D', 'P'))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain atMostOneOf ('E', 'I', 'P', 'D')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not equal (fumList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain atMostOneOf ('I', 'E', 'D', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain atMostOneOf ('E', 'I', 'P', 'D'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') or not contain atMostOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) or not contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not be (toList) or not contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (not be (fumList) or not contain atMostOneOf('i', 'e', 'u', 'p'))
        fumList should (not be (toList) or not contain atMostOneOf("fee", "fie", "foe", "fuu"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain atMostOneOf ("fee", "fie", "foe", "fuu"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "\"fee\", \"fie\", \"foe\", \"fuu\""), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (not be (fumList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))
        fumList should (not be (toList) or not contain atMostOneOf ('I', 'E', 'D', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain atMostOneOf ('E', 'I', 'P', 'D')))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not be (toList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain atMostOneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain atMostOneOf ('I', 'E', 'D', 'P'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain atMostOneOf ('E', 'I', 'P', 'D'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'E', 'I', 'P', 'D'"), fileName, thisLineNumber - 2)
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U') or not contain atMostOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) or not contain atMostOneOf())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) or not contain atMostOneOf('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
  }
  
  "collection of Lists" - {
    
    val list1s: Vector[String] = Vector("12", "12", "12")
    val lists: Vector[String] = Vector("12", "12", "23")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("ie", "ie", "ie")
    val toLists: Vector[String] = Vector("oy", "oy", "oy")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain oneOf (..) and contain oneOf (..)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain atMostOneOf ('1', '6', '8') or contain atMostOneOf ('1', '3', '4'))
        all (list1s) should (contain atMostOneOf ('1', '2', '3') or contain atMostOneOf ('1', '3', '4'))
        all (list1s) should (contain atMostOneOf ('1', '6', '8') or contain atMostOneOf ('1', '2', '3'))
        
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) or contain atMostOneOf ('2', '3', '4'))
        atLeast (2, lists) should (contain atMostOneOf ('1', '2', '3') or contain atMostOneOf ('2', '3', '4'))
        atLeast (2, lists) should (contain atMostOneOf (3, 2, 5) or contain atMostOneOf ('1', '2', '3'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf ('2', '3', '4') or contain atMostOneOf ('4', '3', '2'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " did not contain at most one of ('2', '3', '4'), and " + decorateToStringValue(lists(2)) + " did not contain at most one of ('4', '3', '2')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain atMostOneOf ('I', 'O') or contain atMostOneOf ('O', 'I'))
        all (hiLists) should (contain atMostOneOf ('I', 'E') or contain atMostOneOf ('O', 'I'))
        all (hiLists) should (contain atMostOneOf ('I', 'O') or contain atMostOneOf ('I', 'E'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ('I', 'E') or contain atMostOneOf ('E', 'I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E'), and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('E', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (contain atMostOneOf ('I', 'O') or contain atMostOneOf ('O', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain atMostOneOf ('I', 'E') or contain atMostOneOf ('O', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain atMostOneOf ('I', 'O') or contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ('I', 'E') or contain atMostOneOf ('E', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E'), and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('E', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf () or contain atMostOneOf ('1', '3', '4'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '3', '4') or contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '2', '2', '3') or contain atMostOneOf ('1', '3', '4'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '3', '4') or contain atMostOneOf ('1', '2', '2', '3'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (be (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be ("12") or contain atMostOneOf ('1', '6', '8'))
        all (list1s) should (be ("23") or contain atMostOneOf ('1', '6', '8'))
        all (list1s) should (be ("12") or contain atMostOneOf ('1', '2', '3'))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be ("23") or contain atMostOneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[" + list1s(0) + "]") + " was not equal to " + decorateToStringValue("[23]") + ", and " + decorateToStringValue(list1s(0)) + " did not contain at most one of ('1', '2', '3')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be ("ie") or contain atMostOneOf ('I', 'O'))
        all (hiLists) should (be (List('O')) or contain atMostOneOf ('I', 'O'))
        all (hiLists) should (be ("ie") or contain atMostOneOf ('I', 'E'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (List('O')) or contain atMostOneOf ('I', 'E'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(List('O')) + ", and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (be ("ie") or contain atMostOneOf ('I', 'O'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (List('O')) or contain atMostOneOf ('I', 'O'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be ("ie") or contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (List('O')) or contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was not equal to " + decorateToStringValue(List('O')) + ", and " + decorateToStringValue(hiLists(0)) + " did not contain at most one of ('I', 'E')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be ("12") or contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be ("12") or contain atMostOneOf ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not contain oneOf (..) and not contain oneOf (..))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain atMostOneOf ('3', '2', '1') or not contain atMostOneOf ('1', '2', '3'))
        all (list1s) should (not contain atMostOneOf ('1', '2', '8') or not contain atMostOneOf ('1', '2', '3'))
        all (list1s) should (not contain atMostOneOf ('3', '2', '1') or not contain atMostOneOf ('1', '2', '8'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atMostOneOf ('1', '2', '8') or not contain atMostOneOf (8, 2, 1))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(lists(2)) + " contained at most one of ('1', '2', '8'), and " + decorateToStringValue(lists(2)) + " contained at most one of (8, 2, 1)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain atMostOneOf ('I', 'E') or not contain atMostOneOf ('E', 'I'))
        all (hiLists) should (not contain atMostOneOf ('i') or not contain atMostOneOf ('E', 'I'))
        all (hiLists) should (not contain atMostOneOf ('I', 'E') or not contain atMostOneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atMostOneOf ('i') or not contain atMostOneOf ('e'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('i'), and " + decorateToStringValue(hiLists(0)) + " contained at most one of ('e')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not contain atMostOneOf ('I', 'E') or not contain atMostOneOf ('E', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain atMostOneOf ('i') or not contain atMostOneOf ('E', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain atMostOneOf ('I', 'E') or not contain atMostOneOf ('i'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atMostOneOf ('i') or not contain atMostOneOf ('e'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " contained at most one of ('i'), and " + decorateToStringValue(hiLists(0)) + " contained at most one of ('e')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf () or not contain atMostOneOf ('1', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf ('1', '2', '3') or not contain atMostOneOf ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf ('1', '2', '2', '3') or not contain atMostOneOf ('1', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain atMostOneOf ('1', '2', '3') or not contain atMostOneOf ('1', '2', '2', '3'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
    
    "when used with (not be (...) and not contain atMostOneOf (...))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be ("23") or not contain atMostOneOf ('1', '2', '3'))
        all (list1s) should (not be ("12") or not contain atMostOneOf ('1', '2', '3'))
        all (list1s) should (not be ("23") or not contain atMostOneOf ('2', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be ("12") or not contain atMostOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(list1s(0)) + " was equal to " + decorateToStringValue("12") + ", and " + decorateToStringValue(list1s(0)) + " contained at most one of ('2', '3', '4')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be ("o") or not contain atMostOneOf ('I', 'E'))
        all (hiLists) should (not be ("ie") or not contain atMostOneOf ('E', 'I'))
        all (hiLists) should (not be ("o") or not contain atMostOneOf ('I', 'E'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("ie") or not contain atMostOneOf ('I', 'O'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie") + ", and " + decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not be ("o") or not contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be ("ie") or not contain atMostOneOf ('E', 'I'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be ("o") or not contain atMostOneOf ('I', 'E'))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("ie") or not contain atMostOneOf ('I', 'O'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(hiLists(0)) + " was equal to " + decorateToStringValue("ie") + ", and " + decorateToStringValue(hiLists(0)) + " contained at most one of ('I', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS is empty" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be ("23") or not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      "should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value" in {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be ("23") or not contain atMostOneOf ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
