/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainAtMostOneOfSpec extends Spec with Matchers {

  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Uniformity[Char] = new Uniformity[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedCanHandle(b: Any): Boolean = b.isInstanceOf[Char]
    def normalizedOrSame(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }

  object `a List` {

    val fumList: String = "up"
    val toList: String = "oy"

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain atMostOneOf ('e', 'i', 'p', 'a')
        val e1 = intercept[TestFailedException] {
          fumList should contain atMostOneOf ('e', 'i', 'p', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumList should contain atMostOneOf ('E', 'I', 'P', 'A')
        intercept[TestFailedException] {
          fumList should contain atMostOneOf ('E', 'I', 'P', 'U')
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain atMostOneOf ('E', 'I', 'P', 'A')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should contain atMostOneOf ('E', 'I', 'P', 'U')) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumList should contain atMostOneOf ('E', 'I', 'P', 'U')) (after being lowerCased)
          
        }
        fumList should contain atMostOneOf ('E', 'I', 'P', 'U')
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain atMostOneOf ('e', 'i', 'p', 'i', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain atMostOneOf ('e', 'i', 'p', 'a'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'A'))
        intercept[TestFailedException] {
          fumList should (contain atMostOneOf ('E', 'I', 'P', 'U'))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain atMostOneOf ('E', 'I', 'P', 'A'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          (fumList should (contain atMostOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased)
        }
        fumList should (contain atMostOneOf ('E', 'I', 'P', 'U'))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain atMostOneOf ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should not contain atMostOneOf ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          fumList should not contain atMostOneOf ('e', 'i', 'p', 'a')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val use = upperCaseStringEquality
        fumList should not contain atMostOneOf ('E', 'I', 'P', 'U')
        intercept[TestFailedException] {
          fumList should not contain atMostOneOf ('E', 'I', 'P', 'A')
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should not contain atMostOneOf ('E', 'I', 'P', 'U')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should not contain atMostOneOf ('E', 'I', 'P', 'A')) (decided by upperCaseStringEquality)
        }
        (fumList should not contain atMostOneOf ('E', 'I', 'P', 'U')) (after being lowerCased)
        intercept[TestFailedException] {
          fumList should not contain atMostOneOf ('E', 'I', 'P', 'U')
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should not contain atMostOneOf ('e', 'i', 'p', 'i', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain atMostOneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('e', 'i', 'p', 'a'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtMostOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U'))
        intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('E', 'I', 'P', 'A'))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'A'))) (decided by upperCaseStringEquality)
        }
        (fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased)
        intercept[TestFailedException] {
          fumList should (not contain atMostOneOf ('E', 'I', 'P', 'U'))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain atMostOneOf ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }

  object `a collection of Lists` {

    val list1s: Vector[String] = Vector("12", "12", "12")
    val lists: Vector[String] = Vector("12", "12", "23")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("12", "12", "")
    val hiLists: Vector[String] = Vector("ie", "ie", "ie")
    val toLists: Vector[String] = Vector("oy", "oy", "oy")

    object `when used with contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atMostOneOf ('1', '3', '4')
        atLeast (2, lists) should contain atMostOneOf ('2', '3', '4')
        atMost (2, lists) should contain atMostOneOf ('2', '3', '4')
        no (lists) should contain atMostOneOf ('1', '2', '3')

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atMostOneOf ('2', '3', '4')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of ('2', '3', '4') (StringShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain atMostOneOf ("hi", "ho")
        intercept[TestFailedException] {
          all (hiLists) should contain atMostOneOf ('i', 'e')
        }
        implicit val use = upperCaseStringEquality
        all (hiLists) should contain atMostOneOf ('I', 'O')
        intercept[TestFailedException] {
          all (hiLists) should contain atMostOneOf ('I', 'E')
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain atMostOneOf ("hi", "ho")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain atMostOneOf ('i', 'e')) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should contain atMostOneOf ("hi", "HE")) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should contain atMostOneOf ('i', 'e')) (decided by defaultEquality[Char])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain atMostOneOf ('1', '2', '2', '3')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atMostOneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain atMostOneOf ('2', '3', '4'))
        atMost (2, lists) should (contain atMostOneOf ('2', '3', '4'))
        no (lists) should (contain atMostOneOf ('1', '2', '3'))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atMostOneOf ('2', '3', '4'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain at most one of ('2', '3', '4') (StringShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain atMostOneOf ("hi", "ho"))
        intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ('i', 'e'))
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should (contain atMostOneOf ('I', 'O'))
        intercept[TestFailedException] {
          all (hiLists) should (contain atMostOneOf ('I', 'E'))
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atMostOneOf ("hi", "ho"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ('i', 'e'))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should (contain atMostOneOf ("hi", "HE"))) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should (contain atMostOneOf ('i', 'e'))) (decided by defaultEquality[Char])
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain atMostOneOf ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with not contain atMostOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain atMostOneOf ('h', 'b', 'o', 'y')
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain atMostOneOf ('e', 'i', 'p', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toLists(0)) + " contained at most one of ('e', 'i', 'p', 'u') (StringShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should not contain atMostOneOf ('H', 'B', 'O', 'Y')
        intercept[TestFailedException] {
          all (toLists) should not contain atMostOneOf ('E', 'I', 'P', 'U')
        }
      }
      
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain atMostOneOf ('H', 'B', 'O', 'Y')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain atMostOneOf ('E', 'I', 'P', 'U')) (decided by upperCaseStringEquality)
        }
        (all (toLists) should not contain atMostOneOf ('H', 'B', 'O', 'Y')) (after being lowerCased)
        intercept[TestFailedException] {
          all (toLists) should not contain atMostOneOf ('H', 'B', 'O', 'Y')
        }
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain atMostOneOf ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain atMostOneOf ('e', 'i', 'p', 'i', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }

    object `when used with (not contain atMostOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain atMostOneOf ('h', 'b', 'o', 'y'))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain atMostOneOf ('e', 'i', 'p', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toLists(0)) + " contained at most one of ('e', 'i', 'p', 'u') (StringShouldContainAtMostOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should (not contain atMostOneOf ('H', 'B', 'O', 'Y'))
        intercept[TestFailedException] {
          all (toLists) should (not contain atMostOneOf ('E', 'I', 'P', 'U'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain atMostOneOf ('H', 'B', 'O', 'Y'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain atMostOneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        (all (toLists) should (not contain atMostOneOf ('H', 'B', 'O', 'Y'))) (after being lowerCased)
        intercept[TestFailedException] {
          all (toLists) should (not contain atMostOneOf ('H', 'B', 'O', 'Y'))
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain atMostOneOf ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain atMostOneOf ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtMostOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("atMostOneOfDuplicate")))
      }
    }
  }
}
