/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainOnlyLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[Char] = new Uniformity[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedCanHandle(b: Any): Boolean = b.isInstanceOf[Char]
    def normalizedOrSame(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainOnlyLogicalOrSpec.scala"
  
  object `a List` {
    
    val fumList: String = "upie"
    val toList: String = "yobh"
    
    object `when used with (contain only (..) or contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain only ('e', 'i', 'p', 'u') or contain only ('i', 'e', 'u', 'p'))
        fumList should (contain only ('e', 'i', 'p', 'a') or contain only ('i', 'e', 'u', 'p'))
        fumList should (contain only ('e', 'i', 'p', 'u') or contain only ('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('e', 'i', 'p', 'a') or contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain only ('E', 'I', 'P', 'U') or contain only ('I', 'E', 'U', 'P'))
        fumList should (contain only ('E', 'I', 'P', 'A') or contain only ('I', 'E', 'U', 'P'))
        fumList should (contain only ('E', 'I', 'P', 'U') or contain only ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('E', 'I', 'P', 'A') or (contain only ('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain only ('E', 'I', 'P', 'U') or contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain only ('E', 'I', 'P', 'A') or contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain only ('E', 'I', 'P', 'U') or contain only ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain only ('E', 'I', 'P', 'A') or contain only ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (contain only ('E', 'I', 'P', 'U') or contain only ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only () or contain only ('i', 'e', 'u', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only ('i', 'e', 'u', 'p') or contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only ('e', 'i', 'p', 'i', 'u') or contain only ('i', 'e', 'u', 'p'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only ('i', 'e', 'u', 'p') or contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (equal (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) or contain only ('i', 'e', 'u', 'p'))
        fumList should (equal (toList) or contain only ('i', 'e', 'u', 'p'))
        fumList should (equal (fumList) or contain only ('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain only ('I', 'E', 'U', 'P'))
        fumList should (equal (toList) or contain only ('I', 'E', 'U', 'P'))
        fumList should (equal (fumList) or contain only ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain only ('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) or contain only ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain only ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain only ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain only ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain only Set('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) or contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (equal (fumList) or contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (legacyEqual (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) or contain only ('i', 'e', 'u', 'p'))
        fumList should (legacyEqual (toList) or contain only ('i', 'e', 'u', 'p'))
        fumList should (legacyEqual (fumList) or contain only ('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain only ('i', 'e', 'a', 'p'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'i', 'e', 'a', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain only ('I', 'E', 'U', 'P'))
        fumList should (legacyEqual (toList) or contain only ('I', 'E', 'U', 'P'))
        fumList should (legacyEqual (fumList) or contain only ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain only ('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) or contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain only ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain only ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain only ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) or contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (legacyEqual (fumList) or contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (contain only (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain only ('i', 'e', 'u', 'p') or legacyEqual (fumList))
        fumList should (contain only ('i', 'e', 'a', 'p') or legacyEqual (fumList))
        fumList should (contain only ('i', 'e', 'u', 'p') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('e', 'i', 'p', 'a') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain only ('I', 'E', 'U', 'P') or legacyEqual (fumList))
        fumList should (contain only ('I', 'E', 'A', 'P') or legacyEqual (fumList))
        fumList should (contain only ('I', 'E', 'U', 'P') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('E', 'I', 'P', 'A') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain only ('I', 'E', 'U', 'P') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain only ('I', 'E', 'A', 'P') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain only ('I', 'E', 'U', 'P') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain only ('E', 'I', 'P', 'A') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain only ('E', 'I', 'P', 'U') or legacyEqual (fumList))) (after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only () or legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only ('e', 'i', 'p', 'i', 'u') or legacyEqual (fumList))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not contain only (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain only ("fee", "fie", "foe", "fuu") or not contain only ("fie", "fee", "fuu", "foe"))
        fumList should (not contain only ('e', 'i', 'p', 'u') or not contain only ("fie", "fee", "fuu", "foe"))
        fumList should (not contain only ("fee", "fie", "foe", "fuu") or not contain only ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain only ('e', 'i', 'p', 'u') or not contain only ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain only ('E', 'I', 'P', 'D') or not contain only ('I', 'E', 'D', 'P'))
        fumList should (not contain only ('E', 'I', 'P', 'U') or not contain only ('I', 'E', 'D', 'P'))
        fumList should (not contain only ('E', 'I', 'P', 'D') or not contain only ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain only ('E', 'I', 'P', 'U') or not contain only ('I', 'E', 'U', 'P'))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain only ('E', 'I', 'P', 'D') or not contain only ('I', 'E', 'D', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain only ('E', 'I', 'P', 'U') or not contain only ('I', 'E', 'D', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain only ('E', 'I', 'P', 'D') or not contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain only ('E', 'I', 'P', 'U') or not contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (contain only ('E', 'I', 'P', 'U') or contain only ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain only () or not contain only ("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain only ("fie", "fee", "fuu", "foe") or not contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain only ('e', 'i', 'p', 'i', 'u') or not contain only ("fie", "fee", "fuu", "foe"))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          fumList should (not contain only ("fie", "fee", "fuu", "foe") or not contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not equal (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) or not contain only ("fie", "fee", "fuu", "foe"))
        fumList should (not equal (fumList) or not contain only ("fie", "fee", "fuu", "foe"))
        fumList should (not equal (toList) or not contain only ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain only ('I', 'E', 'D', 'P'))
        fumList should (not equal (fumList) or not contain only ('I', 'E', 'D', 'P'))
        fumList should (not equal (toList) or not contain only ('I', 'E', 'U', 'P'))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain only ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) or not contain only ('I', 'E', 'D', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain only ('I', 'E', 'D', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain only ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain only ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (not contain only ('E', 'I', 'P', 'D') or not contain only ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) or not contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not equal (toList) or not contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not be (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) or not contain only ("fie", "fee", "fuu", "foe"))
        fumList should (not be (fumList) or not contain only ("fie", "fee", "fuu", "foe"))
        fumList should (not be (toList) or not contain only ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain only ('I', 'E', 'D', 'P'))
        fumList should (not be (fumList) or not contain only ('I', 'E', 'D', 'P'))
        fumList should (not be (toList) or not contain only ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain only ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) or not contain only ('I', 'E', 'D', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain only ('I', 'E', 'D', 'P'))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain only ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain only ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (not contain only ('E', 'I', 'P', 'D') or not contain only ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) or not contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (not be (toList) or not contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[String] = Vector("321", "321", "321")
    val lists: Vector[String] = Vector("321", "321", "432")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("321", "321", "")
    val hiLists: Vector[String] = Vector("il", "il", "il")
    val toLists: Vector[String] = Vector("yo", "yo", "yo")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain only (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only ('3', '2', '1') or contain only ('1', '3', '2'))
        all (list1s) should (contain only (3, 2, 5) or contain only ('1', '3', '2'))
        all (list1s) should (contain only ('3', '2', '1') or contain only ('2', '3', '4'))
        
        atLeast (2, lists) should (contain only ('3', '1', '2') or contain only ('1', '2', '3'))
        atLeast (2, lists) should (contain only ('3', '6', '5') or contain only ('1', '3', '2'))
        atLeast (2, lists) should (contain only ('3', '1', '2') or contain only ('8', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only ('3', '1', '2') or contain only ('1', '3', '2'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("432") + " did not contain only " + "('3', '1', '2')" + ", and " + decorateToStringValue("432") + " did not contain only " + "('1', '3', '2')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain only ('L', 'I') or contain only ('l', 'i'))
        all (hiLists) should (contain only ('L', 'O') or contain only ('l', 'i'))
        all (hiLists) should (contain only ('L', 'I') or contain only ('l', 'o'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain only ('L', 'O') or contain only ('l', 'o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('L', 'O')" + ", and " + decorateToStringValue("il") + " did not contain only " + "('l', 'o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain only ('L', 'I') or contain only ('l', 'i'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain only ('L', 'O') or contain only ('l', 'i'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain only ('L', 'I') or contain only ('l', 'o'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain only ('L', 'O') or contain only ('l', 'o'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('L', 'O')" + ", and " + decorateToStringValue("il") + " did not contain only " + "('l', 'o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only () or contain only ('1', '3', '2'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only ('1', '3', '2') or contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only (3, 2, 2, 1) or contain only ('1', '3', '2'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only ('1', '3', '2') or contain only (3, 2, 2, 1))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (be (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("321") or contain only ('1', '2', '3'))
        all (list1s) should (be ("234") or contain only ('1', '2', '3'))
        all (list1s) should (be ("321") or contain only ('2', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be ("234") or contain only ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[321]") + " was not equal to " + decorateToStringValue("[234]") + ", and " + decorateToStringValue("321") + " did not contain only " + "('2', '3', '4')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be ("il") or contain only ('L', 'I'))
        all (hiLists) should (be (List('o', 'l')) or contain only ('L', 'I'))
        all (hiLists) should (be ("il") or contain only ('L', 'O'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (List('o', 'l')) or contain only ('L', 'O'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was not equal to " + decorateToStringValue(List('o', 'l')) + ", and " + decorateToStringValue("il") + " did not contain only " + "('L', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("il") or contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (List('o', 'l')) or contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be ("il") or contain only ('L', 'O'))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (List('o', 'l')) or contain only ('L', 'O'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was not equal to " + decorateToStringValue(List('o', 'l')) + ", and " + decorateToStringValue("il") + " did not contain only " + "('L', 'O')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be ("321") or contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (be ("321") or contain only ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not contain only xx and not contain only xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain only ('3', '2', '8') or not contain only ('8', '3', '4'))
        all (list1s) should (not contain only ('1', '2', '3') or not contain only ('8', '3', '4'))
        all (list1s) should (not contain only ('3', '2', '8') or not contain only ('2', '3', '1'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain only ('4', '2', '3') or not contain only ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("432") + " contained only " + "('4', '2', '3')" + ", and " + decorateToStringValue("432") + " contained only " + "('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain only ('L', 'O') or not contain only ('l', 'o'))
        all (hiLists) should (not contain only ('L', 'I') or not contain only ('l', 'o'))
        all (hiLists) should (not contain only ('L', 'O') or not contain only ('l', 'i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain only ('L', 'I') or not contain only ('l', 'i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('L', 'I')" + ", and " + decorateToStringValue("il") + " contained only " + "('l', 'i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain only ('L', 'O') or not contain only ('l', 'o'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain only ('L', 'I') or not contain only ('l', 'o'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain only ('L', 'O') or not contain only ('l', 'i'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain only ('L', 'I') or not contain only ('l', 'i'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('L', 'I')" + ", and " + decorateToStringValue("il") + " contained only " + "('l', 'i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only () or not contain only ('8', '3', '4'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only ('8', '3', '4') or not contain only ())
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only ('1', '2', '2', '3') or not contain only ('8', '3', '4'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only ('8', '3', '4') or not contain only ('1', '2', '2', '3'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
    
    object `when used with (not be (...) and not contain only (...))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("2") or not contain only ('8', '3', '4'))
        all (list1s) should (not be ("321") or not contain only ('8', '3', '4'))
        all (list1s) should (not be ("2") or not contain only ('1', '2', '3'))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be ("321") or not contain only ('2', '3', '1'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("321") + " was equal to " + decorateToStringValue("321") + ", and " + decorateToStringValue("321") + " contained only " + "('2', '3', '1')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (List('l', 'o')) or not contain only ('L', 'O'))
        all (hiLists) should (not be (List('l', 'i')) or not contain only ('L', 'O'))
        all (hiLists) should (not be (List('l', 'o')) or not contain only ('L', 'I'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("il") or not contain only ('L', 'I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", and " + decorateToStringValue("il") + " contained only " + "('L', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (List('l', 'o')) or not contain only ('L', 'O'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (List('l', 'i')) or not contain only ('L', 'O'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (List('l', 'o')) or not contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("il") or not contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", and " + decorateToStringValue("il") + " contained only " + "('L', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be ("2") or not contain only ())
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not be ("2") or not contain only ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be (fileName)
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
        
        val e2 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (not contain only ('8', '3', '4') or not contain only ('1', '2', '2', '3'))
        }
        e2.failedCodeFileName.get should be (fileName)
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some(Resources("onlyDuplicate")))
      }
    }
  }
}
