/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Uniformity
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainOnlySpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Uniformity[Char] = new Uniformity[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedCanHandle(b: Any): Boolean = b.isInstanceOf[Char]
    def normalizedOrSame(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }

  object `a List` {

    val fumList: String = "upie"
    val toList: String = "yobh"

    object `when used with contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain only ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          fumList should contain only ('h', 'b', 'o', 'y')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should contain only ('E', 'I', 'P', 'U')
        intercept[TestFailedException] {
          fumList should contain only ("fee", "fie", "foe")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain only ('E', 'I', 'P', 'U')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should contain only ("fee", "fie", "foe")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should contain only ('E', 'I', 'P', 'U')
        }
        (fumList should contain only ('E', 'I', 'P', 'U')) (after being lowerCased)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain only ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should contain only ('e', 'i', 'p', 'i', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain only ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('h', 'b', 'o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain only ('E', 'I', 'P', 'U'))
        intercept[TestFailedException] {
          fumList should (contain only ("fee", "fie", "foe"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain only ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain only ("fee", "fie", "foe"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should (contain only ('E', 'I', 'P', 'U'))
        }
        (fumList should (contain only ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          fumList should (contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with not contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should not contain only ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          toList should not contain only ('h', 'b', 'o', 'y')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOnlyElements", decorateToStringValue(toList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should not contain only ("happy", "birthday", "to")
        intercept[TestFailedException] {
          toList should not contain only ('H', 'B', 'O', 'Y')
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should not contain only ("happy", "birthday", "to")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should not contain only ('H', 'B', 'O', 'Y')) (decided by upperCaseStringEquality)
        }
        toList should not contain only ('H', 'B', 'O', 'Y')
        intercept[TestFailedException] {
          (toList should not contain only ('H', 'B', 'O', 'Y')) (after being lowerCased)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toList should not contain only ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toList should not contain only ('e', 'i', 'p', 'i', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (not contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain only ('H', 'B', 'O', 'Y'))
        val e1 = intercept[TestFailedException] {
          toList should (not contain only ('h', 'b', 'o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOnlyElements", decorateToStringValue(toList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should (not contain only ("NICE", "TO", "MEET", "YOU"))
        intercept[TestFailedException] {
          toList should (not contain only ('H', 'B', 'O', 'Y'))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain only ("NICE", "TO", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should (not contain only ('H', 'B', 'O', 'Y'))) (decided by upperCaseStringEquality)
        }
        toList should (not contain only ('H', 'B', 'O', 'Y'))
        intercept[TestFailedException] {
          (toList should (not contain only ('H', 'B', 'O', 'Y'))) (after being lowerCased)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toList should (not contain only ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          toList should (not contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
  }

  object `a col of Lists` {

    val list1s: Vector[String] = Vector("321", "321", "321")
    val lists: Vector[String] = Vector("321", "321", "432")
    val listsString: Vector[String] = Vector("321", "321", "")
    val hiLists: Vector[String] = Vector("ie", "ie", "ie")
    val toLists: Vector[String] = Vector("oy", "oy", "oy")

    object `when used with contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain only ('1', '2', '3')
        atLeast (2, lists) should contain only ('1', '2', '3')
        atMost (2, lists) should contain only ('1', '2', '3')
        no (lists) should contain only ('3', '4', '5')

        val e1 = intercept[TestFailedException] {
          all (lists) should contain only ('1', '2', '3')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("432") + " did not contain only " + "('1', '2', '3')" + " (StringShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (lists) should contain only ('1', '2', '3')
        }
        e3.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("432") + " did not contain only " + "('1', '2', '3')" + " (StringShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain only ('e', 'i')
        intercept[TestFailedException] {
          all (hiLists) should contain only ('o', 'i')
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should contain only ('E', 'I')
        intercept[TestFailedException] {
          all (hiLists) should contain only ('O', 'I')
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain only ('E', 'I')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain only ('O', 'I')) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should contain only ('e', 'i')) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should contain only ('o', 'i')) (decided by defaultEquality[Char])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain only ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should contain only ('1', '2', '2', '3')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only ('1', '2', '3'))
        atLeast (2, lists) should (contain only ('1', '2', '3'))
        atMost (2, lists) should (contain only ('1', '2', '3'))
        no (lists) should (contain only ('3', '4', '5'))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only ('1', '2', '3'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("432") + " did not contain only " + "('1', '2', '3')" + " (StringShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (lists) should (contain only ('1', '2', '3'))
        }
        e4.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("432") + " did not contain only " + "('1', '2', '3')" + " (StringShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain only ('e', 'i'))
        intercept[TestFailedException] {
          all (hiLists) should (contain only ('o', 'i'))
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should (contain only ('E', 'I'))
        intercept[TestFailedException] {
          all (hiLists) should (contain only ('O', 'I'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain only ('E', 'I'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain only ('O', 'I'))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should (contain only ('e', 'i'))) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should (contain only ('o', 'i'))) (decided by defaultEquality[Char])
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (list1s) should (contain only ('1', '2', '2', '3'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with not contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain only ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain only ('y', 'o')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("oy") + " contained only " + "('y', 'o')" +  " (StringShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should not contain only ("NICE", "MEET", "YOU")
        intercept[TestFailedException] {
          all (toLists) should not contain only ('Y', 'O')
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain only ("NICE", "MEET", "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain only ('Y', 'O')) (decided by upperCaseStringEquality)
        }
        all (toLists) should not contain only ('Y', 'O')
        intercept[TestFailedException] {
          (all (toLists) should not contain only ('Y', 'O')) (after being lowerCased)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain only ()
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should not contain only ('e', 'i', 'p', 'i', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }

    object `when used with (not contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain only ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain only ('y', 'o'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("oy") + " contained only " + "('y', 'o')" + " (StringShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should (not contain only ("NICE", "MEET", "YOU"))
        intercept[TestFailedException] {
          all (toLists) should (not contain only ('Y', 'O'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain only ("NICE", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain only ('Y', 'O'))) (decided by upperCaseStringEquality)
        }
        all (toLists) should (not contain only ('Y', 'O'))
        intercept[TestFailedException] {
          (all (toLists) should (not contain only ('Y', 'O'))) (after being lowerCased)
        }
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS is empty` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain only ())
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyEmpty")))
      }
      def `should throw NotAllowedException with correct stack depth and message when RHS contain duplicated value` {
        val e1 = intercept[exceptions.NotAllowedException] {
          all (toLists) should (not contain only ('e', 'i', 'p', 'i', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some(Resources("onlyDuplicate")))
      }
    }
  }
}
