/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class ArrayShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyArray = Array(1, 2, 3)
  val emptyArray = Array.empty[Int]
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptyArray should (equal (emptyArray) and be (empty))
        emptyArray should (be (empty) and equal (emptyArray))
        
        emptyArray should (be (emptyArray) and be (empty))
        emptyArray should (be (empty) and be (emptyArray))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyArray should (equal (nonEmptyArray) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptyArray, nonEmptyArray) + ", but " + wasNotEmpty(nonEmptyArray)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyArray should (be (empty) and equal (nonEmptyArray))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptyArray)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyArray should (be (nonEmptyArray) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptyArray, nonEmptyArray) + ", but " + wasNotEmpty(nonEmptyArray)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyArray should (be (empty) and be (nonEmptyArray))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptyArray)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyArray should (not equal emptyArray and not be empty)
        nonEmptyArray should (not be empty and not equal emptyArray)
        
        nonEmptyArray should (not be emptyArray and not be empty)
        nonEmptyArray should (not be empty and not be emptyArray)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptyArray should (not equal nonEmptyArray and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptyArray, nonEmptyArray) + ", but " + wasEmpty(emptyArray)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyArray should (not be empty and not equal nonEmptyArray)
        }
        assert(caught2.message === Some(wasEmpty(emptyArray)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyArray should (not be nonEmptyArray and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptyArray, nonEmptyArray) + ", but " + wasEmpty(emptyArray)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyArray should (not be empty and not be nonEmptyArray)
        }
        assert(caught4.message === Some(wasEmpty(emptyArray)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Array(emptyArray)) should (be (emptyArray) and be (empty))
        all(Array(emptyArray)) should (be (empty) and be (emptyArray))
        
        all(Array(emptyArray)) should (equal (emptyArray) and be (empty))
        all(Array(emptyArray)) should (be (empty) and equal (emptyArray))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Array(nonEmptyArray)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptyArray) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptyArray, nonEmptyArray) + ", but " + wasNotEmpty(nonEmptyArray), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Array(nonEmptyArray)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptyArray))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyArray), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Array(nonEmptyArray)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptyArray) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptyArray, nonEmptyArray) + ", but " + wasNotEmpty(nonEmptyArray), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Array(nonEmptyArray)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptyArray))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyArray), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Array(nonEmptyArray)) should (not be empty and not be emptyArray)
        all(Array(nonEmptyArray)) should (not be emptyArray and not be empty)
        
        all(Array(nonEmptyArray)) should (not be empty and not equal emptyArray)
        all(Array(nonEmptyArray)) should (not equal emptyArray and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Array(emptyArray)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptyArray and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptyArray, nonEmptyArray) + ", but " + wasEmpty(emptyArray), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Array(emptyArray)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptyArray)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyArray), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Array(emptyArray)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptyArray and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptyArray, nonEmptyArray) + ", but " + wasEmpty(emptyArray), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Array(emptyArray)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptyArray)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyArray), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
