/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class ArrayShouldBeEmptyLogicalOrSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldBeEmptyLogicalOrSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyArray = Array(1, 2, 3)
  val emptyArray = Array.empty[Int]
  
  object `Sorted matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        
        emptyArray should (be (empty) or be (emptyArray))
        nonEmptyArray should (be (empty) or be (nonEmptyArray))
        emptyArray should (be (empty) or be (nonEmptyArray))
        
        emptyArray should (be (emptyArray) or be (empty))
        emptyArray should (be (nonEmptyArray) or be (empty))
        nonEmptyArray should (be (nonEmptyArray) or be (empty))
        
        emptyArray should (be (empty) or equal (emptyArray))
        nonEmptyArray should (be (empty) or equal (nonEmptyArray))
        emptyArray should (be (empty) or equal (nonEmptyArray))
        
        emptyArray should (equal (emptyArray) or be (empty))
        emptyArray should (equal (nonEmptyArray) or be (empty))
        nonEmptyArray should (equal (nonEmptyArray) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when file is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyArray should (be (empty) or be (emptyArray))
        }
        assert(caught1.message === Some(wasNotEmpty(nonEmptyArray) + ", and " + wasNotEqualTo(nonEmptyArray, emptyArray)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyArray should (be (emptyArray) or be (empty))
        }
        assert(caught2.message === Some(wasNotEqualTo(nonEmptyArray, emptyArray) + ", and " + wasNotEmpty(nonEmptyArray)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyArray should (be (empty) or equal (emptyArray))
        }
        assert(caught3.message === Some(wasNotEmpty(nonEmptyArray) + ", and " + didNotEqual(nonEmptyArray, emptyArray)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyArray should (equal (emptyArray) or be (empty))
        }
        assert(caught4.message === Some(didNotEqual(nonEmptyArray, emptyArray) + ", and " + wasNotEmpty(nonEmptyArray)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'file should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyArray should (not be empty or not be emptyArray)
        emptyArray should (not be empty or not be nonEmptyArray)
        nonEmptyArray should (not be empty or not be nonEmptyArray)
        
        nonEmptyArray should (not be emptyArray or not be empty)
        nonEmptyArray should (not be nonEmptyArray or not be empty)
        emptyArray should (not be nonEmptyArray or not be empty)
        
        nonEmptyArray should (not be empty or not equal emptyArray)
        emptyArray should (not be empty or not equal nonEmptyArray)
        nonEmptyArray should (not be empty or not equal nonEmptyArray)
        
        nonEmptyArray should (not equal emptyArray or not be empty)
        nonEmptyArray should (not equal nonEmptyArray or not be empty)
        emptyArray should (not equal nonEmptyArray or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when file is empty` {
        val caught1 = intercept[TestFailedException] {
          emptyArray should (not be empty or not be emptyArray)
        }
        assert(caught1.message === Some(wasEmpty(emptyArray) + ", and " + wasEqualTo(emptyArray, emptyArray)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyArray should (not be emptyArray or not be empty)
        }
        assert(caught2.message === Some(wasEqualTo(emptyArray, emptyArray) + ", and " + wasEmpty(emptyArray)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyArray should (not be empty or not equal emptyArray)
        }
        assert(caught3.message === Some(wasEmpty(emptyArray) + ", and " + equaled(emptyArray, emptyArray)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyArray should (not equal emptyArray or not be empty)
        }
        assert(caught4.message === Some(equaled(emptyArray, emptyArray) + ", and " + wasEmpty(emptyArray)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Array(emptyArray)) should (be (empty) or be (emptyArray))
        all(Array(nonEmptyArray)) should (be (empty) or be (nonEmptyArray))
        all(Array(emptyArray)) should (be (empty) or be (nonEmptyArray))
        
        all(Array(emptyArray)) should (be (emptyArray) or be (empty))
        all(Array(emptyArray)) should (be (nonEmptyArray) or be (empty))
        all(Array(nonEmptyArray)) should (be (nonEmptyArray) or be (empty))
        
        all(Array(emptyArray)) should (be (empty) or equal (emptyArray))
        all(Array(nonEmptyArray)) should (be (empty) or equal (nonEmptyArray))
        all(Array(emptyArray)) should (be (empty) or equal (nonEmptyArray))
        
        all(Array(emptyArray)) should (equal (emptyArray) or be (empty))
        all(Array(emptyArray)) should (equal (nonEmptyArray) or be (empty))
        all(Array(nonEmptyArray)) should (equal (nonEmptyArray) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Array(nonEmptyArray)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (emptyArray) or be (empty))
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(nonEmptyArray, emptyArray) + ", and " + wasNotEmpty(nonEmptyArray), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Array(nonEmptyArray)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) or be (emptyArray))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyArray) + ", and " + wasNotEqualTo(nonEmptyArray, emptyArray), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Array(nonEmptyArray)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (emptyArray) or be (empty))
        }
        assert(caught3.message === Some(allError(didNotEqual(nonEmptyArray, emptyArray) + ", and " + wasNotEmpty(nonEmptyArray), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Array(nonEmptyArray)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) or equal (emptyArray))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyArray) + ", and " + didNotEqual(nonEmptyArray, emptyArray), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Array(nonEmptyArray)) should (not be empty or not be emptyArray)
        all(Array(emptyArray)) should (not be empty or not be nonEmptyArray)
        all(Array(nonEmptyArray)) should (not be empty or not be nonEmptyArray)
        
        all(Array(nonEmptyArray)) should (not be emptyArray or not be empty)
        all(Array(nonEmptyArray)) should (not be nonEmptyArray or not be empty)
        all(Array(emptyArray)) should (not be nonEmptyArray or not be empty)
        
        all(Array(nonEmptyArray)) should (not be empty or not equal emptyArray)
        all(Array(emptyArray)) should (not be empty or not equal nonEmptyArray)
        all(Array(nonEmptyArray)) should (not be empty or not equal nonEmptyArray)
        
        all(Array(nonEmptyArray)) should (not equal emptyArray or not be empty)
        all(Array(nonEmptyArray)) should (not equal nonEmptyArray or not be empty)
        all(Array(emptyArray)) should (not equal nonEmptyArray or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Array(emptyArray)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be emptyArray or not be empty)
        }
        assert(caught1.message === Some(allError(wasEqualTo(emptyArray, emptyArray) + ", and " + wasEmpty(emptyArray), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Array(emptyArray)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty or not be emptyArray)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyArray) + ", and " + wasEqualTo(emptyArray, emptyArray), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Array(emptyArray)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal emptyArray or not be empty)
        }
        assert(caught3.message === Some(allError(equaled(emptyArray, emptyArray) + ", and " + wasEmpty(emptyArray), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Array(emptyArray)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty or not equal emptyArray)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyArray) + ", and " + equaled(emptyArray, emptyArray), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
