/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class JavaColShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  import SharedHelpers.javaList
  
  val fileName: String = "JavaColShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyjavaList = javaList(1, 2, 3)
  val emptyjavaList = javaList[Int]()
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptyjavaList should (equal (emptyjavaList) and be (empty))
        emptyjavaList should (be (empty) and equal (emptyjavaList))
        
        emptyjavaList should (be (emptyjavaList) and be (empty))
        emptyjavaList should (be (empty) and be (emptyjavaList))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyjavaList should (equal (nonEmptyjavaList) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptyjavaList, nonEmptyjavaList) + ", but " + wasNotEmpty(nonEmptyjavaList)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyjavaList should (be (empty) and equal (nonEmptyjavaList))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptyjavaList)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyjavaList should (be (nonEmptyjavaList) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptyjavaList, nonEmptyjavaList) + ", but " + wasNotEmpty(nonEmptyjavaList)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyjavaList should (be (empty) and be (nonEmptyjavaList))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptyjavaList)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyjavaList should (not equal emptyjavaList and not be empty)
        nonEmptyjavaList should (not be empty and not equal emptyjavaList)
        
        nonEmptyjavaList should (not be emptyjavaList and not be empty)
        nonEmptyjavaList should (not be empty and not be emptyjavaList)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptyjavaList should (not equal nonEmptyjavaList and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptyjavaList, nonEmptyjavaList) + ", but " + wasEmpty(emptyjavaList)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyjavaList should (not be empty and not equal nonEmptyjavaList)
        }
        assert(caught2.message === Some(wasEmpty(emptyjavaList)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyjavaList should (not be nonEmptyjavaList and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptyjavaList, nonEmptyjavaList) + ", but " + wasEmpty(emptyjavaList)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyjavaList should (not be empty and not be nonEmptyjavaList)
        }
        assert(caught4.message === Some(wasEmpty(emptyjavaList)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyjavaList)) should (be (emptyjavaList) and be (empty))
        all(Vector(emptyjavaList)) should (be (empty) and be (emptyjavaList))
        
        all(Vector(emptyjavaList)) should (equal (emptyjavaList) and be (empty))
        all(Vector(emptyjavaList)) should (be (empty) and equal (emptyjavaList))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Vector(nonEmptyjavaList)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptyjavaList) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptyjavaList, nonEmptyjavaList) + ", but " + wasNotEmpty(nonEmptyjavaList), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyjavaList)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptyjavaList))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyjavaList), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyjavaList)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptyjavaList) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptyjavaList, nonEmptyjavaList) + ", but " + wasNotEmpty(nonEmptyjavaList), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyjavaList)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptyjavaList))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyjavaList), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Vector(nonEmptyjavaList)) should (not be empty and not be emptyjavaList)
        all(Vector(nonEmptyjavaList)) should (not be emptyjavaList and not be empty)
        
        all(Vector(nonEmptyjavaList)) should (not be empty and not equal emptyjavaList)
        all(Vector(nonEmptyjavaList)) should (not equal emptyjavaList and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Vector(emptyjavaList)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptyjavaList and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptyjavaList, nonEmptyjavaList) + ", but " + wasEmpty(emptyjavaList), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyjavaList)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptyjavaList)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyjavaList), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyjavaList)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptyjavaList and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptyjavaList, nonEmptyjavaList) + ", but " + wasEmpty(emptyjavaList), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyjavaList)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptyjavaList)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyjavaList), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
