/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class JavaMapShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  import SharedHelpers.javaMap
  
  val fileName: String = "JavaMapShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyjavaMap = javaMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))
  val emptyjavaMap = javaMap[Int, Int]()
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptyjavaMap should (equal (emptyjavaMap) and be (empty))
        emptyjavaMap should (be (empty) and equal (emptyjavaMap))
        
        emptyjavaMap should (be (emptyjavaMap) and be (empty))
        emptyjavaMap should (be (empty) and be (emptyjavaMap))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyjavaMap should (equal (nonEmptyjavaMap) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptyjavaMap, nonEmptyjavaMap) + ", but " + wasNotEmpty(nonEmptyjavaMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyjavaMap should (be (empty) and equal (nonEmptyjavaMap))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptyjavaMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyjavaMap should (be (nonEmptyjavaMap) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptyjavaMap, nonEmptyjavaMap) + ", but " + wasNotEmpty(nonEmptyjavaMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyjavaMap should (be (empty) and be (nonEmptyjavaMap))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptyjavaMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyjavaMap should (not equal emptyjavaMap and not be empty)
        nonEmptyjavaMap should (not be empty and not equal emptyjavaMap)
        
        nonEmptyjavaMap should (not be emptyjavaMap and not be empty)
        nonEmptyjavaMap should (not be empty and not be emptyjavaMap)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptyjavaMap should (not equal nonEmptyjavaMap and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptyjavaMap, nonEmptyjavaMap) + ", but " + wasEmpty(emptyjavaMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyjavaMap should (not be empty and not equal nonEmptyjavaMap)
        }
        assert(caught2.message === Some(wasEmpty(emptyjavaMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyjavaMap should (not be nonEmptyjavaMap and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptyjavaMap, nonEmptyjavaMap) + ", but " + wasEmpty(emptyjavaMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyjavaMap should (not be empty and not be nonEmptyjavaMap)
        }
        assert(caught4.message === Some(wasEmpty(emptyjavaMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyjavaMap)) should (be (emptyjavaMap) and be (empty))
        all(Vector(emptyjavaMap)) should (be (empty) and be (emptyjavaMap))
        
        all(Vector(emptyjavaMap)) should (equal (emptyjavaMap) and be (empty))
        all(Vector(emptyjavaMap)) should (be (empty) and equal (emptyjavaMap))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Vector(nonEmptyjavaMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptyjavaMap) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptyjavaMap, nonEmptyjavaMap) + ", but " + wasNotEmpty(nonEmptyjavaMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyjavaMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptyjavaMap))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyjavaMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyjavaMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptyjavaMap) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptyjavaMap, nonEmptyjavaMap) + ", but " + wasNotEmpty(nonEmptyjavaMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyjavaMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptyjavaMap))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyjavaMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Vector(nonEmptyjavaMap)) should (not be empty and not be emptyjavaMap)
        all(Vector(nonEmptyjavaMap)) should (not be emptyjavaMap and not be empty)
        
        all(Vector(nonEmptyjavaMap)) should (not be empty and not equal emptyjavaMap)
        all(Vector(nonEmptyjavaMap)) should (not equal emptyjavaMap and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Vector(emptyjavaMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptyjavaMap and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptyjavaMap, nonEmptyjavaMap) + ", but " + wasEmpty(emptyjavaMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyjavaMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptyjavaMap)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyjavaMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyjavaMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptyjavaMap and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptyjavaMap, nonEmptyjavaMap) + ", but " + wasEmpty(emptyjavaMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyjavaMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptyjavaMap)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyjavaMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
