/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class JavaMapShouldBeEmptyLogicalOrSpec extends Spec with Matchers {
  
  import SharedHelpers.javaMap
  
  val fileName: String = "JavaMapShouldBeEmptyLogicalOrSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyjavaMap = javaMap(Entry(1, 1), Entry(2, 2), Entry(3, 3))
  val emptyjavaMap = javaMap[Int, Int]()
  
  object `Sorted matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        
        emptyjavaMap should (be (empty) or be (emptyjavaMap))
        nonEmptyjavaMap should (be (empty) or be (nonEmptyjavaMap))
        emptyjavaMap should (be (empty) or be (nonEmptyjavaMap))
        
        emptyjavaMap should (be (emptyjavaMap) or be (empty))
        emptyjavaMap should (be (nonEmptyjavaMap) or be (empty))
        nonEmptyjavaMap should (be (nonEmptyjavaMap) or be (empty))
        
        emptyjavaMap should (be (empty) or equal (emptyjavaMap))
        nonEmptyjavaMap should (be (empty) or equal (nonEmptyjavaMap))
        emptyjavaMap should (be (empty) or equal (nonEmptyjavaMap))
        
        emptyjavaMap should (equal (emptyjavaMap) or be (empty))
        emptyjavaMap should (equal (nonEmptyjavaMap) or be (empty))
        nonEmptyjavaMap should (equal (nonEmptyjavaMap) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when file is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyjavaMap should (be (empty) or be (emptyjavaMap))
        }
        assert(caught1.message === Some(wasNotEmpty(nonEmptyjavaMap) + ", and " + wasNotEqualTo(nonEmptyjavaMap, emptyjavaMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyjavaMap should (be (emptyjavaMap) or be (empty))
        }
        assert(caught2.message === Some(wasNotEqualTo(nonEmptyjavaMap, emptyjavaMap) + ", and " + wasNotEmpty(nonEmptyjavaMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyjavaMap should (be (empty) or equal (emptyjavaMap))
        }
        assert(caught3.message === Some(wasNotEmpty(nonEmptyjavaMap) + ", and " + didNotEqual(nonEmptyjavaMap, emptyjavaMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyjavaMap should (equal (emptyjavaMap) or be (empty))
        }
        assert(caught4.message === Some(didNotEqual(nonEmptyjavaMap, emptyjavaMap) + ", and " + wasNotEmpty(nonEmptyjavaMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'file should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyjavaMap should (not be empty or not be emptyjavaMap)
        emptyjavaMap should (not be empty or not be nonEmptyjavaMap)
        nonEmptyjavaMap should (not be empty or not be nonEmptyjavaMap)
        
        nonEmptyjavaMap should (not be emptyjavaMap or not be empty)
        nonEmptyjavaMap should (not be nonEmptyjavaMap or not be empty)
        emptyjavaMap should (not be nonEmptyjavaMap or not be empty)
        
        nonEmptyjavaMap should (not be empty or not equal emptyjavaMap)
        emptyjavaMap should (not be empty or not equal nonEmptyjavaMap)
        nonEmptyjavaMap should (not be empty or not equal nonEmptyjavaMap)
        
        nonEmptyjavaMap should (not equal emptyjavaMap or not be empty)
        nonEmptyjavaMap should (not equal nonEmptyjavaMap or not be empty)
        emptyjavaMap should (not equal nonEmptyjavaMap or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when file is empty` {
        val caught1 = intercept[TestFailedException] {
          emptyjavaMap should (not be empty or not be emptyjavaMap)
        }
        assert(caught1.message === Some(wasEmpty(emptyjavaMap) + ", and " + wasEqualTo(emptyjavaMap, emptyjavaMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyjavaMap should (not be emptyjavaMap or not be empty)
        }
        assert(caught2.message === Some(wasEqualTo(emptyjavaMap, emptyjavaMap) + ", and " + wasEmpty(emptyjavaMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyjavaMap should (not be empty or not equal emptyjavaMap)
        }
        assert(caught3.message === Some(wasEmpty(emptyjavaMap) + ", and " + equaled(emptyjavaMap, emptyjavaMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyjavaMap should (not equal emptyjavaMap or not be empty)
        }
        assert(caught4.message === Some(equaled(emptyjavaMap, emptyjavaMap) + ", and " + wasEmpty(emptyjavaMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyjavaMap)) should (be (empty) or be (emptyjavaMap))
        all(Vector(nonEmptyjavaMap)) should (be (empty) or be (nonEmptyjavaMap))
        all(Vector(emptyjavaMap)) should (be (empty) or be (nonEmptyjavaMap))
        
        all(Vector(emptyjavaMap)) should (be (emptyjavaMap) or be (empty))
        all(Vector(emptyjavaMap)) should (be (nonEmptyjavaMap) or be (empty))
        all(Vector(nonEmptyjavaMap)) should (be (nonEmptyjavaMap) or be (empty))
        
        all(Vector(emptyjavaMap)) should (be (empty) or equal (emptyjavaMap))
        all(Vector(nonEmptyjavaMap)) should (be (empty) or equal (nonEmptyjavaMap))
        all(Vector(emptyjavaMap)) should (be (empty) or equal (nonEmptyjavaMap))
        
        all(Vector(emptyjavaMap)) should (equal (emptyjavaMap) or be (empty))
        all(Vector(emptyjavaMap)) should (equal (nonEmptyjavaMap) or be (empty))
        all(Vector(nonEmptyjavaMap)) should (equal (nonEmptyjavaMap) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(nonEmptyjavaMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (emptyjavaMap) or be (empty))
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(nonEmptyjavaMap, emptyjavaMap) + ", and " + wasNotEmpty(nonEmptyjavaMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyjavaMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) or be (emptyjavaMap))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyjavaMap) + ", and " + wasNotEqualTo(nonEmptyjavaMap, emptyjavaMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyjavaMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (emptyjavaMap) or be (empty))
        }
        assert(caught3.message === Some(allError(didNotEqual(nonEmptyjavaMap, emptyjavaMap) + ", and " + wasNotEmpty(nonEmptyjavaMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyjavaMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) or equal (emptyjavaMap))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyjavaMap) + ", and " + didNotEqual(nonEmptyjavaMap, emptyjavaMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Vector(nonEmptyjavaMap)) should (not be empty or not be emptyjavaMap)
        all(Vector(emptyjavaMap)) should (not be empty or not be nonEmptyjavaMap)
        all(Vector(nonEmptyjavaMap)) should (not be empty or not be nonEmptyjavaMap)
        
        all(Vector(nonEmptyjavaMap)) should (not be emptyjavaMap or not be empty)
        all(Vector(nonEmptyjavaMap)) should (not be nonEmptyjavaMap or not be empty)
        all(Vector(emptyjavaMap)) should (not be nonEmptyjavaMap or not be empty)
        
        all(Vector(nonEmptyjavaMap)) should (not be empty or not equal emptyjavaMap)
        all(Vector(emptyjavaMap)) should (not be empty or not equal nonEmptyjavaMap)
        all(Vector(nonEmptyjavaMap)) should (not be empty or not equal nonEmptyjavaMap)
        
        all(Vector(nonEmptyjavaMap)) should (not equal emptyjavaMap or not be empty)
        all(Vector(nonEmptyjavaMap)) should (not equal nonEmptyjavaMap or not be empty)
        all(Vector(emptyjavaMap)) should (not equal nonEmptyjavaMap or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(emptyjavaMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be emptyjavaMap or not be empty)
        }
        assert(caught1.message === Some(allError(wasEqualTo(emptyjavaMap, emptyjavaMap) + ", and " + wasEmpty(emptyjavaMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyjavaMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty or not be emptyjavaMap)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyjavaMap) + ", and " + wasEqualTo(emptyjavaMap, emptyjavaMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyjavaMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal emptyjavaMap or not be empty)
        }
        assert(caught3.message === Some(allError(equaled(emptyjavaMap, emptyjavaMap) + ", and " + wasEmpty(emptyjavaMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyjavaMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty or not equal emptyjavaMap)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyjavaMap) + ", and " + equaled(emptyjavaMap, emptyjavaMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
