/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class MapShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "MapShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyMap = Map(1 -> 1, 2 -> 2, 3 -> 3)
  val emptyMap = Map.empty[Int, Int]
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptyMap should (equal (emptyMap) and be (empty))
        emptyMap should (be (empty) and equal (emptyMap))
        
        emptyMap should (be (emptyMap) and be (empty))
        emptyMap should (be (empty) and be (emptyMap))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyMap should (equal (nonEmptyMap) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptyMap, nonEmptyMap) + ", but " + wasNotEmpty(nonEmptyMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyMap should (be (empty) and equal (nonEmptyMap))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptyMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyMap should (be (nonEmptyMap) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptyMap, nonEmptyMap) + ", but " + wasNotEmpty(nonEmptyMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyMap should (be (empty) and be (nonEmptyMap))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptyMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyMap should (not equal emptyMap and not be empty)
        nonEmptyMap should (not be empty and not equal emptyMap)
        
        nonEmptyMap should (not be emptyMap and not be empty)
        nonEmptyMap should (not be empty and not be emptyMap)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptyMap should (not equal nonEmptyMap and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptyMap, nonEmptyMap) + ", but " + wasEmpty(emptyMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyMap should (not be empty and not equal nonEmptyMap)
        }
        assert(caught2.message === Some(wasEmpty(emptyMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyMap should (not be nonEmptyMap and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptyMap, nonEmptyMap) + ", but " + wasEmpty(emptyMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyMap should (not be empty and not be nonEmptyMap)
        }
        assert(caught4.message === Some(wasEmpty(emptyMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyMap)) should (be (emptyMap) and be (empty))
        all(Vector(emptyMap)) should (be (empty) and be (emptyMap))
        
        all(Vector(emptyMap)) should (equal (emptyMap) and be (empty))
        all(Vector(emptyMap)) should (be (empty) and equal (emptyMap))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Vector(nonEmptyMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptyMap) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptyMap, nonEmptyMap) + ", but " + wasNotEmpty(nonEmptyMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptyMap))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptyMap) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptyMap, nonEmptyMap) + ", but " + wasNotEmpty(nonEmptyMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptyMap))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Vector(nonEmptyMap)) should (not be empty and not be emptyMap)
        all(Vector(nonEmptyMap)) should (not be emptyMap and not be empty)
        
        all(Vector(nonEmptyMap)) should (not be empty and not equal emptyMap)
        all(Vector(nonEmptyMap)) should (not equal emptyMap and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Vector(emptyMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptyMap and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptyMap, nonEmptyMap) + ", but " + wasEmpty(emptyMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptyMap)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptyMap and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptyMap, nonEmptyMap) + ", but " + wasEmpty(emptyMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptyMap)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
