/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class MapShouldBeEmptyLogicalOrSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "MapShouldBeEmptyLogicalOrSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyMap = Map(1 -> 1, 2 -> 2, 3 -> 3)
  val emptyMap = Map.empty[Int, Int]
  
  object `Sorted matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        
        emptyMap should (be (empty) or be (emptyMap))
        nonEmptyMap should (be (empty) or be (nonEmptyMap))
        emptyMap should (be (empty) or be (nonEmptyMap))
        
        emptyMap should (be (emptyMap) or be (empty))
        emptyMap should (be (nonEmptyMap) or be (empty))
        nonEmptyMap should (be (nonEmptyMap) or be (empty))
        
        emptyMap should (be (empty) or equal (emptyMap))
        nonEmptyMap should (be (empty) or equal (nonEmptyMap))
        emptyMap should (be (empty) or equal (nonEmptyMap))
        
        emptyMap should (equal (emptyMap) or be (empty))
        emptyMap should (equal (nonEmptyMap) or be (empty))
        nonEmptyMap should (equal (nonEmptyMap) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when file is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyMap should (be (empty) or be (emptyMap))
        }
        assert(caught1.message === Some(wasNotEmpty(nonEmptyMap) + ", and " + wasNotEqualTo(nonEmptyMap, emptyMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyMap should (be (emptyMap) or be (empty))
        }
        assert(caught2.message === Some(wasNotEqualTo(nonEmptyMap, emptyMap) + ", and " + wasNotEmpty(nonEmptyMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyMap should (be (empty) or equal (emptyMap))
        }
        assert(caught3.message === Some(wasNotEmpty(nonEmptyMap) + ", and " + didNotEqual(nonEmptyMap, emptyMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyMap should (equal (emptyMap) or be (empty))
        }
        assert(caught4.message === Some(didNotEqual(nonEmptyMap, emptyMap) + ", and " + wasNotEmpty(nonEmptyMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'file should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyMap should (not be empty or not be emptyMap)
        emptyMap should (not be empty or not be nonEmptyMap)
        nonEmptyMap should (not be empty or not be nonEmptyMap)
        
        nonEmptyMap should (not be emptyMap or not be empty)
        nonEmptyMap should (not be nonEmptyMap or not be empty)
        emptyMap should (not be nonEmptyMap or not be empty)
        
        nonEmptyMap should (not be empty or not equal emptyMap)
        emptyMap should (not be empty or not equal nonEmptyMap)
        nonEmptyMap should (not be empty or not equal nonEmptyMap)
        
        nonEmptyMap should (not equal emptyMap or not be empty)
        nonEmptyMap should (not equal nonEmptyMap or not be empty)
        emptyMap should (not equal nonEmptyMap or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when file is empty` {
        val caught1 = intercept[TestFailedException] {
          emptyMap should (not be empty or not be emptyMap)
        }
        assert(caught1.message === Some(wasEmpty(emptyMap) + ", and " + wasEqualTo(emptyMap, emptyMap)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyMap should (not be emptyMap or not be empty)
        }
        assert(caught2.message === Some(wasEqualTo(emptyMap, emptyMap) + ", and " + wasEmpty(emptyMap)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyMap should (not be empty or not equal emptyMap)
        }
        assert(caught3.message === Some(wasEmpty(emptyMap) + ", and " + equaled(emptyMap, emptyMap)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyMap should (not equal emptyMap or not be empty)
        }
        assert(caught4.message === Some(equaled(emptyMap, emptyMap) + ", and " + wasEmpty(emptyMap)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyMap)) should (be (empty) or be (emptyMap))
        all(Vector(nonEmptyMap)) should (be (empty) or be (nonEmptyMap))
        all(Vector(emptyMap)) should (be (empty) or be (nonEmptyMap))
        
        all(Vector(emptyMap)) should (be (emptyMap) or be (empty))
        all(Vector(emptyMap)) should (be (nonEmptyMap) or be (empty))
        all(Vector(nonEmptyMap)) should (be (nonEmptyMap) or be (empty))
        
        all(Vector(emptyMap)) should (be (empty) or equal (emptyMap))
        all(Vector(nonEmptyMap)) should (be (empty) or equal (nonEmptyMap))
        all(Vector(emptyMap)) should (be (empty) or equal (nonEmptyMap))
        
        all(Vector(emptyMap)) should (equal (emptyMap) or be (empty))
        all(Vector(emptyMap)) should (equal (nonEmptyMap) or be (empty))
        all(Vector(nonEmptyMap)) should (equal (nonEmptyMap) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(nonEmptyMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (emptyMap) or be (empty))
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(nonEmptyMap, emptyMap) + ", and " + wasNotEmpty(nonEmptyMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) or be (emptyMap))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyMap) + ", and " + wasNotEqualTo(nonEmptyMap, emptyMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (emptyMap) or be (empty))
        }
        assert(caught3.message === Some(allError(didNotEqual(nonEmptyMap, emptyMap) + ", and " + wasNotEmpty(nonEmptyMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) or equal (emptyMap))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyMap) + ", and " + didNotEqual(nonEmptyMap, emptyMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Vector(nonEmptyMap)) should (not be empty or not be emptyMap)
        all(Vector(emptyMap)) should (not be empty or not be nonEmptyMap)
        all(Vector(nonEmptyMap)) should (not be empty or not be nonEmptyMap)
        
        all(Vector(nonEmptyMap)) should (not be emptyMap or not be empty)
        all(Vector(nonEmptyMap)) should (not be nonEmptyMap or not be empty)
        all(Vector(emptyMap)) should (not be nonEmptyMap or not be empty)
        
        all(Vector(nonEmptyMap)) should (not be empty or not equal emptyMap)
        all(Vector(emptyMap)) should (not be empty or not equal nonEmptyMap)
        all(Vector(nonEmptyMap)) should (not be empty or not equal nonEmptyMap)
        
        all(Vector(nonEmptyMap)) should (not equal emptyMap or not be empty)
        all(Vector(nonEmptyMap)) should (not equal nonEmptyMap or not be empty)
        all(Vector(emptyMap)) should (not equal nonEmptyMap or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(emptyMap)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be emptyMap or not be empty)
        }
        assert(caught1.message === Some(allError(wasEqualTo(emptyMap, emptyMap) + ", and " + wasEmpty(emptyMap), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyMap)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty or not be emptyMap)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyMap) + ", and " + wasEqualTo(emptyMap, emptyMap), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyMap)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal emptyMap or not be empty)
        }
        assert(caught3.message === Some(allError(equaled(emptyMap, emptyMap) + ", and " + wasEmpty(emptyMap), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyMap)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty or not equal emptyMap)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyMap) + ", and " + equaled(emptyMap, emptyMap), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
