/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class OptionShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "OptionShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyOption = Some(123)
  val emptyOption = None
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptyOption should (equal (emptyOption) and be (empty))
        emptyOption should (be (empty) and equal (emptyOption))
        
        emptyOption should (be (emptyOption) and be (empty))
        emptyOption should (be (empty) and be (emptyOption))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyOption should (equal (nonEmptyOption) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptyOption, nonEmptyOption) + ", but " + wasNotEmpty(nonEmptyOption)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyOption should (be (empty) and equal (nonEmptyOption))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptyOption)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyOption should (be (nonEmptyOption) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptyOption, nonEmptyOption) + ", but " + wasNotEmpty(nonEmptyOption)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyOption should (be (empty) and be (nonEmptyOption))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptyOption)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyOption should (not equal emptyOption and not be empty)
        nonEmptyOption should (not be empty and not equal emptyOption)
        
        nonEmptyOption should (not be emptyOption and not be empty)
        nonEmptyOption should (not be empty and not be emptyOption)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptyOption should (not equal nonEmptyOption and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptyOption, nonEmptyOption) + ", but " + wasEmpty(emptyOption)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyOption should (not be empty and not equal nonEmptyOption)
        }
        assert(caught2.message === Some(wasEmpty(emptyOption)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyOption should (not be nonEmptyOption and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptyOption, nonEmptyOption) + ", but " + wasEmpty(emptyOption)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyOption should (not be empty and not be nonEmptyOption)
        }
        assert(caught4.message === Some(wasEmpty(emptyOption)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyOption)) should (be (emptyOption) and be (empty))
        all(Vector(emptyOption)) should (be (empty) and be (emptyOption))
        
        all(Vector(emptyOption)) should (equal (emptyOption) and be (empty))
        all(Vector(emptyOption)) should (be (empty) and equal (emptyOption))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Vector(nonEmptyOption)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptyOption) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptyOption, nonEmptyOption) + ", but " + wasNotEmpty(nonEmptyOption), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyOption)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptyOption))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyOption), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyOption)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptyOption) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptyOption, nonEmptyOption) + ", but " + wasNotEmpty(nonEmptyOption), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyOption)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptyOption))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyOption), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Vector(nonEmptyOption)) should (not be empty and not be emptyOption)
        all(Vector(nonEmptyOption)) should (not be emptyOption and not be empty)
        
        all(Vector(nonEmptyOption)) should (not be empty and not equal emptyOption)
        all(Vector(nonEmptyOption)) should (not equal emptyOption and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Vector(emptyOption)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptyOption and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptyOption, nonEmptyOption) + ", but " + wasEmpty(emptyOption), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyOption)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptyOption)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyOption), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyOption)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptyOption and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptyOption, nonEmptyOption) + ", but " + wasEmpty(emptyOption), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyOption)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptyOption)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyOption), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
