/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class OptionShouldBeEmptyLogicalOrSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "OptionShouldBeEmptyLogicalOrSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyOption = Some(123)
  val emptyOption = None
  
  object `Sorted matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        
        emptyOption should (be (empty) or be (emptyOption))
        nonEmptyOption should (be (empty) or be (nonEmptyOption))
        emptyOption should (be (empty) or be (nonEmptyOption))
        
        emptyOption should (be (emptyOption) or be (empty))
        emptyOption should (be (nonEmptyOption) or be (empty))
        nonEmptyOption should (be (nonEmptyOption) or be (empty))
        
        emptyOption should (be (empty) or equal (emptyOption))
        nonEmptyOption should (be (empty) or equal (nonEmptyOption))
        emptyOption should (be (empty) or equal (nonEmptyOption))
        
        emptyOption should (equal (emptyOption) or be (empty))
        emptyOption should (equal (nonEmptyOption) or be (empty))
        nonEmptyOption should (equal (nonEmptyOption) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when file is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyOption should (be (empty) or be (emptyOption))
        }
        assert(caught1.message === Some(wasNotEmpty(nonEmptyOption) + ", and " + wasNotEqualTo(nonEmptyOption, emptyOption)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyOption should (be (emptyOption) or be (empty))
        }
        assert(caught2.message === Some(wasNotEqualTo(nonEmptyOption, emptyOption) + ", and " + wasNotEmpty(nonEmptyOption)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyOption should (be (empty) or equal (emptyOption))
        }
        assert(caught3.message === Some(wasNotEmpty(nonEmptyOption) + ", and " + didNotEqual(nonEmptyOption, emptyOption)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyOption should (equal (emptyOption) or be (empty))
        }
        assert(caught4.message === Some(didNotEqual(nonEmptyOption, emptyOption) + ", and " + wasNotEmpty(nonEmptyOption)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'file should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyOption should (not be empty or not be emptyOption)
        emptyOption should (not be empty or not be nonEmptyOption)
        nonEmptyOption should (not be empty or not be nonEmptyOption)
        
        nonEmptyOption should (not be emptyOption or not be empty)
        nonEmptyOption should (not be nonEmptyOption or not be empty)
        emptyOption should (not be nonEmptyOption or not be empty)
        
        nonEmptyOption should (not be empty or not equal emptyOption)
        emptyOption should (not be empty or not equal nonEmptyOption)
        nonEmptyOption should (not be empty or not equal nonEmptyOption)
        
        nonEmptyOption should (not equal emptyOption or not be empty)
        nonEmptyOption should (not equal nonEmptyOption or not be empty)
        emptyOption should (not equal nonEmptyOption or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when file is empty` {
        val caught1 = intercept[TestFailedException] {
          emptyOption should (not be empty or not be emptyOption)
        }
        assert(caught1.message === Some(wasEmpty(emptyOption) + ", and " + wasEqualTo(emptyOption, emptyOption)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyOption should (not be emptyOption or not be empty)
        }
        assert(caught2.message === Some(wasEqualTo(emptyOption, emptyOption) + ", and " + wasEmpty(emptyOption)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyOption should (not be empty or not equal emptyOption)
        }
        assert(caught3.message === Some(wasEmpty(emptyOption) + ", and " + equaled(emptyOption, emptyOption)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyOption should (not equal emptyOption or not be empty)
        }
        assert(caught4.message === Some(equaled(emptyOption, emptyOption) + ", and " + wasEmpty(emptyOption)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyOption)) should (be (empty) or be (emptyOption))
        all(Vector(nonEmptyOption)) should (be (empty) or be (nonEmptyOption))
        all(Vector(emptyOption)) should (be (empty) or be (nonEmptyOption))
        
        all(Vector(emptyOption)) should (be (emptyOption) or be (empty))
        all(Vector(emptyOption)) should (be (nonEmptyOption) or be (empty))
        all(Vector(nonEmptyOption)) should (be (nonEmptyOption) or be (empty))
        
        all(Vector(emptyOption)) should (be (empty) or equal (emptyOption))
        all(Vector(nonEmptyOption)) should (be (empty) or equal (nonEmptyOption))
        all(Vector(emptyOption)) should (be (empty) or equal (nonEmptyOption))
        
        all(Vector(emptyOption)) should (equal (emptyOption) or be (empty))
        all(Vector(emptyOption)) should (equal (nonEmptyOption) or be (empty))
        all(Vector(nonEmptyOption)) should (equal (nonEmptyOption) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(nonEmptyOption)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (emptyOption) or be (empty))
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(nonEmptyOption, emptyOption) + ", and " + wasNotEmpty(nonEmptyOption), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyOption)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) or be (emptyOption))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyOption) + ", and " + wasNotEqualTo(nonEmptyOption, emptyOption), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyOption)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (emptyOption) or be (empty))
        }
        assert(caught3.message === Some(allError(didNotEqual(nonEmptyOption, emptyOption) + ", and " + wasNotEmpty(nonEmptyOption), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyOption)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) or equal (emptyOption))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyOption) + ", and " + didNotEqual(nonEmptyOption, emptyOption), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Vector(nonEmptyOption)) should (not be empty or not be emptyOption)
        all(Vector(emptyOption)) should (not be empty or not be nonEmptyOption)
        all(Vector(nonEmptyOption)) should (not be empty or not be nonEmptyOption)
        
        all(Vector(nonEmptyOption)) should (not be emptyOption or not be empty)
        all(Vector(nonEmptyOption)) should (not be nonEmptyOption or not be empty)
        all(Vector(emptyOption)) should (not be nonEmptyOption or not be empty)
        
        all(Vector(nonEmptyOption)) should (not be empty or not equal emptyOption)
        all(Vector(emptyOption)) should (not be empty or not equal nonEmptyOption)
        all(Vector(nonEmptyOption)) should (not be empty or not equal nonEmptyOption)
        
        all(Vector(nonEmptyOption)) should (not equal emptyOption or not be empty)
        all(Vector(nonEmptyOption)) should (not equal nonEmptyOption or not be empty)
        all(Vector(emptyOption)) should (not equal nonEmptyOption or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Vector(emptyOption)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be emptyOption or not be empty)
        }
        assert(caught1.message === Some(allError(wasEqualTo(emptyOption, emptyOption) + ", and " + wasEmpty(emptyOption), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyOption)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty or not be emptyOption)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyOption) + ", and " + wasEqualTo(emptyOption, emptyOption), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyOption)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal emptyOption or not be empty)
        }
        assert(caught3.message === Some(allError(equaled(emptyOption, emptyOption) + ", and " + wasEmpty(emptyOption), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyOption)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty or not equal emptyOption)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyOption) + ", and " + equaled(emptyOption, emptyOption), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
