/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class SetShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "SetShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptySet = Set(1, 2, 3)
  val emptySet = Set.empty[Int]
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptySet should (equal (emptySet) and be (empty))
        emptySet should (be (empty) and equal (emptySet))
        
        emptySet should (be (emptySet) and be (empty))
        emptySet should (be (empty) and be (emptySet))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptySet should (equal (nonEmptySet) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptySet, nonEmptySet) + ", but " + wasNotEmpty(nonEmptySet)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptySet should (be (empty) and equal (nonEmptySet))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptySet)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptySet should (be (nonEmptySet) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptySet, nonEmptySet) + ", but " + wasNotEmpty(nonEmptySet)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptySet should (be (empty) and be (nonEmptySet))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptySet)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptySet should (not equal emptySet and not be empty)
        nonEmptySet should (not be empty and not equal emptySet)
        
        nonEmptySet should (not be emptySet and not be empty)
        nonEmptySet should (not be empty and not be emptySet)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptySet should (not equal nonEmptySet and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptySet, nonEmptySet) + ", but " + wasEmpty(emptySet)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptySet should (not be empty and not equal nonEmptySet)
        }
        assert(caught2.message === Some(wasEmpty(emptySet)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptySet should (not be nonEmptySet and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptySet, nonEmptySet) + ", but " + wasEmpty(emptySet)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptySet should (not be empty and not be nonEmptySet)
        }
        assert(caught4.message === Some(wasEmpty(emptySet)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Set(emptySet)) should (be (emptySet) and be (empty))
        all(Set(emptySet)) should (be (empty) and be (emptySet))
        
        all(Set(emptySet)) should (equal (emptySet) and be (empty))
        all(Set(emptySet)) should (be (empty) and equal (emptySet))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Set(nonEmptySet)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptySet) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptySet, nonEmptySet) + ", but " + wasNotEmpty(nonEmptySet), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Set(nonEmptySet)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptySet))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptySet), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Set(nonEmptySet)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptySet) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptySet, nonEmptySet) + ", but " + wasNotEmpty(nonEmptySet), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Set(nonEmptySet)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptySet))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptySet), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Set(nonEmptySet)) should (not be empty and not be emptySet)
        all(Set(nonEmptySet)) should (not be emptySet and not be empty)
        
        all(Set(nonEmptySet)) should (not be empty and not equal emptySet)
        all(Set(nonEmptySet)) should (not equal emptySet and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Set(emptySet)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptySet and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptySet, nonEmptySet) + ", but " + wasEmpty(emptySet), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Set(emptySet)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptySet)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptySet), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Set(emptySet)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptySet and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptySet, nonEmptySet) + ", but " + wasEmpty(emptySet), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Set(emptySet)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptySet)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptySet), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
