/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class SetShouldBeEmptyLogicalOrSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "SetShouldBeEmptyLogicalOrSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptySet = Set(1, 2, 3)
  val emptySet = Set.empty[Int]
  
  object `Sorted matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        
        emptySet should (be (empty) or be (emptySet))
        nonEmptySet should (be (empty) or be (nonEmptySet))
        emptySet should (be (empty) or be (nonEmptySet))
        
        emptySet should (be (emptySet) or be (empty))
        emptySet should (be (nonEmptySet) or be (empty))
        nonEmptySet should (be (nonEmptySet) or be (empty))
        
        emptySet should (be (empty) or equal (emptySet))
        nonEmptySet should (be (empty) or equal (nonEmptySet))
        emptySet should (be (empty) or equal (nonEmptySet))
        
        emptySet should (equal (emptySet) or be (empty))
        emptySet should (equal (nonEmptySet) or be (empty))
        nonEmptySet should (equal (nonEmptySet) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when file is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptySet should (be (empty) or be (emptySet))
        }
        assert(caught1.message === Some(wasNotEmpty(nonEmptySet) + ", and " + wasNotEqualTo(nonEmptySet, emptySet)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptySet should (be (emptySet) or be (empty))
        }
        assert(caught2.message === Some(wasNotEqualTo(nonEmptySet, emptySet) + ", and " + wasNotEmpty(nonEmptySet)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptySet should (be (empty) or equal (emptySet))
        }
        assert(caught3.message === Some(wasNotEmpty(nonEmptySet) + ", and " + didNotEqual(nonEmptySet, emptySet)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptySet should (equal (emptySet) or be (empty))
        }
        assert(caught4.message === Some(didNotEqual(nonEmptySet, emptySet) + ", and " + wasNotEmpty(nonEmptySet)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'file should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptySet should (not be empty or not be emptySet)
        emptySet should (not be empty or not be nonEmptySet)
        nonEmptySet should (not be empty or not be nonEmptySet)
        
        nonEmptySet should (not be emptySet or not be empty)
        nonEmptySet should (not be nonEmptySet or not be empty)
        emptySet should (not be nonEmptySet or not be empty)
        
        nonEmptySet should (not be empty or not equal emptySet)
        emptySet should (not be empty or not equal nonEmptySet)
        nonEmptySet should (not be empty or not equal nonEmptySet)
        
        nonEmptySet should (not equal emptySet or not be empty)
        nonEmptySet should (not equal nonEmptySet or not be empty)
        emptySet should (not equal nonEmptySet or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when file is empty` {
        val caught1 = intercept[TestFailedException] {
          emptySet should (not be empty or not be emptySet)
        }
        assert(caught1.message === Some(wasEmpty(emptySet) + ", and " + wasEqualTo(emptySet, emptySet)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptySet should (not be emptySet or not be empty)
        }
        assert(caught2.message === Some(wasEqualTo(emptySet, emptySet) + ", and " + wasEmpty(emptySet)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptySet should (not be empty or not equal emptySet)
        }
        assert(caught3.message === Some(wasEmpty(emptySet) + ", and " + equaled(emptySet, emptySet)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptySet should (not equal emptySet or not be empty)
        }
        assert(caught4.message === Some(equaled(emptySet, emptySet) + ", and " + wasEmpty(emptySet)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Set(emptySet)) should (be (empty) or be (emptySet))
        all(Set(nonEmptySet)) should (be (empty) or be (nonEmptySet))
        all(Set(emptySet)) should (be (empty) or be (nonEmptySet))
        
        all(Set(emptySet)) should (be (emptySet) or be (empty))
        all(Set(emptySet)) should (be (nonEmptySet) or be (empty))
        all(Set(nonEmptySet)) should (be (nonEmptySet) or be (empty))
        
        all(Set(emptySet)) should (be (empty) or equal (emptySet))
        all(Set(nonEmptySet)) should (be (empty) or equal (nonEmptySet))
        all(Set(emptySet)) should (be (empty) or equal (nonEmptySet))
        
        all(Set(emptySet)) should (equal (emptySet) or be (empty))
        all(Set(emptySet)) should (equal (nonEmptySet) or be (empty))
        all(Set(nonEmptySet)) should (equal (nonEmptySet) or be (empty))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Set(nonEmptySet)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (emptySet) or be (empty))
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(nonEmptySet, emptySet) + ", and " + wasNotEmpty(nonEmptySet), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Set(nonEmptySet)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) or be (emptySet))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptySet) + ", and " + wasNotEqualTo(nonEmptySet, emptySet), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Set(nonEmptySet)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (emptySet) or be (empty))
        }
        assert(caught3.message === Some(allError(didNotEqual(nonEmptySet, emptySet) + ", and " + wasNotEmpty(nonEmptySet), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Set(nonEmptySet)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) or equal (emptySet))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptySet) + ", and " + didNotEqual(nonEmptySet, emptySet), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Set(nonEmptySet)) should (not be empty or not be emptySet)
        all(Set(emptySet)) should (not be empty or not be nonEmptySet)
        all(Set(nonEmptySet)) should (not be empty or not be nonEmptySet)
        
        all(Set(nonEmptySet)) should (not be emptySet or not be empty)
        all(Set(nonEmptySet)) should (not be nonEmptySet or not be empty)
        all(Set(emptySet)) should (not be nonEmptySet or not be empty)
        
        all(Set(nonEmptySet)) should (not be empty or not equal emptySet)
        all(Set(emptySet)) should (not be empty or not equal nonEmptySet)
        all(Set(nonEmptySet)) should (not be empty or not equal nonEmptySet)
        
        all(Set(nonEmptySet)) should (not equal emptySet or not be empty)
        all(Set(nonEmptySet)) should (not equal nonEmptySet or not be empty)
        all(Set(emptySet)) should (not equal nonEmptySet or not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Set(emptySet)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be emptySet or not be empty)
        }
        assert(caught1.message === Some(allError(wasEqualTo(emptySet, emptySet) + ", and " + wasEmpty(emptySet), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Set(emptySet)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty or not be emptySet)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptySet) + ", and " + wasEqualTo(emptySet, emptySet), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Set(emptySet)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal emptySet or not be empty)
        }
        assert(caught3.message === Some(allError(equaled(emptySet, emptySet) + ", and " + wasEmpty(emptySet), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Set(emptySet)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty or not equal emptySet)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptySet) + ", and " + equaled(emptySet, emptySet), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
