/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber

class StringShouldBeEmptyLogicalAndSpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  val fileName: String = "StringShouldBeEmptyLogicalAndSpec.scala"
  
  def wasEqualTo(left: Any, right: Any): String = 
    FailureMessages("wasEqualTo", left, right)
    
  def wasNotEqualTo(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("wasNotEqualTo", leftee, rightee)
  }
    
  def equaled(left: Any, right: Any): String = 
    FailureMessages("equaled", left, right)
    
  def didNotEqual(left: Any, right: Any): String = {
    val (leftee, rightee) = Suite.getObjectsForFailureMessage(left, right)
    FailureMessages("didNotEqual", leftee, rightee)
  }
  
  def wasNotEmpty(left: Any): String = 
    FailureMessages("wasNotEmpty", left)
    
  def wasEmpty(left: Any): String = 
    FailureMessages("wasEmpty", left)
    
  def allError(message: String, lineNumber: Int, left: Any): String = {
    val messageWithIndex = UnquotedString("  " + FailureMessages("forAssertionsGenTraversableMessageWithStackDepth", 0, UnquotedString(message), UnquotedString(fileName + ":" + lineNumber)))
    FailureMessages("allShorthandFailed", messageWithIndex, left)
  }
    
  val nonEmptyString = "123"
  val emptyString = ""
  
  object `Emptiness matcher` {
    
    object `when work with 'list should be (empty)'` {
      
      def `should do nothing when list is empty` {
        emptyString should (equal (emptyString) and be (empty))
        emptyString should (be (empty) and equal (emptyString))
        
        emptyString should (be (emptyString) and be (empty))
        emptyString should (be (empty) and be (emptyString))
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          nonEmptyString should (equal (nonEmptyString) and be (empty))
        }
        assert(caught1.message === Some(equaled(nonEmptyString, nonEmptyString) + ", but " + wasNotEmpty(nonEmptyString)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          nonEmptyString should (be (empty) and equal (nonEmptyString))
        }
        assert(caught2.message === Some(wasNotEmpty(nonEmptyString)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          nonEmptyString should (be (nonEmptyString) and be (empty))
        }
        assert(caught3.message === Some(wasEqualTo(nonEmptyString, nonEmptyString) + ", but " + wasNotEmpty(nonEmptyString)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          nonEmptyString should (be (empty) and be (nonEmptyString))
        }
        assert(caught4.message === Some(wasNotEmpty(nonEmptyString)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'list should not be empty'` {
      
      def `should do nothing when file is not empty` {
        nonEmptyString should (not equal emptyString and not be empty)
        nonEmptyString should (not be empty and not equal emptyString)
        
        nonEmptyString should (not be emptyString and not be empty)
        nonEmptyString should (not be empty and not be emptyString)
      }
      
      def `should throw TestFailedException with correct stack depth when list is not empty` {
        val caught1 = intercept[TestFailedException] {
          emptyString should (not equal nonEmptyString and not be empty)
        }
        assert(caught1.message === Some(didNotEqual(emptyString, nonEmptyString) + ", but " + wasEmpty(emptyString)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          emptyString should (not be empty and not equal nonEmptyString)
        }
        assert(caught2.message === Some(wasEmpty(emptyString)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          emptyString should (not be nonEmptyString and not be empty)
        }
        assert(caught3.message === Some(wasNotEqualTo(emptyString, nonEmptyString) + ", but " + wasEmpty(emptyString)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          emptyString should (not be empty and not be nonEmptyString)
        }
        assert(caught4.message === Some(wasEmpty(emptyString)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should be (empty)'` {
      
      def `should do nothing when all(xs) is empty` {
        all(Vector(emptyString)) should (be (emptyString) and be (empty))
        all(Vector(emptyString)) should (be (empty) and be (emptyString))
        
        all(Vector(emptyString)) should (equal (emptyString) and be (empty))
        all(Vector(emptyString)) should (be (empty) and equal (emptyString))
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is not empty` {
        val left1 = Vector(nonEmptyString)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (nonEmptyString) and be (empty))
        }
        assert(caught1.message === Some(allError(wasEqualTo(nonEmptyString, nonEmptyString) + ", but " + wasNotEmpty(nonEmptyString), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(nonEmptyString)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (empty) and be (nonEmptyString))
        }
        assert(caught2.message === Some(allError(wasNotEmpty(nonEmptyString), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(nonEmptyString)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (nonEmptyString) and be (empty))
        }
        assert(caught3.message === Some(allError(equaled(nonEmptyString, nonEmptyString) + ", but " + wasNotEmpty(nonEmptyString), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(nonEmptyString)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (empty) and equal (nonEmptyString))
        }
        assert(caught4.message === Some(allError(wasNotEmpty(nonEmptyString), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
    
    object `when work with 'all(xs) should not be empty'` {
      def `should do nothing when all(xs) is not empty` {
        all(Vector(nonEmptyString)) should (not be empty and not be emptyString)
        all(Vector(nonEmptyString)) should (not be emptyString and not be empty)
        
        all(Vector(nonEmptyString)) should (not be empty and not equal emptyString)
        all(Vector(nonEmptyString)) should (not equal emptyString and not be empty)
      }
      
      def `should throw TestFailedException with correct stack depth when all(xs) is empty` {
        val left1 = Vector(emptyString)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be nonEmptyString and not be empty)
        }
        assert(caught1.message === Some(allError(wasNotEqualTo(emptyString, nonEmptyString) + ", but " + wasEmpty(emptyString), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some(fileName))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Vector(emptyString)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be empty and not be nonEmptyString)
        }
        assert(caught2.message === Some(allError(wasEmpty(emptyString), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some(fileName))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Vector(emptyString)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal nonEmptyString and not be empty)
        }
        assert(caught3.message === Some(allError(didNotEqual(emptyString, nonEmptyString) + ", but " + wasEmpty(emptyString), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some(fileName))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Vector(emptyString)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be empty and not equal nonEmptyString)
        }
        assert(caught4.message === Some(allError(wasEmpty(emptyString), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some(fileName))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
    }
  }
}
