/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import SharedHelpers.thisLineNumber
import org.scalatest.enablers.Sortable
import FailureMessages.decorateToStringValue

class ShouldBeSortedLogicalAndForArraySpec extends Spec with Matchers {
  
  //ADDITIONAL//
  
  def wasEqualTo(left: Any, right: Any): String = 
    decorateToStringValue(left) + " was equal to " + decorateToStringValue(right)
    
  def wasNotEqualTo(left: Any, right: Any): String = 
    decorateToStringValue(left) + " was not equal to " + decorateToStringValue(right)
    
  def equaled(left: Any, right: Any): String = 
    decorateToStringValue(left) + " equaled " + decorateToStringValue(right)
    
  def didNotEqual(left: Any, right: Any): String = 
    decorateToStringValue(left) + " did not equal " + decorateToStringValue(right)
  
  def wasNotSorted(left: Any): String = 
    decorateToStringValue(left) + " was not sorted"
    
  def wasSorted(left: Any): String = 
    decorateToStringValue(left) + " was sorted"
    
  def allInspectionFailed(idx: Int, message: String, lineNumber:Int, left: Any) = 
    "'all' inspection failed, because: \n" + 
    "  at index " + idx + ", " + message + " (ShouldBeSortedLogicalAndForArraySpec.scala:" + lineNumber + ") \n" + 
    "in " + decorateToStringValue(left)
    
  case class Student(name: String, scores: Int)
  implicit val studentOrdering = new Ordering[Student] {
    def compare(a: Student, b: Student) = a.scores compare b.scores
  }
  
  val orderedInts = Array(1, 2, 3)
  val outOfOrderInts = Array(3, 2, 1)
  
  val orderedStudents = Array(Student("Student 1", 80), Student("Student 2", 88), Student("Student 3", 90))
  val outOfOrderStudents = Array(Student("Student 3", 90), Student("Student 2", 88), Student("Student 1", 80))
  
  val trueSortable = 
    new Sortable[Array[Int]] {
      def isSorted(o: Array[Int]) = true
    }
  
  val falseSortable = 
    new Sortable[Array[Int]] {
      def isSorted(o: Array[Int]) = false
    }
  
  object `Sorted matcher` {
    
    object `when work with 'xs should be (sorted)'` {
      
      def `should do nothing when xs is sorted` {
        orderedStudents should (be (sorted) and be (sorted))
        orderedStudents should (be (sorted) and be (orderedStudents))
        orderedStudents should (be (orderedStudents) and be (sorted))
        
        orderedStudents should (be (sorted) and be (sorted))
        orderedStudents should (be (sorted) and equal (orderedStudents))
        orderedStudents should (equal (orderedStudents) and be (sorted))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val caught1 = intercept[TestFailedException] {
          outOfOrderInts should (be (outOfOrderInts) and be (sorted))
        }
        assert(caught1.message === Some(wasEqualTo(outOfOrderInts, outOfOrderInts) + ", but " + wasNotSorted(outOfOrderInts)))
        assert(caught1.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          outOfOrderInts should (be (sorted) and be (outOfOrderInts))
        }
        assert(caught2.message === Some(wasNotSorted(outOfOrderInts)))
        assert(caught2.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          (outOfOrderStudents should (equal (outOfOrderStudents) and be (sorted)))
        }
        assert(caught3.message === Some(equaled(outOfOrderStudents, outOfOrderStudents) + ", but " + wasNotSorted(outOfOrderStudents)))
        assert(caught3.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          (outOfOrderStudents should (be (sorted) and equal (outOfOrderStudents)))
        }
        assert(caught4.message === Some(wasNotSorted(outOfOrderStudents)))
        assert(caught4.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
      
      def `should use implicit Sortable when available` {
        intercept[TestFailedException] {
          outOfOrderInts should (be (sorted) and be (outOfOrderInts))
        }
        implicit val imp = trueSortable
        outOfOrderInts should (be (sorted) and be (outOfOrderInts))
      }
      
      def `should use explicitly specified Sortable` {
        intercept[TestFailedException] {
          outOfOrderInts should (be (sorted) and equal (outOfOrderInts))
        }
        (outOfOrderInts should (be (sorted) and equal (outOfOrderInts))) (trueSortable, defaultEquality)
      }
    }
    
    object `when work with 'xs should not be sorted'` {
      
      def `should do nothing when xs is not sorted` {
        outOfOrderInts should (not be sorted and not be outOfOrderStudents)
        outOfOrderInts should (not be outOfOrderStudents and not be sorted)
        
        outOfOrderStudents should (not be sorted and not equal outOfOrderInts)
        outOfOrderStudents should (not equal outOfOrderInts and not be sorted)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val caught1 = intercept[TestFailedException] {
          orderedInts should (not be outOfOrderStudents and not be sorted)
        }
        assert(caught1.message === Some(wasNotEqualTo(orderedInts, outOfOrderStudents) + ", but " + wasSorted(orderedInts)))
        assert(caught1.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught2 = intercept[TestFailedException] {
          orderedInts should (not be sorted and not be outOfOrderStudents)
        }
        assert(caught2.message === Some(wasSorted(orderedInts)))
        assert(caught2.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught3 = intercept[TestFailedException] {
          orderedStudents should (not equal outOfOrderInts and not be sorted)
        }
        assert(caught3.message === Some(didNotEqual(orderedStudents, outOfOrderInts) + ", but " + wasSorted(orderedStudents)))
        assert(caught3.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val caught4 = intercept[TestFailedException] {
          orderedStudents should (not be sorted and not equal outOfOrderInts)
        }
        assert(caught4.message === Some(wasSorted(orderedStudents)))
        assert(caught4.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
      
      def `should use implicit Sortable when available` {
        intercept[TestFailedException] {
          orderedInts should (not be (sorted) and not be (outOfOrderInts))
        }
        implicit val imp = falseSortable
        orderedInts should (not be (sorted) and not be (outOfOrderInts))
      }
      
      def `should use explicitly specified Sortable` {
        intercept[TestFailedException] {
          orderedInts should (not be (sorted) and not equal (outOfOrderInts))
        }
        (orderedInts should (not be (sorted) and not equal (outOfOrderInts))) (falseSortable, defaultEquality)
      }
    }
    
    object `when work with 'all(xs) should be (sorted)'` {
      
      def `should do nothing when xs is sorted` {
        all(Array(orderedStudents)) should (be (sorted) and be (sorted))
        all(Array(orderedStudents)) should (be (sorted) and be (orderedStudents))
        all(Array(orderedStudents)) should (be (orderedStudents) and be (sorted))
        
        all(Array(orderedStudents)) should (be (sorted) and be (sorted))
        all(Array(orderedStudents)) should (be (sorted) and equal (orderedStudents))
        all(Array(orderedStudents)) should (equal (orderedStudents) and be (sorted))
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Array(outOfOrderInts)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (be (outOfOrderInts) and be (sorted))
        }
        assert(caught1.message === Some(allInspectionFailed(0, wasEqualTo(outOfOrderInts, outOfOrderInts) + ", but " + wasNotSorted(outOfOrderInts), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Array(outOfOrderInts)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (be (sorted) and be (outOfOrderInts))
        }
        assert(caught2.message === Some(allInspectionFailed(0, wasNotSorted(outOfOrderInts), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Array(outOfOrderStudents)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (equal (outOfOrderStudents) and be (sorted))
        }
        assert(caught3.message === Some(allInspectionFailed(0, equaled(outOfOrderStudents, outOfOrderStudents) + ", but " + wasNotSorted(outOfOrderStudents), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Array(outOfOrderStudents)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (be (sorted) and equal (outOfOrderStudents))
        }
        assert(caught4.message === Some(allInspectionFailed(0, wasNotSorted(outOfOrderStudents), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
      
      def `should use implicit Sortable when available` {
        intercept[TestFailedException] {
          all(Array(outOfOrderInts)) should (be (sorted) and be (outOfOrderInts))
        }
        implicit val imp = trueSortable
        all(Array(outOfOrderInts)) should (be (sorted) and be (outOfOrderInts))
      }
      
      def `should use explicitly specified Sortable` {
        intercept[TestFailedException] {
          all(Array(outOfOrderInts)) should (be (sorted) and equal (outOfOrderInts))
        }
        (all(Array(outOfOrderInts)) should (be (sorted) and equal (outOfOrderInts))) (trueSortable, defaultEquality)
      }
    }
    
    object `when work with 'all(xs) should not be sorted'` {
      def `should do nothing when xs is not sorted` {
        all(Array(outOfOrderInts)) should (not be sorted and not be outOfOrderStudents)
        all(Array(outOfOrderInts)) should (not be outOfOrderStudents and not be sorted)
        
        all(Array(outOfOrderStudents)) should (not be sorted and not equal outOfOrderInts)
        all(Array(outOfOrderStudents)) should (not equal outOfOrderInts and not be sorted)
      }
      
      def `should throw TestFailedException with correct stack depth when xs is not sorted` {
        val left1 = Array(orderedInts)
        val caught1 = intercept[TestFailedException] {
          all(left1) should (not be outOfOrderStudents and not be sorted)
        }
        assert(caught1.message === Some(allInspectionFailed(0, wasNotEqualTo(orderedInts, outOfOrderStudents) + ", but " + wasSorted(orderedInts), thisLineNumber - 2, left1)))
        assert(caught1.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught1.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left2 = Array(orderedInts)
        val caught2 = intercept[TestFailedException] {
          all(left2) should (not be sorted and not be outOfOrderStudents)
        }
        assert(caught2.message === Some(allInspectionFailed(0, wasSorted(orderedInts), thisLineNumber - 2, left2)))
        assert(caught2.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught2.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left3 = Array(orderedStudents)
        val caught3 = intercept[TestFailedException] {
          all(left3) should (not equal outOfOrderInts and not be sorted)
        }
        assert(caught3.message === Some(allInspectionFailed(0, didNotEqual(orderedStudents, outOfOrderInts) + ", but " + wasSorted(orderedStudents), thisLineNumber - 2, left3)))
        assert(caught3.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught3.failedCodeLineNumber === Some(thisLineNumber - 4))
        
        val left4 = Array(orderedStudents)
        val caught4 = intercept[TestFailedException] {
          all(left4) should (not be sorted and not equal outOfOrderInts)
        }
        assert(caught4.message === Some(allInspectionFailed(0, wasSorted(orderedStudents), thisLineNumber - 2, left4)))
        assert(caught4.failedCodeFileName === Some("ShouldBeSortedLogicalAndForArraySpec.scala"))
        assert(caught4.failedCodeLineNumber === Some(thisLineNumber - 4))
      }
      
      def `should use implicit Sortable when available` {
        intercept[TestFailedException] {
          all(Array(orderedInts)) should (not be (sorted) and not be (outOfOrderInts))
        }
        implicit val imp = falseSortable
        all(Array(orderedInts)) should (not be (sorted) and not be (outOfOrderInts))
      }
      
      def `should use explicitly specified Sortable` {
        intercept[TestFailedException] {
          all(Array(orderedInts)) should (not be (sorted) and not equal (outOfOrderInts))
        }
        (all(Array(orderedInts)) should (not be (sorted) and not equal (outOfOrderInts))) (falseSortable, defaultEquality)
      }
    }
    
    // shouldBe and shouldNot does not support logical expression, and they never will!
  }
}
